/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content
 *
 * @author Hartmut Schorrig, Pinzberg, Germany
 *
 * @version 0.83
 * @content: This file contains all methods from CRuntimeJavalike which have access to the operation system level.
 * Only the routines here uses the os_wrapper.h and are dependent on it. All other routines are os-independent.
 * The os_wrapper.h defines the interface to the operation system in a os-independently way.
 * But the os_wrapper.h may be adapted to os-requirements in the future. So only this source is involved.
 *
 * @author Jchartmut www.vishia.org
 * @version 0.82
 * list of changes:
 * 2008-04-22: JcHartmut adap: changes from objectBaseC.h
 * 2008-03-15: JcHartmut creation
 *
 ****************************************************************************/


#include "ObjectJc.h"
#include "Fwc/fw_Exception.h"
#include "OSAL/inc/os_thread.h"
#include "OSAL/inc/os_time.h"
#include "OSAL/inc/os_mutex.h"
#include "OSAL/inc/os_waitnotify.h"
#include "Fwc/fw_ThreadContext.h"

/**Because only a int16 value is used in Object_Jc for wait and threading, but some operation systems uses 32 bit-handles,
 * a translation of handle to index is necessary. This type stores several types of OS_Handle.
 */
typedef struct HandleItem_t
{ /**union of several handles, all are pointers. */
  union
  { struct OS_HandleWaitNotify_t const* wait;
    struct OS_HandleThread_t const* thread;
    /**If it is a free handle, the pointer to the next free. */
    struct HandleItem_t* nextFree;
  }handle;


  //struct OS_HandleMutex_t const* handleMutex;
  uint handleMutex;
  /**a next index if a queueu is necessarry. TODO using?*/
  int16 idxNext;

  /**A dummy to respect 8-byte alignment. */
  int16 reserve;
} HandleItem;



/**A static array of handles to assing int16-indices to handles.
 * It is upgradable later to a intelligent management.
 * TODO: Blocks of about 32..128 HandleItem allocated, a block managment. It will be simple.
 */
HandleItem handleItemsJc[128];  // 1 kByte

/**This handle addresses the next free handles, all free handles are queued. */
HandleItem freeHandle = {null};

/**Nr of handles, initial 0, th initFreeHandleEntry() routine should be called! */
int nrofHandle =0;


/**The value to designate a not free item. */
#define kNoFreeHandleItem ((HandleItem*)(-1))

/**This routine is called only onetime after startup. */
int initFreeHandleEntry()
{ int zHandleItems = ARRAYLEN(handleItemsJc);
  int ii;
  handleItemsJc[0].handle.nextFree = kNoFreeHandleItem;  //let the first unused, no using of index 0!
  freeHandle.handle.nextFree = &handleItemsJc[1];
  for(ii = 1; ii < zHandleItems -1; ii++)
  { handleItemsJc[ii].handle.nextFree = &(handleItemsJc[ii+1]);
  }
  handleItemsJc[ii].handle.nextFree = kNoFreeHandleItem;  //the last one.
  return zHandleItems;
}






/**gets a free handle.
 * @param idx the returned index.
 */
HandleItem* getFreeHandleEntry(int16* idx)
{ //this operation have to be executed under mutex:
  HandleItem* theHandleItem;
  if(nrofHandle == 0)
  { nrofHandle = initFreeHandleEntry();
  }
  theHandleItem = freeHandle.handle.nextFree;
  if(theHandleItem == kNoFreeHandleItem)
  { *idx = -1;  //TODO passenden Win-error
  }
  else
  { HandleItem* nextFree = theHandleItem->handle.nextFree;  //it may be null if there is no more handle.
    freeHandle.handle.nextFree = nextFree;
    //here mutex release!
    { int idxHandle = theHandleItem - handleItemsJc;
      if(idxHandle < 0 || idxHandle >= ARRAYLEN(handleItemsJc))
      { STACKTRC_ENTRY("getFreeHandleEntry");
        THROW_s0(RuntimeException, "corrupt handles",idxHandle);
      }
      *idx = (int16)(idxHandle);
    }
  }
  return theHandleItem;
}


/**gets the handle to the index.
 */
HandleItem* getHandleEntry(int idx)
{ if(idx < 0 || idx > ARRAYLEN(handleItemsJc))
  { STACKTRC_ENTRY("getHandleEntry");
    THROW_s0(RuntimeException, "error fault idx for handle", idx);
  }
  return &handleItemsJc[idx];
}


/**************************************************************************************************/
/* Implementations of ObjectJc.h
*/


/**implements from ObjectJc.h. */
void wait_ObjectJc(ObjectJc* obj, int milliseconds)
{ //TODO test wether the thread has the monitor.
  HandleItem* handle;
  if(obj->idSyncHandles == kNoSyncHandles_ObjectJc)
  { handle = getFreeHandleEntry(&obj->idSyncHandles);
    { int error = os_createWaitNotifyObject(&handle->handle.wait);
      if(error != 0)
      { //it may be throwable
        STACKTRC_ENTRY("wait_ObjectJc");
        THROW_s0(RuntimeException, "error os_createWaitNotifyObject", error);
      }
    }
  }
  else
  { handle = getHandleEntry(obj->idSyncHandles);
  }
  os_wait(handle->handle.wait, milliseconds);
}




void notify_ObjectJc(ObjectJc* obj)
{ //TODO test wether the thread has the monitor.
  //TODO it is possible that more as one thread waits, implement a list of threads.
  if(obj->idSyncHandles != kNoSyncHandles_ObjectJc)
  { HandleItem* handleItem = getHandleEntry(obj->idSyncHandles);
    os_notify(handleItem->handle.wait);
  }
}



void notifyAll_ObjectJc(ObjectJc* obj)
{ //TODO test wether the thread has the monitor.
  //TODO it is possible that more as one thread waits, implement a list of threads.
  if(obj->idSyncHandles != kNoSyncHandles_ObjectJc)
  { struct OS_HandleWaitNotify_t const* handle;
    HandleItem* handleItem = getHandleEntry(obj->idSyncHandles);
    handle = handleItem->handle.wait;
    os_notify(handle);
  }
}











/**implements from ObjectJc.h. */
void synchronized(ObjectJc* obj)
{ //TODO test wether the thread has the monitor.
  HandleItem* handle;
  if(obj->idSyncHandles == kNoSyncHandles_ObjectJc)
  { handle = getFreeHandleEntry(&obj->idSyncHandles);
    { int error = os_createMutex("", &handle->handleMutex);
      if(error != 0)
      { //it may be throwable
        STACKTRC_ENTRY("synchronized_ObjectJc");
        THROW_s0(RuntimeException, "error os_createWaitNotifyObject", error);
      }
    }
  }
  else
  { handle = getHandleEntry(obj->idSyncHandles);
  }
  os_lockMutex(handle->handleMutex);
}








void synchronizedEnd(ObjectJc* obj)
{
  STACKTRC_ENTRY("synchronizedEnd");
  //if(obj == &allThreads_Rtos.object)
  {
  }
  //else
  {
    if(obj->idSyncHandles != kNoSyncHandles_ObjectJc)
    { HandleItem* handleItem = getHandleEntry(obj->idSyncHandles);
      os_unlockMutex(handleItem->handleMutex);
    }
  }
  STACKTRC_LEAVE;
}








void sleep_Thread_Jc(int32 millisecond)
{ STACKTRC_ENTRY("sleep_Thread_Jc");
  os_delayThread(millisecond);
  STACKTRC_LEAVE;
}
