/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @author JcHartmut: hartmut.schorrig@vishia.de
 * @version 2008-04-06  (year-month-day)
 * list of changes:
 * 2008-04-06 JcHartmut: some correction
 * 2008-03-15 JcHartmut: creation
 *
 ****************************************************************************/
package org.vishia.java2C;

import java.io.File;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.vishia.zbnf.ZbnfParseResultItem;



/**This class represents the content of a java class while Translation.
 * The instance is created and filled while executing the first pass of translation,
 * that is the detecting of the class elements and generating the C-header. 
 * It is used for the second pass, the generation of C-file.
 * <br>
 * Chars are used to identify the kind of a element or type:
 * <table>
 * <tr><td>C</td><td>a class type identifier</td></tr>
 * <tr><td>I</td><td>an interface type identifier</td></tr>
 * <tr><td>B</td><td>a basic scalar type</td></tr>
 * </table>
 */
class ClassData
{
  /**The full name, like produced in C. If it is an inner class, outer__inner. */
  public final String sClassNameC;
  
  /**The full name in java, dot to separate outer.inner class names. */
  public final String sClassNameJava;
  
  /**Short form of type-name for argument sensitive C-routine names. */
  public final String sArgSensitiveName;
  
  /**Char which identifies a value in a variable argument list. The following chars are used:
   * <ul>
   * <li><code>Z</code>: boolean
   * <li><code>B</code>: byte, uint8
   * <li><code>C</code>: char
   * <li><code>L</code>: Object
   * <li><code>D</code>: double
   * <li><code>F</code>: float
   * <li><code>I</code>: int
   * <li><code>J</code>: long
   * <li><code>S</code>: short
   * <li><code>z</code>: zero terminated string literal
   * <li><code>s</code>: StringJc
   * <li><code>b</code>: StringBufferJc
   * <li><code>x</code>: ExceptionJc
   * <li><code>t</code>: OS_TimeStamp
   * <li><code>.</code>: unknown
   * </ul>
   * The upper-case-chars are the same, which are used in Java language 
   * to identify Array types returned by <code>java.lang.Class.getName()</code>.
   * 
   */
  public final char cVaArgIdent;
  
  /**The file were the class is defined in. Without extension .java.
   * 
   */
  public final String sFileName;
  
  /**Set if <code>@java2c = staticInstance</code> is found in description of the class.
   * A class is a static instance if the class should not be created dynamically using garbage collection,
   * but only in the construction path of the application.
   * In such classes all final references to final classes (may not be staticInstance itself)
   * are simple pointers in the C Conversion. Otherwise enhanced references are to be used.
   */
  public final boolean isStaticInstanciated;
  
  /**Information String given in constructor.
   */
  private final String sInfo;

  /**Set if the class is final. */
  public final boolean isFinal;

  /**Set if instances should only used embedded. 
   * It is a property able to set to classes with less elements. */
  public final boolean bEmbedded;
  
  /**True if the class isn't known in Java2C context. It is a external used class.*/
  private final boolean bExternal;

  /**For second pass, the parse result. NOTE: Not final because it should be removed after running second path. */
  private ZbnfParseResultItem zbnfClass;

  /**For finalize method body, null if no finalize method is defined for java. */
  public ZbnfParseResultItem zbnfFinalizeMethod;
  
  
  private short bitArrayNeed;
  
  /**All known identifiers of this class. They are also fields of this class, 
   * types (= inner classes) of this class
   */
  public final LocalIdents classLevelIdents;
  
  /**Simple Data class to hold Strings for generation casting.
   * The class contains a final pre- and a post String. 
   * They will be disposed around the input expression to build a expression with cast.
   * The class is used in {@link ClassData#castable} and {@link ClassData#castableInitializer}. 
   * @author JcHartmut
   *
   */ 
  private static class CastInfo
  { public final String pre, post;
    
    /**Constructs the data. */
    CastInfo(String pre, String post){ this.pre = pre; this.post = post; }
  }
  
  /**Types from which a cast for initial values is possible, and its cast rule. 
   * This list is build using {@link #addCastInitializer(ClassData srcType, String cast)}
   * and used in {@link #testAndcast(ClassData srcType, String value, char intension)}.
   */
  private TreeMap<String, CastInfo> castableInitializer;
  
  /**Types from which a cast is possible, and its cast rule. 
   * This list is build using {@link #addCast(ClassData srcType, String cast)}
   * and used in {@link #testAndcast(ClassData srcType, String value, char intension)}.
   */
  private TreeMap<String, CastInfo> castable;
  
  
  /**Holds Informations about superclasses and interfaces.
   * <ul>
   * <li>Reference to the ClassData of the given level.
   * <li>Reference to InheritanceInfo of the superclass.
   * <li>List of References to InheritanceInfo of all Interfaces of this level.
   * <li>All Methods of this level, which are able to overload (virtual methods).
   * </ul>
   */
  static class InheritanceInfo
  { final ClassData classData;
    InheritanceInfo superClass;
    Map<String, InheritanceInfo> interfaces;
    List<String> methodTable;
    
    InheritanceInfo(ClassData classDataP, ClassData superClassP)
    { this.classData = classDataP; 
      if(superClassP != null)
      { final ClassData superSuperClass = superClassP.getSuperClassData();  //superclass of next level? 
        //recursively call.
        this.superClass = new InheritanceInfo(superClassP, superSuperClass);
        /*add methods: */
        //InheritanceInfo inheritance1 = superInheritanceInfo;
        if(superClassP.inheritanceInfo != null && superClassP.inheritanceInfo.methodTable != null )
        { //methodTable of given superclass
          this.superClass.methodTable = new LinkedList<String>();
          //copy all method names to my methodTable of superClass
          for(String name: superClassP.inheritanceInfo.methodTable)
          { this.superClass.methodTable.add(name); 
          }
          //TODO interfaces...
        }  
      }
    }
  }
  
  /**Tree of all method tables of interfaces and the superclasses. */
  InheritanceInfo inheritanceInfo;  
  
  /**unused yet, need? */
  static class MethodArg
  { ClassData type;
    
  }
  
  
  /**Describes a method of a class.
   * All methods are stored in the private attribute {@link #methods},
   * filled using {@link #addMethod(String sNameJava, String sCName, int modifier, LocalIdents.IdentInfos returnType, ClassData[] paramsType)}
   * and there derivatives.
   * 
   * @author JcHartmut
   *
   */
  public static class Method
  { 
    /**Name of the method used in Java. The directly short name. */
    public final String sJavaName;
    
    /**Search name of method in Map.*/
    public final String sKeyName;
    
    /**Name of the method used in C, with postfixes. */
    public final String sCName;
    
    /**All argument type of the method. */
    public final FieldData[] paramsType;
    
    /**Return type of the method. */
    public final FieldData returnType;
    
    /**The class which declares the method primary. It may be an interface or superclass
     * or the creating class itself, if the method are not found in deeper base.
     */
    public final ClassData declaringClass;
    
    /**The idents which are seen in the methods focus. 
     * This element remains with null, if the methode is only declared, no body is generated
     * (external classes without code generation). */
    private LocalIdents methodIdents;
    
    /**Set if _thCxt is need in method calls */
    public final boolean need_thCxt;
    
    /**The method head definition is set, if a method is parsed in first pass.
     * It is used in the second path.
     */
    private String sMethodHeadDefiniton;
    
    /**Modifier bit, when set the method has ThCxt as last argument. 
     * If not set, than it is a simple method without exceptions, 
     * so the thread context pointer may be overmuch. 
     */
    public final static int modeNoThCxt = 0x0100;
    
    /**Modifier bit, when set the method is static. */
    public final static int modeStatic = 0x0200;
    
    /**Modifier bit, when set the method is overloadable (virtual). */
    public final static int modeOverloadable = 0x0400;
    
    /**Some mode bits, ones of {@link #modeNoThCxt}, {@link #modeStatic}. */
    public final int mode;
    
    /**Set true if more as one methode with the same name is stored. */
    private boolean bUnambiguousness;

    /**If it is an overloadable method, its number in InheritanceInfo. Else -1. */
    private short idxMtbl= -1;
    
    /**Initializes a method to store in ClassData.
     * @param sKeyName Name to search
     * @param sNameJava Name of the method in Java
     * @param sCName Name of the method to translate in C. If more than 1 method with the same sNameJava is given,
     *               the sCName should be unambiguous. 
     * @param returnType The type of return. It isn't a Classdata, but a IdentInfo, 
     *                   because some additional properties like return by value or reference should be present.
     * @param params Array of all argument types.
     * @param need_thCxt True than the thread context pointer should be the last argument.
     *                  If false, than it is a simple method without exceptions, 
     *                  so the thread context pointer may be overmuch. 
     */
    public Method(ClassData declaringClass, String sKeyName, String sNameJava, String sCName, int modifier, FieldData returnType
        , FieldData[] params)
    { this.declaringClass = declaringClass;
      this.sKeyName = sKeyName; this.sJavaName = sNameJava; 
      //if()
      this.sCName = sCName; 
      this.paramsType = params; this.returnType = returnType;
      assert( (modifier & ~(modeNoThCxt|modeStatic|modeOverloadable)) == 0);
      this.mode = modifier;
      this.need_thCxt = (modifier & modeNoThCxt) == 0;
      bUnambiguousness = true;
    }
    /**Arguments of the method. */
    //private List<MethodArg> args;

    /**Sets the definiton of a method (C-code). This method could only call one time. */
    public void setMethodHeadDefiniton(String def)
    { assert(sMethodHeadDefiniton == null); //set only 1 time.
      sMethodHeadDefiniton = def;
    }
    
    /**Gets the definition of a method (C-code), used in second pass. */
    public String getMethodHeadDefiniton(){ return sMethodHeadDefiniton; }
    
    /**Sets the method ambiguous. This method is called, if a method with the same java name 
     * and the same number of arguments, but other argument types are found in translation process.
     */
    public void setAmbiguousness(){ bUnambiguousness = false; }
    
    
    /**Returns true, if more as one method with the same java name and the same number of arguments exists.
     */
    public boolean isAmbigous(){ return !bUnambiguousness; }

    
    /**Writes the structure information of the method for file.stc. The structure information is parsed
     * to know the class with its method without processing the first pass of translation,
     * if the file.java is older as existing file.h and file.c
     * @return String with the informations for the file.stc.
     */
    public String writeStruct()
    { String out;
      //out = bUnambiguousness ? "!" : "~";
      out = sJavaName + "; " + sCName + "; ";
      out += returnType == null ? " return void_" : "return " + returnType.writeStruct();
      if(paramsType != null)
      { for(FieldData param: paramsType)
        { out += "param" + param.writeStruct();
        }
      }
      return out;
    }
    
    /**Returns -1 if it is a final method, or 0..., the index in the InheritanceInfo. */
    public int getIdxMtbl(){ return idxMtbl; }
    
    
    /**Helpfull for debugging in eclipse.
     * @see java.lang.Object#toString()
     */
    public String toString()
    { return sCName;
    }

    public final LocalIdents getMethodIdents(){ return methodIdents;}

    public final void setMethodIdents(LocalIdents methodIdents)
    { assert(this.methodIdents == null);  //call only one time!
      this.methodIdents = methodIdents;
    }
    
  }
  
  /**Index of all methods with the same number of arguments. 
   * The key is the java method name, following "#nr", where nr is the number of arguments.
   * The value is either a {@link Method} or a List<Methode> if more as one method 
   * with the same number of argumentes exists.  
   */
  private TreeMap<String, Object> methods;
  
  /**Index of all methods with its C name.  
   */
  private TreeMap<String, Method> methodsCname;
  
  
  /**Index of all methods which are not final. They are overloadable by a derivated class. 
   * The key is the C-method name.
   */
  private TreeMap<String, Object> methodsOverloadable;
  
  
  class MethodWithZbnfItem
  { final Method method;
    final ZbnfParseResultItem zbnfMethod;
    private final char isConstructor;
    
    public MethodWithZbnfItem(Method method, ZbnfParseResultItem zbnfMethod, char isConstructor)
    { this.method = method;
      this.zbnfMethod = zbnfMethod;
      this.isConstructor = isConstructor;
      if(isConstructor == 'c'){ nrofConstructor +=1; } 
    }

    public final boolean isConstructor()
    { return isConstructor == 'c';
    }
    
  }
  
  final List<MethodWithZbnfItem> listMethodWithZbnf = new LinkedList<MethodWithZbnfItem>(); 
  
  int nrofConstructor = 0;
  
  /**If <code>this</code> or a unnamed instance of the class is used as a reference, 
   * this field supplied the necessary instance infos.
   * It represents the instance as simple forward reference, like <code>this</code>
   * respectively <code>ythis</code> in C.
   * Unnamed instances are especially returned instances of a method call.
   * In opposite: a field identifier supplies it IdentInfos using {@link LocalIdents#get(String)}. 
   */
  public final FieldData classTypeInfo;
  
  /**This codeinfo is used if a classlevel ident is accessed.*/
  final CCodeData thisCodeInfo;
  
  /**This codeinfo is used if a classlevel ident is accessed.*/
  final CCodeData typeCodeInfo;
  
  /**The superclass of this or null. */
  //final ClassData superClazz;
  
  
  /**The outer class of this or null. */
  final ClassData outerClazz;
  
  /**This class saves the position of itemValue in a parse result to the named variable.
   * It is used if the initialization is generated.
   */
  static class VariablesToInit
  { /**The zbnf parse result element which contains the value. */
    final ZbnfParseResultItem itemValue;
    
    final FieldData identInfos;
    
    public VariablesToInit(FieldData identInfo, ZbnfParseResultItem zbnfValue)
    { this.itemValue = zbnfValue; this.identInfos = identInfo;
    }
    
  }

  /**List of parse result items of initial values for variables. 
   * Filled in first path, used in second pass.
   */
  final List<VariablesToInit> variablesToInit;

  /**List of all static variables either with initial values or not.
   * Filled in first path, used in second pass.
   */
  final List<VariablesToInit> staticVariables;

  /**Initializes new translated classes.
   * @param sFileName
   * @param sNameJava
   * @param sNameCP
   * @param outerClassData
   * @param superClassData
   * @param itemClass
   * @param bIsStaticInstance
   * @param bFinal
   * @param identsParent
   */
  ClassData(String sFileName, String sNameJava, String sNameCP
  , ClassData outerClassData, ClassData superClassData    
  , ZbnfParseResultItem itemClass, boolean bIsStaticInstance, boolean bFinal
  , LocalIdents identsParent    
  )
  { this.sFileName = sFileName;
    this.sClassNameJava = sNameJava;
    if(sNameJava.equals("Entry"))
      stop();
    if(superClassData != null)
    { //the superclass has overloadable methods:
      inheritanceInfo = new InheritanceInfo(this, superClassData);
    }
    this.outerClazz = outerClassData;
    this.sArgSensitiveName = "XX";
    this.cVaArgIdent = 'L';
    if(sNameCP.equals("ReadTargetFromText__Target"))
      stop();
    this.sClassNameC = sNameCP; 
    this.zbnfClass= itemClass;
    String sInfo = "+";
    if(bIsStaticInstance){ sInfo += "static+"; }
    if(bFinal){ sInfo += "final+"; }
    this.sInfo = sInfo;
    this.isStaticInstanciated = bIsStaticInstance;
    this.isFinal = bFinal;
    this.classLevelIdents = new LocalIdents(identsParent, this);
    this.bExternal = false;
    this.bEmbedded = false;
    //this.dimensionArray = 0;
    //this.parentArrayType = null;
    variablesToInit = new LinkedList<VariablesToInit>();
    staticVariables = new LinkedList<VariablesToInit>();
    classTypeInfo = new FieldData(sNameCP, this, '.', 'C', 0, null, '.', this );
    //CCodeData for class member reference: use ythis as simple pointer.
    this.thisCodeInfo = new CCodeData("ythis", classTypeInfo, '*', 0);
    this.typeCodeInfo = new CCodeData("", classTypeInfo, 'C', 0);
  }

  
  
  /**Initializes known instances.
   * @param sNameC name of the type in C.
   * @param sArgSensitiveName short identification, typically one char, used in methods to indicate the type of argument
   * @param sFileName Name of the file to include, if this type is used.
   * @param infos String with some options:
   *        <table>
   *        <tr><td>static</td><td>static instance</td>
   *        </table>
   * @param sNameC
   * @param sArgSensitiveName
   * @param sFileName
   * @param infos
   */
  ClassData(String sNameJava, String sNameC, String sArgSensitiveName, char cTypeName, String sFileName, String infos) //, LocalIdents identsParent)
  { this.sClassNameC = sNameC; 
    this.sClassNameJava = sNameJava;
    this.sArgSensitiveName = sArgSensitiveName;
    this.cVaArgIdent = cTypeName;
    this.sFileName = sFileName;
    this.sInfo = infos;
    this.zbnfClass= null;
    this.bExternal = infos.contains("+extern+");
    this.bEmbedded = infos.contains("+embedded+");
    if(infos.contains("+ObjectJc+"))
    { inheritanceInfo = new InheritanceInfo(this, Java2C_Main.singleton.standardClassData.clazzObjectJc);
    }
    this.outerClazz = null;
    //this.dimensionArray = 0;
    //this.parentArrayType = null;
    this.isStaticInstanciated = infos.contains("+static+");
    this.isFinal = infos.contains("+final+");
    this.classLevelIdents = infos.contains("+primitive+")? null : new LocalIdents(null,this); //identsParent);
    this.variablesToInit = bExternal ? null : new LinkedList<VariablesToInit>();
    this.staticVariables = bExternal ? null : new LinkedList<VariablesToInit>();
    this.classTypeInfo = new FieldData(sNameC, this, '.', 'C', 0, null, '.', this );
    this.thisCodeInfo = new CCodeData("ythis->", classTypeInfo, '*', 0);
    this.typeCodeInfo = new CCodeData("", classTypeInfo, 'C', 0);
  }


  
  
  /**initializes a instance as next array instance. Only called private.
   * @param parent The non-array instance or previous instance (with less dimension)
   */
  private ClassData(ClassData parent)
  { //this.parentArrayType = parent;
    assert(false);
    this.sClassNameC = parent.sClassNameC; // + ( parent.dimensionArray > 0 
                               // ? (char)((int)('0') + parent.dimensionArray) 
                               // : "_AY" );
    this.sClassNameJava = parent.sClassNameJava;
    this.sArgSensitiveName = parent.sArgSensitiveName;
    this.cVaArgIdent = parent.cVaArgIdent;
    //this.superClazz = parent.superClazz;
    this.outerClazz = parent.outerClazz;
    this.sFileName = parent.sFileName;
    this.sInfo = parent.sInfo;
    this.zbnfClass = parent.zbnfClass;
    this.bExternal = parent.bExternal;
    this.bEmbedded = parent.bEmbedded;
    //this.dimensionArray = parent.dimensionArray +1;   //next dimension.
    this.isStaticInstanciated = parent.isStaticInstanciated;
    this.isFinal = parent.isFinal;
    this.classLevelIdents = parent.classLevelIdents;
    this.variablesToInit = null;
    this.staticVariables = null;
    this.classTypeInfo = new FieldData(parent.sClassNameC, this, '.', 'C', 0, null, '.', this );;
    this.thisCodeInfo = new CCodeData("ythis->", classTypeInfo, '*', 0);
    this.typeCodeInfo = new CCodeData("", classTypeInfo, 'C', 0);
  }
  
  
  
  /**Returns true if the ClassData represents an Array type. */
  //public boolean isArray(){ return dimensionArrayOrFixSize >0; }
  
  void      need_Y() {         bitArrayNeed |= 0x01; }
  boolean isneed_Y() { return (bitArrayNeed  & 0x01) !=0; }
  void      need_YP(){         bitArrayNeed |= 0x02; }
  boolean isneed_YP(){ return (bitArrayNeed  & 0x02) !=0; }
  
  
  
  public void addField(String sIdent, FieldData identInfo)
  {
    classLevelIdents.putClassElement(sIdent, identInfo);
  }
  
  
  /**adds a casting possibility for initializer.
   * @param srcType The src type for which the cast is valid.
   * @param cast String in form "pre?post". To implement the cast the pre and post part will wrap the value.
   */
  public void addCastInitializer(ClassData srcType, String cast)
  { int posSep = cast.indexOf('?');
    String sPre = cast.substring(0, posSep);
    String sPost = cast.substring(posSep+1);
    CastInfo castInfo = new CastInfo(sPre, sPost);
    if(castableInitializer == null){ castableInitializer = new TreeMap<String, CastInfo>(); }
    castableInitializer.put(srcType.sClassNameC, castInfo);
    
  }
  
  
  
  /**adds a casting possibility.
   * @param srcType The src type for which the cast is valid.
   * @param cast String in form "pre?post". To implement the cast the pre and post part will wrap the value.
   */
  public void addCast(ClassData srcType, String cast)
  { int posSep = cast.indexOf('?');
    String sPre = cast.substring(0, posSep);
    String sPost = cast.substring(posSep+1);
    CastInfo castInfo = new CastInfo(sPre, sPost);
    if(castable == null){ castable = new TreeMap<String, CastInfo>(); }
    castable.put(srcType.sClassNameC, castInfo);
    
  }
  
  
  /**tests wether a cast is necessary, if it, returns the casted value.
   * This is checked to srcType. If srcType is the same as this, no cast occurs, the value is returned direct.
   * If srcType is a derivateable type of this, the cast is set with {@link #addCast(ClassData, String)}.
   * That is used to produce the casted return value. 
   * @param srcType The type of the value
   * @param value The value string.
   * @param intension 'i' init-value may be in {...}
   * @return The value itself if no cast is necessary
   */
  public String testAndcast(ClassData srcType, String value, char intension)
  { String sRet;
    CastInfo castInfo;
    if(  srcType == this 
      || this == Java2C_Main.singleton.standardClassData.clazz_void  //void as argument is a void*
      || this == Java2C_Main.singleton.standardClassData.clazz_va_argRaw  //any type is valid
      ) 
    { //no cast necessary
      sRet = value; 
    }
    else if( value.equals("null"))
    { //a null-pointer should be compatible with all expect StringJc.
      if(sClassNameC.equals("StringJc")){ 
        sRet = "null_StringJc"; }
      else {
        sRet = value; } 
    }
    else
    { //normal value
      if(  intension == 'i' && castableInitializer != null
        && (castInfo = castableInitializer.get(srcType.sClassNameC)) != null
        )
      { sRet = castInfo.pre + value + castInfo.post;
      }
      else if( castable != null
        && (castInfo = castable.get(srcType.sClassNameC)) != null
        )
      { sRet = castInfo.pre + value + castInfo.post;
      }
      else
      { sRet = "((/*cast*/" + sClassNameC + (isPrimitiveType()? "" : "*") + ")(" + value + "))";
      }
    }
    return sRet;    
  }
  
  
  /**Registers a Method if the class is parsed or a standard class is initialized.
   * Inside {@link #addMethod(String, String, int, org.vishia.java2C.FieldData, org.vishia.java2C.FieldData[])}
   * is called with {@link ClassData#classTypeInfo}. 
   * @param sNameJava Name of the method in Java
   * @param sCName Name of the method to translate in C. If more than 1 method with the same sNameJava is given,
   *               the sCName should be unambiguous.
   * @param modifier Ones of {@link ClassData.Method#modeStatic}, {@link ClassData.Method#modeNoThCxt}               
   * @param returnType The type of return. It isn't a Classdata, but a IdentInfo, 
   *                   because some additional properties like return by value or reference should be present.
   * @param paramsType Array of all argument types. They are given in form of pure types, 
   *                   because they are taken with standard conventions: simple pointers or simple primitive types.
   */
  public void addMethod(String sNameJava, String sCName, int modifier, FieldData returnType, ClassData[] paramsType)
  { FieldData[] paramsIdent = new FieldData[paramsType.length];
    if(sCName.equals("ctorO_Target"))
      stop();
    for(int ii=0; ii<paramsType.length; ii++)
    { paramsIdent[ii] = paramsType[ii].classTypeInfo;
    }
    addMethod(sNameJava, sCName, modifier, returnType,paramsIdent);
  }

  /**Adds. @deprecated, use {@link #addMethod(String, String, int, FieldData, ClassData[])}
   * 
   * @param sNameJava
   * @param sCName
   * @param returnType
   * @param paramsType
   */
  public void addMethod(String sNameJava, String sCName, FieldData returnType, ClassData[] paramsType)
  { final int modifier;
    if(sCName.startsWith("&"))              //special designation:
    { sCName = sCName.substring(1);
      modifier = Method.modeNoThCxt;
    }
    else
    { modifier = 0 & Method.modeNoThCxt;
    }
    addMethod(sNameJava, sCName, modifier, returnType, paramsType);
  }

  /**Registers a argumentless method if the class is parsed or a standard class is initialized.
   * @param sNameJava
   * @param sCName
   * @param modifier
   * @param returnType
   */
  public void addMethod(String sNameJava, String sCName, int modifier, FieldData returnType)
  { FieldData[] paramsIdent = null;
    addMethod(sNameJava, sCName, modifier, returnType, paramsIdent);
  }

  public void addMethod(String sNameJava, String sCName, int modifier, FieldData returnType, FieldData[] paramsType)
  {
    addMethod(this, sNameJava, sCName, modifier, returnType, paramsType);

  }
  
  /**Registers a Method if the class is parsed or a standard class is initialized.
   * If a method with the same number of arguments if registered already, a List<Method> is build with the same key. 
   * 
   * @param sNameJava Name of the method in Java
   * @param sCName Name of the method to translate in C. If more than 1 method with the same sNameJava is given,
   *               the sCName should be unambiguous. 
   * @param modifier Ones of {@link ClassData.Method#modeStatic}, {@link ClassData.Method#modeNoThCxt}               
   * @param returnType The type of return. It isn't a Classdata, but a IdentInfo, 
   *                   because some additional properties like return by value or reference should be present.
   * @param paramsType Array of all argument types with there type modifier.
   */
  @SuppressWarnings("unchecked")
  public Method addMethod(ClassData declaringClass, String sNameJava, String sCName, int modifier, FieldData returnType, FieldData[] paramsType)
  { if(sCName.equals("ctorO_Target"))
      stop();
    if(methods == null) { methods = new TreeMap<String, Object>(); }
    if(methodsCname == null) { methodsCname = new TreeMap<String, Method>(); }
    String sKeyName = sNameJava +"#"+ (paramsType == null ? "0" : paramsType.length);
    final boolean bWithThreadContext;
    Method method = new Method(declaringClass, sKeyName, sNameJava, sCName, modifier, returnType, paramsType);
      //sNameJava; //the name put in Map as key
    //String sKeyNameNull = sNameJava; //the name put in Map as key
    /*
    if(params != null)for(ClassData paramType: params)
    { sKeyName += "_" + paramType.getName();
    }
    */
    Object oMethods = methods.get(sKeyName);
    if(oMethods != null)
    { if(oMethods instanceof Method)
      { List<Method> list = new LinkedList<Method>();
        list.add((Method)oMethods);  //the first
        list.add(method);  //the second
        methods.put(sKeyName, list);  //now it is a list.
      }
      else
      { assert(oMethods instanceof List);
        List<Method> list = (List<Method>)oMethods; //it is already a list
        list.add(method);
      }
    }
    else
    { //methods with the given number of paramter is unknown, now one method.
      methods.put(sKeyName, method);
    }
    methodsCname.put(method.sCName, method);
    if( (modifier & Method.modeOverloadable) != 0)
    {
      if(methodsOverloadable == null) { methodsOverloadable = new TreeMap<String, Object>(); }
      oMethods = methodsOverloadable.get(sKeyName);
      if(oMethods != null)
      { if(oMethods instanceof Method)
        { List<Method> list = new LinkedList<Method>();
          list.add((Method)oMethods);  //the first
          list.add(method);  //the second
          methodsOverloadable.put(sKeyName, list);  //now it is a list.
        }
        else
        { assert(oMethods instanceof List);
          List<Method> list = (List<Method>)oMethods; //it is already a list
          list.add(method);
        }
      }
      else
      { //methods with the given number of paramter is unknown, now one method.
        methodsOverloadable.put(sKeyName, method);
      }
      //save the name in the method table.
      if(inheritanceInfo == null){ inheritanceInfo = new InheritanceInfo(this, null);}
      if(inheritanceInfo.methodTable == null){ inheritanceInfo.methodTable = new LinkedList<String>();}
      method.idxMtbl = (short)inheritanceInfo.methodTable.size();  //save its index.
      inheritanceInfo.methodTable.add(sCName);
      
    }  
    return method;
  }
  

  /**Registers a given Method. This routine is used if a file.stc is parsed.
   * @param m The Method data.
   */
  public void addMethod(Method m)
  { String sKeyName = m.sKeyName;
    final Object methodExist;
    if(methods == null)
    { methods = new TreeMap<String, Object>(); 
      methodExist = null;
    }
    else
    { methodExist = methods.get(sKeyName);
    }
    if(methodExist == null)
    { methods.put(sKeyName, m);  //the first one
    }
    else if(methodExist instanceof Method)  //the second
    { List<Method> listMethods = new LinkedList<Method>();
      listMethods.add((Method)methodExist);
      listMethods.add(m);
      methods.put(sKeyName, listMethods); //exchange it.
    }
    else  //it is a list!
    { List<Method> listMethods = (List<Method>)methodExist;
      listMethods.add(m);  //more as 2 methods with same nr of params
    }
  }
  
  
  
  /*
  public ClassData nextArrayDimension()
  { if(arrayType == null)
    { //An arraytype doesn't exist, create it because it is required. 
      arrayType = new ClassData(this ); //special private constructor for next array dimension.
      //NOTE: special constructor is necessary because some final fields.
    }
    return arrayType;
  }
  */
  
  
  /**Tests whether a method is registered, registers it or set it to ambiguous if it exists already.
   * This routine is called processing the first pass of translation. 
   * It creates an entry with key sNameJava + "?" to check ambiguously.
   * In the second pass the methods are registered with full informations.
   * @param sNameJava The method name.
   */
  public void testAndSetAmbiguousnessOfMethod(String sNameJava)
  { if(methods == null) { methods = new TreeMap<String, Object>(); }
    Method m = (Method)methods.get(sNameJava+"?");
    if(m != null)
    { //The method with the same java name is present already:
      m.setAmbiguousness();
    }
    else
    { m = new Method(this, sNameJava+"?", sNameJava, null, 0, null, null);  //it is a preliminary instance.
      methods.put(sNameJava+"?", m);
    }
  }

  
  /**tests whether the method is ambiguous. The method have to be exist.
   * @param sNameJava
   * @return true if more as one method with the same Java name exists. 
   */
  public boolean isAmbiguousnessMethod(String sNameJava)
  { Method m = (Method)methods.get(sNameJava+"?");
    if(m == null)
    { stop();
      return false;
    }
    return m.isAmbigous();
  }

  
  /**Searches a method with the given Java name and all argument types.
   * @param sNameJavaP The name in Java.
   * @param paramsType All argument types. It is tested whether the argument types 
   *                   are able to cast to the matching types.
   * 
   * @return The appropriate method or null if no method matches.
   */
  public Method searchOverloadedMethod(String sNameJavaP, FieldData[] paramsType)
  { boolean bFound = false;
    //String sNameTranslate = null;
    Method methodFound = null;
    String sKeyName;
    if(methodsOverloadable != null)
    { //do
      { sKeyName = sNameJavaP +"#"+ (paramsType == null ? "0" : paramsType.length);
        if(sKeyName.equals("new#1"))
          stop();
        Object omethodFound = methods.get(sKeyName);
        Method methodTest;
        bFound = (omethodFound != null);
        if(omethodFound != null)
        { //sNameTranslate = methodFound.sCName;
          if(omethodFound instanceof Method)
          { methodTest = (Method)omethodFound;
            if(checkParameterTypesMethod(methodTest, paramsType))
            { methodFound = methodTest;
            }
            else
            { methodFound = null;  //because not the same paramter types.
            }
          }
          else
          { List<Method> listMethods = (List<Method>)omethodFound;
            Iterator<Method> iterMethods = listMethods.iterator();
            while(methodFound == null && iterMethods.hasNext())
            { methodTest = iterMethods.next();
              if(checkParameterTypesMethod(methodTest, paramsType))
              { methodFound = methodTest;
              }
            }
          }
        }
      } //while(!bFound);
    }
    else
    { sKeyName = "no methods";
    }
    return methodFound;
    //return sNameTranslate;
  }
  
  
  private boolean checkParameterTypesMethod(Method methodTest, FieldData[] paramsType)
  {
    boolean bMatching;
    int idxParam = 0;
    if(paramsType == null && methodTest.paramsType.length == 0)
    { 
      bMatching = true;
    }
    else
    { bMatching = true; //default, abort test if false.
      int idxParamsType = 0;
      while(bMatching && idxParamsType < paramsType.length)
      { FieldData infoActParam = paramsType[idxParamsType++];
        FieldData typeParamMethod = methodTest.paramsType[idxParam++];
        if(typeParamMethod.typeClazz != infoActParam.typeClazz)
          //TODO test array properties
        { bMatching = false;
        }
      }
    }
    return bMatching;
  }              
  
  
  Method getMethodPerCname(String sNameC){ return methodsCname.get(sNameC); }
  
  
  /**Searches a method with the given Java name and all argument types.
   * @param sNameJavaP The name in Java.
   * @param paramsType All argument types. It is tested whether the argument types 
   *                   are able to cast to the matching types.
   * 
   * @return The appropriate method or null if no method matches.
   */
  public Method searchMethod(String sNameJavaP, List<CCodeData> paramsType)
  { boolean bFound = false;
    //String sNameTranslate = null;
    Method methodFound = null;
    String sKeyName;
    if(sNameJavaP.equals("assert"))
    {
      return Java2C_Main.singleton.standardClassData.methodASSERT; 
    }
    else
    {
      if(sNameJavaP.equals("arraycopy") && sClassNameC.equals("SystemJc"))
        stop();
      if(methods != null)
      { do
        { sKeyName = sNameJavaP +"#"+ (paramsType == null ? "0" : paramsType.size());
          if(sKeyName.equals("new#1"))
            stop();
          Object omethodFound = methods.get(sKeyName);
          bFound = (omethodFound != null);
          if(omethodFound != null)
          { //sNameTranslate = methodFound.sCName;
            if(omethodFound instanceof Method)
            { methodFound = (Method)omethodFound; 
            }
            else
            { List<Method> listMethods = (List<Method>)omethodFound;
              boolean bMatching = false;
              Iterator<Method> iterMethods = listMethods.iterator();
              while(!bMatching && iterMethods.hasNext())
              { Method methodTest = iterMethods.next();
                int idxParam = 0;
                if(paramsType == null && methodTest.paramsType.length == 0)
                { 
                  bMatching = true;
                }
                else
                { bMatching = true; //default, abort test if false.
                  Iterator<CCodeData> iterParamAct = paramsType.iterator();
                  while(bMatching && iterParamAct.hasNext())
                  { CCodeData actParam = iterParamAct.next();
                    FieldData infoActParam = actParam.identInfo;
                    FieldData typeParamMethod = methodTest.paramsType[idxParam++];
                    if(!typeParamMethod.typeClazz.matchedTo(infoActParam.typeClazz))
                    //TODO test array properties
                    { bMatching = false;
                    }
                  }
                }  
                if(bMatching)
                { methodFound = methodTest;
                }
              }
            }
          }
          if(!bFound)
          { //variation of the paramsType
            //sNameTranslate = "/*?? " + sFullName + "*/"; 
            bFound = true; //TODO later.
          }
        } while(!bFound);
      }
      else
      { sKeyName = "no methods";
      }
      if(methodFound == null)
      { String sNameC1 = sNameJavaP + "_" + sClassNameC + "/*" + sKeyName + "?method*/";
        methodFound = new Method(this, sKeyName, sNameJavaP, sNameC1, 0, Java2C_Main.singleton.standardClassData.clazz_void.classTypeInfo, null);
      }
      return methodFound;
      //return sNameTranslate;
    }  
  }
  
  /**Returns the name of the class used as C-type. If it is an generated class, that type names
   * have an extension <code>_s</code>. The type name without this extension is useable, 
   * if a C++-wrapper-class of that type is desired. 
   */
  public String getClassCtype_s(){ return sClassNameC; }
  
  
  /**Returns the name of the class used in Java without package information.
   */
  public String getClassNameJava(){ return sClassNameJava; }
  
  
  /**Returns the indentification name of a class. If the class is generated from java2c, 
   * it is the name without the <code>_s</code> on end, used to sign static visible class elements.
   */
  public String getClassIdentName()
  { if(sClassNameC.endsWith("_s") || sClassNameC.endsWith("_i"))
    { return sClassNameC.substring(0, sClassNameC.length()-2); }
    else{ return sClassNameC; }
  }
  
  /**Returns true if it is a primitive type without any own elements. It is, if the element 
   * {@link #classLevelIdents} is null.
   */
  public boolean isPrimitiveType(){ return classLevelIdents == null; }


  /**Returns the superclass or null. */
  public ClassData getSuperClassData()
  {
    final ClassData superClass;  //superclass of next level? 
    if(inheritanceInfo != null)
    { InheritanceInfo superInheritanceInfo = inheritanceInfo.superClass;
      superClass = superInheritanceInfo == null ? null : superInheritanceInfo.classData;
    }
    else
    { superClass = null;
    }
    return superClass;        
  }
  
  
  /**returns a String, which describes the access to the ObjectJc-base, or return null,
   * if the class isn't based on any class. It assumes, all superclasses should have the name "super",
   * the ObjectJc-superclass should have the name "object".
   * At ex. returns "object"
   * 
   * @return
   */
  public String getRefObjectJc()
  { String ret = "";
    if(inheritanceInfo != null)
    { ClassData superClazz = getSuperClassData();
      while(superClazz != null)
      { if(superClazz == Java2C_Main.singleton.standardClassData.clazzObjectJc)
        { ret += ".object";
        }
        else
        { ret += ".super";
        }
        superClazz = superClazz.getSuperClassData();
      }
    }  
    return ret.length() ==0 ? null: ret.substring(1);  //without first "."
  }
  
  /**Returns true if the class isn't known in Java2C context. It is a external used class.*/
  public boolean isExternal(){ return bExternal; }
  
  boolean matchedTo(ClassData src)
  { String typeSrc = src.sClassNameC;
    return this == src 
        || castable != null && castable.get(typeSrc) != null  //the typeSrc is found in castable
        || this.sClassNameJava.equals("va_argRaw");           //the src is matching to a raw variable argument always.
  }

  ZbnfParseResultItem getParseResult(){ return zbnfClass; }

  /**relinquishes the parse result, because it is processed. It should not consumed memory.*/
  void relinquishParseResult(){ zbnfClass = null; }

  void writeStructure(File file) 
  throws IOException
  { BufferedWriter out = new BufferedWriter(new FileWriter(file));
    out.append("Java2C: structure of file-class: " + getClassIdentName() + "\n");
    writeStructureClass(out,0);
    out.close();
  }

  /**Helper variable to generate indentation, see {@link #indent(int)} */
  private final static String newLineIndent = "\n                                            ";
  
  /**Generates an indentation, used inside {@link #writeStructureClass(BufferedWriter, int)}
   * @param recursion nesting level
   * @return A newline char and the appropritate number of spaces for indentation. 
   */
  private String indent(int recursion){ return newLineIndent.substring(0, 2*recursion +1); } 
  
  
  /**Writes the structure data of the class in file.stc.
   * @param out destination
   * @param recursion nesting level
   * @throws IOException
   */
  public void writeStructureClass(final BufferedWriter out, final int recursion) 
  throws IOException
  { out.append("class " + sClassNameJava + "; "+ sClassNameC + ";");
    out.append(indent(recursion) + "{  ");
    out.append(classLevelIdents.writeStruct(recursion+1));
    if(methods != null)
    { out.append("methods");
      out.append(indent(recursion+1) + "{  ");
      Set<Map.Entry<String,Object>> listEntries = methods.entrySet();
      for(Map.Entry<String,Object> field: listEntries)
      { String sKey = field.getKey();
        Object methodEntry = field.getValue();
        if(methodEntry instanceof Method)
        { Method method2 = (Method)(methodEntry);
          if(method2.sCName != null)
          { out.append( "    method;" + sKey + ";" + method2.writeStruct() + "\n");
          }
        }
        else //it is a list, otherwise ClassCastException!
        { List<Method> listMethod1 = (List<Method>)methodEntry;
          for(Method method: listMethod1)
          { if(method.sCName != null)
            { out.append( "    method;" + sKey + ";" + method.writeStruct() + "\n");
            }
          }  
        }
      }
      out.append( "}\n" );
    }
    Set<Map.Entry<String, ClassData>> typeSet = classLevelIdents.getTypeSet();
    if(typeSet.size() >0)
    { out.append(indent(recursion) + "InnerClass" + indent(recursion) + "{ ");
      for(Map.Entry<String, ClassData> innerClassEntry: typeSet)
      { ClassData innerClass = innerClassEntry.getValue();
        innerClass.writeStructureClass(out, recursion+2);
      }
      out.append(indent(recursion) + "}");
    }  
    out.append("}\n");
  }
  
  
  //List<VariablesToInit> getVariablesToInit(){ return variablesToInit; }

  /**Helpfull for debugging in eclipse.
   * @see java.lang.Object#toString()
   */
  public String toString(){ return sClassNameC; }
  
  /**Helper method for debugging. */
  private void stop(){}
  
  
}
