package org.vishia.java2C;

/**Data class holds the data of one variable in a local context respectively a field of a class, 
 * see {@link LocalIdents} 
 * or accessed instance, see {@link CCodeData}.
 * <br>  
 * The {@link ClassData} represents the type of the instance. Primary that may be an inheritance,
 * because the type informations may be a part of the field informations, 
 * this class should than inherit {@link ClassData}. 
 * But there are less instances of {@link ClassData} 
 * for some more instances of this class. The data should be duplicated, if an inheritance is designed.
 * Therefore it is better there is an aggregation to the appropriate {@link ClassData}, 
 * see {@link #typeClazz}. 
 */
public class FieldData
{
  /**The name of the identifier. <code>null</code> if it is the description of an access to an instance. */
  private final String sName;
  
  /**The type of the instance. */
  public final ClassData typeClazz;
  
  /**The class where the identifier is member of, null if it is a local identifier. */
  public final ClassData clazz;
  
  /**Char to indicate the location and kind of definition.
   * <ul><li>'S' static constant
   *     <li>'s' static non constant
   *     <li>'d' immediate constant with define
   *     <li>'.' non static
   * </ul>     
   */
  public final char modeStatic;
  
  /**If the identifier is an array with fix size, programmed in java in form of
   * <pre>
   * Type[][] myArray = new Type[expr][expr];
   * </pre>
   * The <code>expr</code> translated in C-Form are stored here. This expr should be a simple value
   * in C. There are used to generate the initialization of an array.
   * <br>
   * <code>null</code> if it isn't an array or it is a dynamic array.
   */
  public final String[] fixArraySizes;
  
  /**0 if it isn't an array. 1.. if it is an array.*/
  public final int dimensionArrayOrFixSize;
  
  /**Kind of the access to an element, if it is an array.
   * The adequate chars are used see {@link #modeAccess}.
   */
  public final char modeArrayElement;
  
  /**Kind of the instance.
   * <table>
   * <tr><td>$</td><td>an embedded struct</td></tr>
   * <tr><td>%</td><td>a variable of basic primitive type such as int32, float, char,
   *                   but also an embedded indivisible struct like StringJc, MemC, OS_TimeStamp,
   *                   whose elements aren't accessed in Java. </td></tr>
   * <tr><td>*</td><td>a simple reference, pointer in C, used for non-GC-controlled associations.</td></tr>
   * <tr><td>@</td><td>an enhanced reference, for GC (garbage collector) -controlled associations.</td></tr>
   * <tr><td>t</td><td>a StringJc, it is an enhanced reference also, but with special accesses</td></tr>
   * <tr><td>&</td><td>an interface enhanced reference, using C-jump-table with index, GC is possible</td></tr>
   * </table>
   */
  public final char modeAccess;
  
  /**see {@link getClassLevel()}*/
  int nClassLevel;

  /**see {@link getOuterLevel()}. Note: this private member is set in the outer class, 
   * see {@link putClassElement(String, String, ClassData, String, ClassData)}
   * or  {@link putLocalElement(String, String, ClassData, String)} 
   */
  int nOuterLevel;
  
  /**initializes the instance node. */
  FieldData
  ( String sName
  , ClassData typeClazz
  , char staticMode
  , char accessMode
  , int dimensionArray
  , String[] fixArraySizes
  , char modeArrayElement
  , ClassData clazz
  )
  { this.sName = sName;
    this.modeAccess = accessMode;
    //this.sModifier = sModifier; 
    this.fixArraySizes = fixArraySizes; 
    this.clazz = clazz; this.typeClazz = typeClazz;
    this.dimensionArrayOrFixSize = dimensionArray; 
    if(modeArrayElement == '$')
      stop();
    this.modeArrayElement = modeArrayElement;
    this.modeStatic = staticMode;
  }
  
  /*
  FieldData(FieldData parent, int dimensionArrayOrFixSize)
  { //this.sTypeIdent = parent.sTypeIdent; 
    this.sName = parent.sName;
    this.modeAccess = parent.modeAccess;
    //this.sModifier = sModifier; 
    this.fixArraySizes = parent.fixArraySizes; 
    this.clazz = parent.clazz; 
    this.typeClazz = parent.typeClazz;
    this.dimensionArray = (byte)dimensionArrayOrFixSize; 
    this.modeArrayElement = parent.modeArrayElement;
    this.modeStatic = parent.modeStatic;
  }
  */
  
  /**level of the affiliation to a class in the inheritance level.
   * 0= it is a local visible variable. 1=element of this class, 2=element of super class etc.
   */
  public int getClassLevel(){ return nClassLevel; }

  /**level of the affiliation to a class in the outer class relation.
   * 0= it is a local visible variable. 1=element of this class, 2=element of outer class etc.
   */
  public int getOuterLevel(){ return nOuterLevel; }
  
  /**returns the name of the element. */
  public String getName(){ return sName; }

  /**returns the name of the type of the element. */
  public String getTypeName(){ return typeClazz.getClassCtype_s(); }
  
  public String getTypeChar()
  { return "" + typeClazz.cVaArgIdent;
  }
  
  /**Writes the structure of this instance to a <code>*.stc</code>-file. 
   * It is an class level identifier, see {@link ClassData#classLevelIdents}, 
   * or a return type or argument of a method,
   * because only for such FieldData this method is called. 
   * See {@link LocalIdents#writeStruct(int)} and {@link ClassData.Method#writeStruct()}
   * @return The content to write to <code>*.stc</code>-file. The form of string is the same
   * described in the syntax <code>Structures.zbnf</code>.
   */
  public String writeStruct() 
  { String out = "(" 
          + modeAccess 
          + modeArrayElement 
          + modeStatic
          + "; ";
    for(int ii=0; ii<dimensionArrayOrFixSize; ii++)
    { final String sFixArraySize = 
        fixArraySizes == null ? "??"
        : fixArraySizes[ii] == null ? "?"
        : fixArraySizes[ii];
      out += "[" + sFixArraySize + "]";
    }
    out += "; ";
    out += typeClazz.sClassNameJava + "; " + typeClazz.sClassNameC + "; " + sName + "; )";
    //if()
    return out;
  }    
          
  
  /**builds a String representation especially to test under eclipse. */
  public String toString()
  { String sRet = typeClazz.getClassIdentName() + ":" + modeAccess + "[" + modeArrayElement + "]" + nClassLevel + nOuterLevel
           + (clazz != null ? clazz.sClassNameC : " -stacklocal");
    return sRet;
  }


  /**tests wether a cast is necessary, if it, returns the casted value.
   * This is checked to srcType. If srcType is the same as this, no cast occurs, the value is returned direct.
   * If srcType is a derivateable type of this, the cast is set with {@link #addCast(ClassData, String)}.
   * That is used to produce the casted return value. 
   * @param srcType The type of the value
   * @param value The value string.
   * @return The value itself if no cast is necessary
   */
  public String testAndcast(ClassData srcType, String value)
  { String sRet;
    if( value.equals("null"))
    { //a null-pointer should be compatible with all expect StringJc.
      if(typeClazz.sClassNameC.equals("StringJc")
        && dimensionArrayOrFixSize == 0
        )
      { 
        sRet = "null_StringJc"; }
      else {
        sRet = value; } 
    }
    else
    { //TODO may be some array tests
      
      sRet = typeClazz.testAndcast(srcType, value, '.');
    }
    return sRet;    
  }
  
  /**It's a debug helper. The method is empty, but it is a mark to set a breakpoint. */
  void stop()
  { //debug
  }



  

}
