/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @author JcHartmut: hartmut.schorrig@vishia.de
 * @version 0.82
 * list of changes:
 * 2008-04-06 JcHartmut: some correction
 * 2008-03-15 JcHartmut: creation
 *
 ****************************************************************************/
package org.vishia.java2C;

import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.vishia.zbnf.ZbnfParseResultItem;

/**This class processes the first pass of translating Java to C of a detect class in a java file.
 * Parsing the content of the java file is done before, see {@link GenerateFile}.
 * In the first pass all declared elements of a class are captured and store in the {@link ClassData}.  
 * 
 * @author JcHartmut
 *
 */
public class FirstPass extends GenerateClass
{

  /**initializes the instances with the given associations. */ 
  //FirstPass(iWriteContent writeContent, TreeMap<String, ClassData> allClassData, RunRequiredFirstPass_ifc runRequiredFirstPass)
  FirstPass(iWriteContent writeContent, LocalIdents allClassData, RunRequiredFirstPass_ifc runRequiredFirstPass)
  { super(writeContent, allClassData, runRequiredFirstPass);
  }


  /**Generates the content of a class in the h-file with given ZBNF parse result item of the class.
   * This routine is called from {@link GenerateFile}.  
   * <ul>
   * <li> All internal classes and interfaces are detect, this method is called recursively with it.
   * <li>The class fields are written in a struct in the h-File. 
   * <li> The static fields are written in h-file as extern declaration.
   * <li> The constructors are written as forward declaration in h-file of routines ctor...
   * <li> The methods are written as forward declaration in h-file.
   * </ul> 
   * For fields, the initial values are detected and stored in {@link ClassData#variablesToInit}.
   * This action is done inside the called routine {@link GenerateClass#gen_variableDefinition(ZbnfParseResultItem, LocalIdents, List, char, ClassData[])}.
   * <br>
   * The method heads are generated calling {@link #write_methodDeclaration(ZbnfParseResultItem, String, LocalIdents)}. 
   * 
   * @param sFileName Name of the h- and c-file. It is stored in the created ClassData of the class
   *        to allow the generating of a include statement, if another class uses this class.
   * @param zbnfClass The zbnf parser result item of the class.
   * @param sOuterClassName "" or the name of the outer class. 
   *        The name of the outer class is setted as prefix of the name of the struct in C 
   * @param outerClassData The outer class data or null, to register the inner class in a outer.
   * @param itemsClassesForSecondPath List to add the detect ClassData. 
   *        The list is used as input for running the second path of classes. 
   *        See {@link GenerateFile.itemsClassesForSecondPath}.  
   * @return
   * @throws IOException
   * @throws ParseException
   */
  ClassData runFirstPass
  ( String sFileName
  , String sPrefix, String sPostfix    
  , ZbnfParseResultItem zbnfClass
  , String sOuterClassName
  , ClassData outerClassData
  , List<ClassData> itemsClassesForSecondPath
  , LocalIdents identsParent
  )
  throws IOException, ParseException
  {

    ZbnfParseResultItem itemClassDescription = zbnfClass.getChild("description");
    boolean bStaticInstance =  itemClassDescription != null
                            && itemClassDescription.getChild("staticInstance")!= null;
    boolean bNoObject =  itemClassDescription != null
                            && itemClassDescription.getChild("noObject")!= null;

    boolean bFinal = zbnfClass.getChild("final")!= null;
    ZbnfParseResultItem itemClassIdent = zbnfClass.getChild("classident");
    if(itemClassIdent == null)
      throw new ParseException("classident not detect", 0);
    final String sClassNameLocal = itemClassIdent.getParsedString();
    if(sClassNameLocal.equals("LogMessageFile"))
      stop();
    final String sClassNameC;
    final String sClassNameJava;
    final ClassData superClazz;
    final ZbnfParseResultItem zbnfSuperClass = zbnfClass.getChild("Superclass");
    if(zbnfSuperClass != null)
    { String sSuperClass = zbnfSuperClass.getParsedString();
      superClazz = runRequiredFirstPass.runRequestedFirstPass(sSuperClass);
    }
    else
    {
      superClazz = bNoObject ? null : Java2C_Main.singleton.standardClassData.clazzObjectJc;
    }
    if(outerClassData != null)
    { sClassNameC = outerClassData.getClassIdentName() + "__" + sClassNameLocal;
      //sClassNameJava = outerClassData.sClassNameJava + "." + sClassNameLocal;
      sClassNameJava = sClassNameLocal;  
      //NOTE: use sClassName for classData from outer class "__" + read from itemClassIdent 
      classData = new ClassData(sFileName, sClassNameJava, sClassNameC + "_s", outerClassData, superClazz, zbnfClass, bStaticInstance, bFinal, identsParent);
      outerClassData.classLevelIdents.putClassType(sClassNameLocal, classData);
    }
    else
    { //NOTE: use sClassName for classData only read from itemClassIdent 
      sClassNameC = sPrefix + sClassNameLocal + sPostfix;
      sClassNameJava = sClassNameLocal;
      classData = new ClassData(sFileName, sClassNameJava, sClassNameC + "_s", null, superClazz, zbnfClass, bStaticInstance, bFinal, identsParent);
      allClassData.putClassType(sClassNameLocal, classData);   //to find it later sorted by names.
    }
    //allClassData.put(sClassName, classData);   //to find it later sorted by names.
    itemsClassesForSecondPath.add(classData);  //to process in the second path sorted per file

    //setInitialTypeIdentifiers();

    { /**First process subclasses because there may be known before the outer class is translated. */
      List<ZbnfParseResultItem> listSubClasses = zbnfClass.listChildren("classDefinition");
      if(listSubClasses != null)
      { for(ZbnfParseResultItem itemSubClasses : listSubClasses)
        { FirstPass genClass = new FirstPass(writeContent, allClassData, runRequiredFirstPass);
          genClass.runFirstPass(sFileName, sPrefix, sPostfix, itemSubClasses, sClassNameLocal + "__", classData , itemsClassesForSecondPath, identsParent);
      } }
    }
    { List<ZbnfParseResultItem> listSubInterfaces = zbnfClass.listChildren("interfaceDefinition");
      if(listSubInterfaces != null)
      { for(ZbnfParseResultItem itemSubInterfaces : listSubInterfaces)
        { write_Interface(itemSubInterfaces, sClassNameLocal + "__", classData);
      } }
    }

    //inner classes will be written before, now the own class: 
    writeContent.writeClassH("\n\n/*@CLASS_C " + sClassNameC + " @@@@@@@@@@@@@@@@@@@@@@@@*/");
    writeContent.writeClassH("\n\ntypedef struct " + sClassNameC + "_t\n{ ");
    //if(!bNoObject)
    ClassData superClass = classData.getSuperClassData();
    if(superClass != null)
    { String sNameSuper = superClass.sClassNameC;
      if(sNameSuper.equals("ObjectJc"))
      { writeContent.writeClassH("ObjectJc object;");
      }
      else
      { writeContent.writeClassH(sNameSuper + " super;");
      }
    }
    else
      stop();
    if(sClassNameLocal == null) throw new ParseException("no className found", 0);

    //process the class Content, write all class variable in the struct
    List<ZbnfParseResultItem> listVariables = zbnfClass.listChildren("variableDefinition");
    if(listVariables != null)for(ZbnfParseResultItem itemVariable : listVariables)
    { if(itemVariable.getChild("static")== null)
      { //only non static members.
        { String sVariableName = itemVariable.getChild("name").getParsedString();
          if(sVariableName.equals("targets"))
            stop();
        }
        CCodeData sDef = gen_variableDefinition(itemVariable, classData.classLevelIdents, classData.variablesToInit, 'c');
        writeContent.writeClassH("\n  " + sDef.cCode);
      }  
    }
    
    writeContent.writeClassH("\n} " + sClassNameC + "_s;\n  \n");


    writeContent.writeClassH("\ntypedef struct " + sClassNameC + "_Y_t { ObjectJc object; int32 length; int16 sizeElement; int16 mode; " + sClassNameC + "_s data[50]; } *" + sClassNameC + "_Y;");
    
    writeContent.writeClassH("\n\nextern struct ClassJc_t const reflection_" + sClassNameC + "_s;\n  \n");

    
    if(listVariables != null)for(ZbnfParseResultItem itemVariable : listVariables)
    { if(itemVariable.getChild("static")!= null)
      { //only non static members.
        CCodeData sDef = gen_variableDefinition(itemVariable, classData.classLevelIdents, classData.variablesToInit, 's');
        writeContent.writeClassH("\n  " + sDef.cCode);
      }  
    }
    writeContent.writeClassH("\n\n");
    
    { List<ZbnfParseResultItem> listConstructors = zbnfClass.listChildren("constructorDefinition");
      if(listConstructors != null)
      { boolean bArgumentSensitive = (listConstructors.size() >1);
        for(ZbnfParseResultItem itemConstructor : listConstructors)
        { ClassData.Method method = gen_methodHeadAndRegisterMethod(itemConstructor, classData, "ctorO", classData.classTypeInfo, bArgumentSensitive);
          writeContent.writeClassH(method.getMethodHeadDefiniton() + ";\n\n");
          ClassData.MethodWithZbnfItem methodDef = classData.new MethodWithZbnfItem(method, itemConstructor, 'c'); 
          classData.listMethodWithZbnf.add(methodDef);
        }
      }
      else
      { //no constructor found, create a default constructor because all variable initializations should be done.
        ClassData.Method method = gen_methodHeadAndRegisterMethod(null, classData, "ctorO", classData.classTypeInfo, false);
        writeContent.writeClassH(method.getMethodHeadDefiniton() + ";\n\n");
        ClassData.MethodWithZbnfItem methodDef = classData.new MethodWithZbnfItem(method, null, 'c'); 
        classData.listMethodWithZbnf.add(methodDef);
      }
    }
    writeContent.writeClassH("\n\n" + "void finalize_" + sClassNameC + "(ObjectJc* yObj, ThCxt* _thCxt);\n\n");
    
    { List<ZbnfParseResultItem> listMethods = zbnfClass.listChildren("methodDefinition");
      if(listMethods != null)
      { 
        for(ZbnfParseResultItem itemMethod : listMethods)
        { String sName = itemMethod.getChild("name").getParsedString();
          //register the method with following "?" to test if it is ambiguous.
          classData.testAndSetAmbiguousnessOfMethod(sName);
        } 
        for(ZbnfParseResultItem itemMethod : listMethods)
        { write_methodDeclaration(itemMethod, sClassNameC, classData.classLevelIdents);
        } 
      }
    }
    return classData;
  }



  
  
  /**Generates the content of an interface in c- and h-file. */
  void write_Interface(ZbnfParseResultItem itemClass, String sOuterClassName, ClassData outerClassData)
  throws IOException, ParseException
  {
    //ZbnfParseResultItem resultItem = resultItemClass.nextSkipIntoComponent(resultItemClass);

    ZbnfParseResultItem itemClassIdent = itemClass.getChild("interfaceIdent");
    if(itemClassIdent == null) throw new ParseException("interfaceIdent not detect", 0);

    ZbnfParseResultItem itemDescription = itemClass.getChild("description");
    if(itemDescription != null)
    { write_Description(itemDescription);
    }


    String sClassName = sOuterClassName + itemClassIdent.getParsedString();
    writeContent.writeClassH("typedef struct " + sClassName + "_t  /*interface*/\n{");
    writeContent.writeClassH("ObjectJc object;");

    if(sClassName == null) throw new ParseException("no className found", 0);

    Iterator<ZbnfParseResultItem> iterMethod = itemClass.iterChildren("methodDefinition");
    if(iterMethod != null)
    { while(iterMethod.hasNext())
      { ZbnfParseResultItem itemMethod = iterMethod.next();
        write_methodDeclaration(itemMethod, sClassName, classData.classLevelIdents);
    } }

    writeContent.writeClassH("\n} " + sClassName + "_i;\n  \n");


    List<ZbnfParseResultItem> listSubClasses = itemClass.listChildren("classDefinition");
    if(listSubClasses != null)
    { for(ZbnfParseResultItem itemSubClasses : listSubClasses)
      { //runFirstPass(itemSubClasses, sClassName + "__", itemsxxx);
      }
    }
  }



  /**writes a methodDefinition with syntax methodDefinition::=.
   *
   * @param zbnfMethod
   * @throws IOException
   * @throws ParseException
   */
  public void write_methodDeclaration(ZbnfParseResultItem zbnfMethod, String sClassName, LocalIdents localIdents)
  throws IOException, ParseException
  { String sName = zbnfMethod.getChild("name").getParsedString();
    //if(!sName.equals("finalize"))  //NOTE: finalize Method see write_finalizeDefinition()
    {    
      //ClassData.Method method = classData.searchMethod(sName + "?", null);
      boolean bArgumentSensitive = classData.isAmbiguousnessMethod(sName); //method.isAmbigous();
      ZbnfParseResultItem itemType = zbnfMethod.getChild("type");
      ClassData methodType = getType(itemType,localIdents);
      
      ClassData.Method method = gen_methodHeadAndRegisterMethod(zbnfMethod, classData, sName, methodType.classTypeInfo, bArgumentSensitive);
      if(!sName.equals("finalize"))  //NOTE: finalize Method see write_finalizeDefinition()
      {   
        writeContent.writeClassH(method.getMethodHeadDefiniton() + ";\n\n");
      }
      ClassData.MethodWithZbnfItem methodDef = classData.new MethodWithZbnfItem(method, zbnfMethod, 'm'); 
      classData.listMethodWithZbnf.add(methodDef);
             
    }
  }


  

  /**generates a method Head with syntax <code>methodDefinition::=</code>.
   * @param zbnfMethod Zbnf item defined in <code>&lt;methodDefinition></code> 
   *        or <code>&lt;constructorDefinition></code>. The head hasn't an own zbnf-result item.
   *        The following contained elements are tested:
   *        <ul>
   *        <li><code>&lt;description></code>, calling {@link #get_description(ZbnfParseResultItem)}
   *        <li><code>&lt;static></code> defined in <code>Modifier::=<?></code> 
   *        <li><code>&lt;argumentList></code>, calling {@link #gen_methodArguments(ZbnfParseResultItem, LocalIdents, char, String[], List)}
   *        </ul>
   *        If this argument is null, a default constructor is generated.
   * @param sClassName environment class
   * @param sMethodName Name of the method, "ctorO" if a constructor from given ObjectJc should be generated.
   * @param localIdents The known identificators of the calling environment, the type identificators are used especially.
   *        This parameter is used calling {@link #gen_variableDefinition(ZbnfParseResultItem, LocalIdents, List, char intension, ClassData[])}
   *        inside with the parameters of the head. The intension parameter on call is set to 'a' if intension=='c' (Method definition in C-File).
   *        Therefore the The parameter-variables are added to the localIdents.
   * @param typeReturn Type of the method, get outside because this routine is also used to generate constructors.
   *        On constructor generation this argument should be equal the environment class type. 
   * @param bArgumentSensitive true than build the name of constructor regarding the type of the arguments.
   * @param intension Intension of call: 'h'-declaration in h-File, 'c'-definition in c-file. 
   *        If intension=='h', no informations are added to localIdents. Only the type informations there are used. 
   *        Thats why localidents may be the {@link ClassData#classLevelIdents}.
   * @return The method head definition in one line. 
   *         If an associated comment is found, it is written as lines above the head line separated with a simple newline.
   * @throws IOException
   * @throws ParseException
   */
  protected ClassData.Method gen_methodHeadAndRegisterMethod
  ( ZbnfParseResultItem zbnfMethod
  //, String sClassName
  , ClassData classOfMethod    
  , String sMethodName
  , FieldData typeReturn
  , boolean bArgumentSensitive
  )
  throws IOException, ParseException
  { LocalIdents methodIdents = new LocalIdents(classData.classLevelIdents, classData);
    String ret = "";
    String sDescription = zbnfMethod!=null ? get_description(zbnfMethod) : "Default constructor. ";
    if(sDescription != null)
    { ret += "/**" + sDescription + "*/\n";
    }
    assert(classOfMethod == classData);  //the argument classOfMethod is unnecessary.
    String sMethodNameC = sMethodName + "_";
    String sArguments;
    String sClassName = classOfMethod.getClassIdentName();
    String sMethodClassType_s = classOfMethod.getClassCtype_s();
    if(sMethodName.equals("ctorO") && sClassName.equals("MainController"))
      stop();
    if(sMethodName.equals("seekAnyString") && sClassName.equals("StringPart"))
      stop();
    boolean isStatic;
    FieldData[] argTypes; // =   bArgumentSensitive ? new ClassData[30] : null; //more than 30 args are not expected.
    //List<ClassData> listArgTypes =   bArgumentSensitive ? new LinkedList<ClassData>() : null; //more than 30 args are not expected.
    List<FieldData> listArgTypes = new LinkedList<FieldData>(); //more than 30 args are not expected.
    if(zbnfMethod!=null) //not the default constructor
    { //NOTE: call without zbnfMethod if a default ctor is created.
      isStatic = zbnfMethod.getChild("static") != null;
      String[] sArgSensitive = bArgumentSensitive ? new String[1] : null;
      char intensionArgument = 'a'; //(intension == 'h') ? 'A' : 'a'; //declaration or definition list.
      sArguments = gen_methodArguments(zbnfMethod, methodIdents, intensionArgument, sArgSensitive, listArgTypes);
      if(bArgumentSensitive && sArgSensitive[0].length() >0)
      { sMethodNameC += sArgSensitive[0] + "_";
      }
      if(listArgTypes != null && listArgTypes.size() >0)
      { argTypes = new FieldData[listArgTypes.size()];
        listArgTypes.toArray(argTypes);
      }
      else
      { argTypes = null;
      }
    }
    else
    { sArguments = "";
      argTypes = null;
      isStatic = false;
    }
    /**The declaring class of the method may be a super-class or interface. */
    ClassData declaringClassOfMethod = classData;
    sMethodNameC += classOfMethod.getClassIdentName();
    //this.sMethodNameCurrent = sMethodNameC;  //used for STACKTRC_ENTRY.
    final ClassData.Method methodData;
    { /*Test whether this method overloads a superclass or interface method: */
      ClassData.Method primaryMethod =null; 
      ClassData.InheritanceInfo primaryInheritanceInfo = null;
      ClassData.InheritanceInfo inheritanceInfo = classData.inheritanceInfo;
      while(inheritanceInfo != null)
      { if(inheritanceInfo.superClass != null)
        { //search in the superclass
          ClassData.Method method = inheritanceInfo.superClass.classData.searchOverloadedMethod(sMethodName, argTypes);
          if(method != null)
          { //found
            primaryMethod = method; primaryInheritanceInfo = inheritanceInfo.superClass; 
          }
        }
        //TODO Test the interfaces.
        
        inheritanceInfo = inheritanceInfo.superClass;
      }
      if( primaryInheritanceInfo != null)
      { /*replace in the table of the own class, it is a copy of the table of the superclass: */
        int idxMtbl = primaryMethod.getIdxMtbl();
        /*It was created because the superclass has overloadable methods. see ctor of ClassData.Inheritance. */
        assert(primaryInheritanceInfo.methodTable != null);
        assert(primaryInheritanceInfo.methodTable.size() >= idxMtbl); 
        primaryInheritanceInfo.methodTable.set(idxMtbl, sMethodNameC);
        declaringClassOfMethod = primaryInheritanceInfo.classData;
      }
      //TODO: final is not overloadable, all other are.
      methodData = classData.addMethod(declaringClassOfMethod, sMethodName, sMethodNameC, 0, typeReturn, argTypes);
    }
    methodData.setMethodIdents(methodIdents);  //classLevelIdents plus arguments.
    String sTypeReturn = typeReturn.typeClazz.getClassCtype_s(); //  .sTypeIdent;
    if(sTypeReturn.equals("Entry"))
      stop();
    if(!typeReturn.typeClazz.isPrimitiveType() && !sTypeReturn.equals("StringJc"))
    { //All not primitive types expect StringJc returns a simple reference.
      sTypeReturn += "*"; 
    }
    else
    { //
    }
    String sMethodHead = "METHOD_C " + sTypeReturn + " " +sMethodNameC + "(";
    if(sMethodName.equals("ctorO"))
    { sMethodHead += "ObjectJc* yObj, ";
    }
    else if(declaringClassOfMethod != classData)
    { sMethodHead += declaringClassOfMethod.getClassCtype_s() + "* ithis, ";
    }
    else if(!isStatic)
    { sMethodHead += sMethodClassType_s + "* ythis, ";
    }
    else
    { sMethodHead += "/*static*/ ";
    }
    sMethodHead += sArguments; 
    sMethodHead += "ThCxt* _thCxt)";
    ret+= sMethodHead;
    methodData.setMethodHeadDefiniton(ret);
    return methodData;
  }


  
  
  
  /**Generates the formal argument list of a method used both by generating method declaration or method definition.
   * @param zbnfArglist Zbnf item defined in <code>argumentList::=<?>...</code>.
   * @param localIdents The known identificators of the calling environment, the type identificators are used especially.
   *        This parameter is used calling {@link #gen_variableDefinition(ZbnfParseResultItem, LocalIdents, List, char intension, ClassData[])}
   *        inside with the parameters of the head. If the intension=='a' (Method definition in C-File),the parameter-variables are added to the localIdents.
   * @param intension Intension of call: 'A'-declaration in h-File, 'a'-definition in c-file, passing on <code>gen_variableDefinition()</code>. 
   * @param retArgSensitiveName [0] is filled with the short name of arg types if not null.
   * @param retArgTypes The list is filled with the detect argument types, used for argument sensitive method call.
   * @return String of formal arguments.
   * @throws ParseException
   */
  String gen_methodArguments
  ( ZbnfParseResultItem zbnfArglist
  , LocalIdents localIdents
  , char intension
  , String[] retArgSensitiveName
  , List<FieldData> retArgTypes
  ) 
  throws ParseException
  { Iterator<ZbnfParseResultItem> iterArguments = zbnfArglist.iterChildren("argument");
    boolean bArgumentSensitive = (retArgSensitiveName != null);
    String sArgs = "";
    String sArgSensitiveName = "";
    String sArgSeparator = "";
    if(iterArguments != null)
    { byte idxArg = 0;
      while(iterArguments.hasNext())
      { ZbnfParseResultItem itemArgument = iterArguments.next();
        CCodeData codeArg = gen_variableDefinition(itemArgument, localIdents, null, intension);
        //sArgs += sArgSeparator + sArg;
        sArgs += codeArg.cCode + ", ";
        if(bArgumentSensitive)
        { //sArgSensitiveName += typeVariable[0].sArgSensitiveName;
          sArgSensitiveName += codeArg.identInfo.typeClazz.sArgSensitiveName;
          int dimension = codeArg.identInfo.dimensionArrayOrFixSize;
          while(dimension >0)
          { dimension -=1;
            sArgSensitiveName += "Y";  //array  
          }
        }
        if(retArgTypes != null) { retArgTypes.add(codeArg.identInfo); }
        idxArg +=1;
        sArgSeparator = ", ";
      }
    }
    if(bArgumentSensitive){ retArgSensitiveName[0] = sArgSensitiveName; }
    return sArgs;
  }  


  
  
  
  
}
