/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @author JcHartmut: hartmut.schorrig@vishia.de
 * @version 2008-04-06  (year-month-day)
 * list of changes:
 * 2008-04-06 JcHartmut: some correction
 * 2008-03-15 JcHartmut: creation
 *
 ****************************************************************************/
package org.vishia.java2C;

import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeMap;

import org.vishia.zbnf.ZbnfParseResultItem;

/**This is the superclass for {@link FirstPass} and {@link SecondPass}.
 * It contains commonly methods used in both passes and commonly data for both passes.
 * This class isn't store as instance between both passes, elsewhere the ClassData to store are stored
 * in the instance of {@link ClassData}, all the ClassData are contained in a list
 * inside {@link AllClassData}. 
 * The attributes of this class are copied from the {@link Java2C_Main} in construction 
 * and there are forgotten after running the pass.
 * 
 *  
 * 
 * @author JcHartmut
 *
 */ 
class GenerateClass
{
  /**String to generate newline and indent. */
  final private String sNewLine = "\n                                                                              ";

  //ZbnfParseResultItem resultItemClass;
  /**A writer to file. */
  protected iWriteContent writeContent;

  /**A TreeMap of all classData sorted by classname to find all other classData.
   * This list is complete if the second path runs.
   */
  //protected final TreeMap<String, ClassData> allClassData;
  protected final LocalIdents allClassData;
  
  
  /**The data of this class for first and second path. A nested class will be recognized as a normal class. */
  protected ClassData classData;
  
  /**The C-name of the current translated method, used for _Stacktrace and alloc_s_BlockHeapJc. 
   * This value is set and used inside {@link SecondPass#write_methodDefinition(ZbnfParseResultItem, String, LocalIdents)}.
   */
  protected String sMethodNameCurrent;
  
  /**The interface to call a firstpass run of a needed type. */
  final RunRequiredFirstPass_ifc runRequiredFirstPass;


  
  
  
  
  
  /**constructs the class. This constructor is called from the superclass. 
   * @param writeContent The writer to the C- and H-File.
   * @param allClassData List of all known Classes while translation Java2C
   * @param runRequiredFirstPass Reference to run the first pass of a required class. 
   *        If in the first or second pass any {@link ClassData} from another class are
   *        necessary but there was not found in {@link allClassData}, the first pass of this
   *        required class will called recursively while processing the pass of this class. 
   */
  //protected GenerateClass(iWriteContent writeContent, TreeMap<String, ClassData> allClassData, RunRequiredFirstPass_ifc runRequiredFirstPass)
  protected GenerateClass(iWriteContent writeContent, LocalIdents allClassData, RunRequiredFirstPass_ifc runRequiredFirstPass)
  { //this.sClassName = sClassName;
    //this.resultItemClass = resultItem;
    this.writeContent = writeContent;
    this.allClassData = allClassData;
    this.runRequiredFirstPass = runRequiredFirstPass;
  }




  /**writes the parsed description in the H-File.
   *
   * @param item description::= test_description::=
   * @throws IOException
   */
  protected void write_Description(ZbnfParseResultItem item) throws IOException
  {
    Iterator<ZbnfParseResultItem> iterDescr = item.iteratorChildren();
    if(iterDescr != null)
    { while(iterDescr.hasNext())
      { ZbnfParseResultItem resultItemDesription = iterDescr.next();
        String sSemanticDescription = resultItemDesription.getSemantic();
        if(sSemanticDescription.equals("text"))
        { String sText = resultItemDesription.getParsedString();
          writeContent.writeClassH("\n/**" + sText + "*/\n");
        }
    } }
  }


  /**generates a variable definition with syntax of < variableDefinition>.
   * The variableDefinition is used both in first and second pass of translation,
   * in the first pass there are the class variables, in the second pass there are variables
   * in block statement. But also arguments for method and constructor calls with the syntax 
   * of a <?argument> in a <code>argumentList::=...</code> are translated with this method.
   * It is the same syntax.
   * <br>
   * There are some special cases all considered here:
   * <ul>
   * <li>If it is a <code> final static </code> variable with a immediate simple assignment like
   *   <code>final static int name = 5;</code>, a <code>#define name_Classname 5</code> is produced.
   * <li>If it is a <code>final Type ... = new Type(...)</code>, an embedded struct is defined in C.
   *   That's correct, because a final reference can't be changed and the reference type is equal the
   *   instanciated type. Embedded struct constructs are opportune in C to optimize access and 
   *   memory layout of the data.
   * <li>A reference is generated either as normal forward declaration like <code>struct Type_t* ...</code>
   *   or with using an enhanced reference like <code>Type_REF ...</code>. All enhanced references 
   *   are defined together in 1 headerfile In form <pre>
   *   typedef struct NAMEREF_t { ObjectRefValuesJc refbase; struct NAME_t* ref; } NAMEREF;
   *   </pre>, see {@link referenceNeeded()} and {@link Java2C_Main.writeFileRef()}.
   * </ul>      
   *
   * @param itemVariableDefinition from parsers result.
   * @param idents The environment identifiers.
   * @param variablesToInit In this list the initializations of variables are stored. 
   *        The initializations should be done in C not by the variables declaration 
   *        but in another part of code. For class variables that is the constructor(s). 
   *        Therefore this parameter should be the {@link ClassData#variablesToInit} of this.
   *        But if this routine is called inside a statement block, an extra List instance
   *        for the block should given locally.
   * @param intension intension of calling 
   *        <ul>
   *        <li>A = argument list of declaration of a method or constructor. 
   *        <li>a = argument list of definition of a method or constructor.
   *        <li>b = inside statement block
   *        <li>c = class level 
   *        <li>s = static class level
   *        </ul> 
   * @return Code snipped and informations of the variables declaration.
   * @throws ParseException
   */
  protected CCodeData gen_variableDefinition
  ( ZbnfParseResultItem itemVariableDefinition
  , LocalIdents idents
  , List<ClassData.VariablesToInit> variablesToInit
  , char intension
  )
  throws ParseException
  {
    assert("Aabcs".indexOf(intension)>=0);
    boolean bReference = true;
    boolean bEmbeddedStruct = false;
    final boolean bSimpleArray;
    final boolean bEmbeddedArrayElements;
    final ZbnfParseResultItem zbnfVariablesInitialization;
    /**true if the reference is final. */
    boolean bFinalReference = itemVariableDefinition.getChild("final")!= null;
    boolean bStaticField = itemVariableDefinition.getChild("static")!= null;
    List<ZbnfParseResultItem> listModifier = itemVariableDefinition.listChildren("Modifier");
    final String sNameJava = itemVariableDefinition.getChild("name").getParsedString();
    if(sNameJava.equals("sFilenameBuffer"))
      stop();
    if(sNameJava.equals("nLastIntegerNumber"))
      stop();
    ZbnfParseResultItem zbnfDescription = itemVariableDefinition.getChild("description");
    String[] java2CTags = getJava2CTags(zbnfDescription);
    
    final ClassData typeVariable;
    final int dimensionArray;
    ZbnfParseResultItem itemType = itemVariableDefinition.getChild("type");
    if(itemType != null)
    {
      typeVariable = getType(itemType, idents);
      dimensionArray = getDimensionsArray(itemType);
      if(typeVariable.sClassNameC.equals("OS_TimeStamp"))
        stop();
    }
    else if( (itemType = itemVariableDefinition.getChild("va_arg")) != null)
    { typeVariable = Java2C_Main.singleton.standardClassData.clazz_int32; //field_int_va_arg; //clazzObjectJc;
      dimensionArray = 1;
    }
    else
    { //it may be a:
      itemType = itemVariableDefinition.getChild("ContainerType");
      String sType = itemType.getParsedString();
      if(itemType == null)
        stop();
      typeVariable = null;
      dimensionArray = 0;
    }
    String sType = typeVariable.getClassIdentName(); //gen_EnvIdent(itemType, "__", idents);  //Type may be a enclosed type in an environment class.
    final String[] fixArraySizes;  //set if a fix array should be created.
    final ZbnfParseResultItem zbnfAssignment;
    final boolean bNoGC = zbnfDescription != null && zbnfDescription.getChild("noGC")!= null;
    int sizeFixStringBuffer = -1;
    /**Check if it is an new Type(param), it may be an embedded struct. */
    //if(sType.contains("RefValueGen__Target"))
    if(sType.equals("int32"))
      stop();
    if(dimensionArray >0)
    { zbnfAssignment = null; //a array initialization is tested instead.
    
      /**embedded array elements are possible especially for an embedded whole array,
       * there is no further memory, all in one. But also for an allocated array.
       * The type of embedded array elements are signed with "_Y" as postfix.
       * But the type for non embedded array elements is "_YP".
       */
      //TODO tested is one-dimension.
      //typeVariableBase = typeVariable;  //may be a parentArrayType
      //The last element type:
      //bEmbeddedArrayElements = typeVariable.isFinal;
      bEmbeddedArrayElements = zbnfDescription != null && zbnfDescription.getChild("embeddedYElements")!= null;
      
      ClassData typeElement = typeVariable;
      ZbnfParseResultItem zbnfNewArray = itemVariableDefinition.getChild("newArray");  //it may be or may null.
      ZbnfParseResultItem zbnfConstArray = itemVariableDefinition.getChild("constArray");  //it may be or may null.
      if("csSb".indexOf(intension)>=0)
      { //only at class level, test if it should be an embedded struct in C
        if(zbnfNewArray != null)
        { zbnfVariablesInitialization = zbnfNewArray;
          ZbnfParseResultItem zbnfTypeOfNew = zbnfNewArray.getChild("className");
          ClassData classdataNewArray = getType(zbnfTypeOfNew, idents);
          if(classdataNewArray == typeElement)
          { //it is a form Type final name = new Type(...), implement an embedded struct in C.
            if("csS".indexOf(intension)>=0)
            { //if it is a variable of a class, static or non static,
              //it is an embedded struct only if the referenced type is final also.
              bEmbeddedStruct = typeElement.isFinal;
            }
            else
            { assert(intension=='b');
              //if it is in an statement block, it is only an embedded struct, in this case in stack,
              //  only if the @java2c=stackInstance. is set.
              bEmbeddedStruct = zbnfDescription != null && zbnfDescription.getChild("stackInstance")!= null;
            }
            if(bEmbeddedStruct)
            { bSimpleArray = isJava2CTag(java2CTags, "simpleArray")
                         || zbnfDescription != null && zbnfDescription.getChild("simpleArray")!= null;
              bReference = false;
              bFinalReference = false;
              fixArraySizes = new String[dimensionArray];
              List<ZbnfParseResultItem> listSize = zbnfNewArray.listChildren("value");
              int idxArraySize = 0;
              for(ZbnfParseResultItem zbnfValueSize : listSize)
              { CCodeData codeValue = genConstantValue(zbnfValueSize);
                fixArraySizes[idxArraySize] = codeValue.cCode;
              }
            }
            else //not bEmbeddedStruct
            { bSimpleArray = false;
              fixArraySizes = null;
            }
          }
          else
          { //different Type1[] name = new Type2[]
            bSimpleArray = false; //unsued, but it have to be initzialized.
            fixArraySizes = null;
          }
          
        }
        else if(zbnfConstArray != null)
        { zbnfVariablesInitialization = zbnfConstArray;
          fixArraySizes = new String[dimensionArray];
          bEmbeddedStruct = typeElement.isFinal;
          bSimpleArray = bEmbeddedStruct;  //everytime simple if it is final, because it has only deterministic elements. 
          bReference = false;
          bFinalReference = false;
          List<ZbnfParseResultItem> listValues = zbnfConstArray.listChildren(); 
          fixArraySizes[0] = "" + listValues.size();   //only 1-dimensional, nr of values is the size 
        }
        else
        { //no ...new Type[]
          zbnfVariablesInitialization = null;
          bSimpleArray = false;
          fixArraySizes = null;
        }
      }
      else
      { //not at class level Type[]
        zbnfVariablesInitialization = null;
        bSimpleArray = false;
        fixArraySizes = null;
      }
    }
    else 
    { 
      fixArraySizes = null;
      bEmbeddedArrayElements = bSimpleArray = false; //unsued, but it have to be initzialized.
      ZbnfParseResultItem itemNewObject = itemVariableDefinition.getChild("newObject");  //it may be or may null.
      if(itemNewObject != null)
      { //only at class level, test if it should be an embedded struct in C
        zbnfVariablesInitialization = itemNewObject;
        zbnfAssignment = null;
        if("cb".indexOf(intension) >=0)
        { ZbnfParseResultItem zbnfTypeOfNew = itemNewObject.getChild("className");
          ClassData classdataNewObject = getType(zbnfTypeOfNew, idents);
          if(classdataNewObject == typeVariable)
          //String sTypeNewObject = gen_EnvIdent(itemNewObject.getChild("className"), "__", idents);  //itemNewObject.getChild("className").getParsedString();
          //if(sTypeNewObject.equals(sType))
          { //it is a form Type final name = new Type(...), implement an embedded struct in C.
            if(typeVariable == Java2C_Main.singleton.standardClassData.clazzStringBufferJc
              && zbnfDescription != null && zbnfDescription.getChild("fixStringBuffer")!= null
              )
            { //The StringBuffer should have a fix direct buffer.
              final ZbnfParseResultItem zbnfArg = zbnfVariablesInitialization.getChild("actualArguments");
              if(zbnfArg != null)
              { final ZbnfParseResultItem zbnfValue = zbnfArg.getChild("value");
                if(zbnfValue != null)
                { final ZbnfParseResultItem zbnfNumber = zbnfValue.getChild("intNumber");
                  if(zbnfNumber != null)
                  {
                    sizeFixStringBuffer = (int)zbnfNumber.getParsedInteger();
                  }
                }
                
              }
            }
            if("csS".indexOf(intension)>=0)
            { //if it is a variable of a class, static or non static,
              //it is an embedded struct only if the referenced type is final also.
              bEmbeddedStruct = true;
              bReference = false;
              bFinalReference = false;
              writeContent.addIncludeH(typeVariable.sFileName, "embedded type in class data");  //it is need, because the type should be known.
            }
            else
            { assert(intension=='b');
              //if it is in an statement block, it is only an embedded struct, in this case in stack,
              //  only if the @java2c=stackInstance. is set.
              bEmbeddedStruct = zbnfDescription != null && zbnfDescription.getChild("stackInstance")!= null;
              if(bEmbeddedStruct)
              { bReference = false;
                bFinalReference = false;
                writeContent.addIncludeC(typeVariable.sFileName, "embedded type in block");  //it is need, because the type should be known.
              }
            }
          }
        }
      }  
      else 
      { //no new, no array
        zbnfVariablesInitialization = zbnfAssignment = itemVariableDefinition.getChild("value");  //it may be null
      }
    }  
    
    if(typeVariable.isPrimitiveType() || typeVariable.bEmbedded)
    { bReference = false;
    }

    boolean bFinalType = typeVariable.isFinal;
    CCodeData codeVariableDefinition = gen_VariableDefinitionCore
    ( zbnfVariablesInitialization, sNameJava, typeVariable, dimensionArray
    , bFinalType, bFinalReference, bStaticField, bEmbeddedStruct, sizeFixStringBuffer, bNoGC, fixArraySizes, bSimpleArray, bEmbeddedArrayElements, bReference, intension
    );
    
    if(  codeVariableDefinition.identInfo.modeStatic != 'd'  //if a #define is generated, no static definition is need
      && (  zbnfVariablesInitialization != null
         || "Ss".indexOf(codeVariableDefinition.identInfo.modeStatic) >0
         )
      )
    { if(  intension == 's' )         //a static declaration in h file needs a definition in c-File.
      { //TODO what is with bFinal? const?
        ClassData.VariablesToInit variableToInit = 
          new ClassData.VariablesToInit(codeVariableDefinition.identInfo, zbnfVariablesInitialization);
        classData.staticVariables.add(variableToInit);
      }
      else if(zbnfVariablesInitialization !=null && variablesToInit !=null)
      { //char kind = bEmbeddedStruct ? '$' : '@';
        ClassData.VariablesToInit variableToInit = 
          new ClassData.VariablesToInit(codeVariableDefinition.identInfo, zbnfVariablesInitialization);
        variablesToInit.add(variableToInit);
      }
    }
    
    if(idents != null)
    { /**Put identInfo.*/
      if("cs".indexOf(intension)>=0)
      { //at class level, class variable or static variable of the class.
        idents.putClassElement(sNameJava, codeVariableDefinition.identInfo);
      }
      else if(intension!='A')
      { //elements in a argument list of declaration should not be stored.
        idents.putLocalElement(sNameJava, codeVariableDefinition.identInfo);
      }
    }
    
    String sDescription = get_description(itemVariableDefinition);
    String sRet = codeVariableDefinition.cCode;
    if(sDescription != null)
    { codeVariableDefinition.cCode += "  /*" + sDescription + "*/";
    }
    return codeVariableDefinition;
  }


  
  
  /**generates the code for the variable regarding some conditions.
   * @param zbnfAssignment
   * @param sNameC
   * @param typeVariable
   * @param dimensionArrayOrFixSize
   * @param bFinalType
   * @param bFinalReference
   * @param bStaticField
   * @param bEmbeddedStruct
   * @param fixArraySizes
   * @param bSimpleArray true if the Java2C=simpleArray is found in description, 
   *        than a fix Array is defined without Array head (Object, length,...) 
   * @param bReference
   * @param intension
   * @param retModifier
   * @return
   */
  private CCodeData gen_VariableDefinitionCore
  ( final ZbnfParseResultItem zbnfAssignment
  //, final ZbnfParseResultItem zbnfNewArray    
  , final String sNameJava
  , final ClassData typeVariable
  , int dimensionArray
  , final boolean bFinalType
  , final boolean bFinalReference
  , final boolean bStaticField
  , final boolean bEmbeddedStruct
  , final int sizeFixStringBuffer
  , final boolean bNoGC
  , String[] fixArraySizes
  , final boolean bSimpleArray
  , final boolean bEmbeddedArrayElements
  , final boolean bReference
  , final char intension
  )
  { final String sPreModifier;
    final String sType_Name;
    final String sComment;
    final char modeArrayElement;
    final char staticMode;
    final char accessMode;
    final String sNameC = bStaticField 
                       ? sNameJava +  "_" + classData.getClassIdentName()  //with postfix _Type
                       : sNameJava;                                        //the same as in Java 
    List<ZbnfParseResultItem> listZbnfAssignments;
    CCodeData codeConstantValue;
    final String sClassTypeC;
    if(sNameJava.equals("testCnt"))
      stop();
    //ClassData[] typeConstantValue = new ClassData[1];
    if(  intension == 's'   //static variable
      && bFinalType         //final static TYPE name = assignment
      && zbnfAssignment!= null
      && (listZbnfAssignments = zbnfAssignment.listChildren()).size()==1   //a simple constant assignemnt
      && (codeConstantValue = genConstantValue(listZbnfAssignments.get(0))) != null
      && !codeConstantValue.identInfo.typeClazz.sClassNameC.equals("s0")
      )
    { //an assignment with one static constant value: implement in C as #define name value.
      //a define is used in C as immediate constant, useable at example in case val:
      //but a const TYPE name = val is not immediate, but indirect.
      //therefore the #define is a important language element in C.
      sPreModifier = "#define ";
      sType_Name = sNameC + " " + codeConstantValue.cCode;
      sComment = "j2c:%d";
      staticMode = 'd';
      accessMode = '%';
      modeArrayElement = '.';
      sClassTypeC = null;  //unused.
      //assignment = null;  //donot create entry for variablesToInit
    }
    else 
    { 
      final String sTypeResult;
      if(intension == 's')   //static variable, it is in headerFile outside a frame
      { if(bFinalType && zbnfAssignment!= null)
        { sPreModifier = "extern const ";
          staticMode = 'S'; //static const element
        }
        else
        { sPreModifier = "extern ";
          staticMode = 's'; //static element
        }
      }
      else
      { staticMode = '.';
        sPreModifier = "";
      }
      
      //if(typeVariable.isArray())
      if(dimensionArray >0)
      { if(fixArraySizes != null)
        { ClassData typeElement = typeVariable;
          String sDimension = "";
          for(String sArraySize: fixArraySizes)
          { sDimension += "[" + sArraySize + "]";
            //typeElement = typeElement.parentArrayType; 
          }
          sTypeResult = typeElement.getClassIdentName();
          sClassTypeC = typeElement.getClassCtype_s();
          if(bSimpleArray)
          { accessMode = '%';
            if(typeVariable.isPrimitiveType())
            { sType_Name = sClassTypeC + " " + sNameC + sDimension;  //array of type direct.
              modeArrayElement = '%';
              sComment = "j2c:sarray[%]";
            }
            else if( sTypeResult.equals("StringJc"))   //array of string references
            { sType_Name = "StringJc " + sNameC + sDimension;  //array of type direct
              modeArrayElement = 't';    //Elements of the array are embedded in the array
              sComment = "j2c:sarray[%t]";
            }
            else if(bEmbeddedStruct)
            { sType_Name = sClassTypeC + " " + sNameC + sDimension;  //array of type direct.
              //its false, see next comment: modeArrayElement = '$';    //Elements of the array are embedded in the array
              //Elements of the array are embedded in the array, 
              //but the expression array[idx] presents a pointer to the struct, it is a peculiarity of C-Language.
              modeArrayElement = '$';//'Y';
              sComment = "j2c:sarray[$]";
            }
            else
            { sType_Name = "struct " + sTypeResult + "_t* " + sNameC + sDimension; //array direct of pointer
              modeArrayElement = '*';
              sComment = "j2c:sarray[*]";
            }
          }
          else
          { //fix array, but with head
            String sStructArray = "struct { ObjectJc object; int32 length; int16 sizeElement; int16 mode; ";
            accessMode = '$';
            if(typeVariable.isPrimitiveType())
            { sType_Name = sStructArray+ sClassTypeC + " data" + sDimension + "; } " + sNameC;  //array of type direct.
              modeArrayElement = '%';
              sComment = "j2c:larray[%]";
            }
            else if( sTypeResult.equals("StringJc"))   //array of string references
            { sType_Name = sStructArray+ "StringJc data" + sDimension + "; } " + sNameC;  //array of type direct
              modeArrayElement = 't';    //Elements of the array are embedded in the array
              sComment = "j2c:larray[%t]";
            }
            else if(bEmbeddedArrayElements)
            { sType_Name = sStructArray+ sClassTypeC + " data" + sDimension + "; } " + sNameC;
              //its false, see next comment: modeArrayElement = '$';    //Elements of the array are embedded in the array
              //Elements of the array are embedded in the array, 
              //but the expression array[idx] presents a pointer to the struct, it is a peculiarity of C-Language.
              modeArrayElement = '$';//'Y';
              sComment = "j2c:larray[$]";
            }
            else
            { sType_Name = sStructArray+ "struct " + sTypeResult + "_t* data" + sDimension + "; } " + sNameC; //array direct of pointer
              modeArrayElement = '*';
              sComment = "j2c:larray[*]";
            }
            if(false && bStaticField)
            { sType_Name += "_" + classData.sClassNameC;
            }
          }
        }
        else
        { //not an fix array 
          sTypeResult = typeVariable.getClassIdentName(); //classDataType.sName;
          sClassTypeC = typeVariable.getClassCtype_s();
          String sTypeRef;
          if("cs".indexOf(intension) >=0 && !bNoGC)
          { accessMode = '@';     //enhanced Reference to an Type_Y
            sTypeRef = "REF ";
          }
          else
          { accessMode = '*';     //Type_Y is a simple reference
            sTypeRef = " ";
          }
          
          if( typeVariable.isPrimitiveType())   //array of int, float etc.
          { sType_Name = sTypeResult + "_Y" + sTypeRef + sNameC;  //array of type direct
            modeArrayElement = '%';    //Elements of the array are embedded in the array
            sComment = "j2c:array_Y[%]";
          }
          else if( sTypeResult.equals("StringJc"))   //array of string references
          { sType_Name = sTypeResult + "_Y" + sTypeRef + sNameC;  //array of type direct
            modeArrayElement = 't';    //Elements of the array are embedded in the array
            sComment = "j2c:array_Y[%t]";
          }
          else if(  bEmbeddedArrayElements )                 //special case of complex data in an array 
          { sType_Name = sTypeResult + "_Y" + sTypeRef + sNameC;  //array of type direct
            //its false, see next comment: modeArrayElement = '$';    //Elements of the array are embedded in the array
            //Elements of the array are embedded in the array, 
            //but the expression array[idx] presents a pointer to the struct, it is a peculiarity of C-Language.
            modeArrayElement = '$';//'Y';    
            sComment = "j2c:array_Y[$]";
          }
          else
          { sType_Name = sTypeResult + "_YP" + sTypeRef + sNameC;  //array of pointer TODO
            //The expression array[idx] is the pointer itself.
            modeArrayElement = '*';
            typeVariable.need_YP();
            sComment = "j2c:array_Y[*]";
          }
          if(false && bStaticField)
          { sType_Name += "_" + classData.getClassIdentName();
          }
        }
      } 
      else 
      { //not an array
        if(bEmbeddedStruct)
        {
          sTypeResult = typeVariable.getClassIdentName(); //classDataType.sName;
          sClassTypeC = typeVariable.getClassCtype_s();
          sType_Name = sClassTypeC + " " + sNameC;
          if(false && bStaticField)
          { sType_Name += "_" + classData.getClassIdentName();  //sClassNameC;
          }
          //ret += " /*Java2C: embedded struct because new(...) in Java line.*/ ";
          accessMode = '$';
          sComment = "j2c:$";
          modeArrayElement = '.';
        }
        else if(bReference)   //not an primitive type
        { //forward declared reference or enhanced reference?
          if(  intension == 'A'        //forward declaration always in argument lists
            || intension == 'a'        //forward declaration always in argument lists
            || intension == 'b'        //forward declaration always in stack variable of block statement
            || typeVariable.isExternal()
            //|| classDataType == null   //to a unknown, not in Java for C, converted type
            || (  bFinalType                      //the type is final, no index of jump table is needed
               && bFinalReference                 //and the reference variable is final
               && classData.isStaticInstanciated  //either isStaticInstanciated and final reference
               )
            || bNoGC    
            )
          { //forward declared reference type, no garbage collector is needed, no index of jump table is needed.
            sTypeResult = typeVariable.getClassIdentName(); //classDataType.sName;
            sClassTypeC = typeVariable.getClassCtype_s();
            if(typeVariable.bEmbedded || sTypeResult.equals("StringJc") || sTypeResult.equals("MemC"))
            { sType_Name = sClassTypeC + " " + sNameC;      //it is always the enhanced reference type, but not regarded in GC here.
              accessMode = '%';
              sComment = "j2c:%";
            }
            else
            { sType_Name = "struct " + sTypeResult + "_t* " + sNameC;
              accessMode = '*';
              sComment = "j2c:*";
            }
          }
          else    
          { //enhanced reference
            sClassTypeC = null; //unused?
            sTypeResult = typeVariable.getClassIdentName(); //classDataType.sName;
            if(sTypeResult.equals("StringJc"))
            { sType_Name = "StringJc " + sNameC;      //StringJc is an enhanced reference itself.
              accessMode = 't';
              sComment = "j2c:%t";
            }
            else
            { sType_Name = sTypeResult + "REF " + sNameC;
              accessMode = '@';
              sComment = "j2c:@";
            }
          }  
          if(false && bStaticField)
          { sType_Name += "_" + classData.sClassNameC;
          }
          modeArrayElement = '.';
        }
        else
        { //primitive type:
          sClassTypeC = sTypeResult = typeVariable.getClassIdentName(); //classDataType.sName;
          sType_Name = sTypeResult + " " + sNameC;
          if(false && bStaticField)
          { sType_Name += "_" + classData.sClassNameC;
          }
          accessMode = '%';
          sComment = "j2c:%";
          modeArrayElement = '.';
        }
      }
    }
    final String sInitializer;
    if(intension == 'b' && bEmbeddedStruct) //&& typeVariable.isPrimitiveType())
    { sInitializer = "= { 0 }";
    }
    else { sInitializer = ""; }
    final String sFixBufferC = sizeFixStringBuffer >0 
                             ? "char " + sNameC + "_SbY[" + sizeFixStringBuffer + "];"
                             : "";
    final String sVariableDefC;
    if(sizeFixStringBuffer >0)
    { sVariableDefC = "struct SbY_" + sNameC + "_t {"
                     + sPreModifier + sClassTypeC 
                     + " sb; char _b[" + sizeFixStringBuffer + "]; }"
                     + sNameC + ";";
      
    }
    else
    { sVariableDefC = sPreModifier + sType_Name + sInitializer 
                     + (intension != 'a' && intension != 'A' && staticMode != 'd' ? ";" : "")
                     //+ sFixBufferC
                     + " /*" + sComment + "*/";
    }
    FieldData identInfo = new FieldData
    ( sNameJava, typeVariable, staticMode, accessMode
    , sizeFixStringBuffer > 0 ? sizeFixStringBuffer : dimensionArray
    , fixArraySizes
    , sizeFixStringBuffer > 0 ? 'B' : modeArrayElement
    , classData
    ); 
    CCodeData retData = new CCodeData(sVariableDefC, identInfo);
    return retData;
  }    
  
  
  /**gets the description from any item which contains < description>.
   * The description is shorted to one line, but only to ". ", the first sentence.
   * @param parent The item
   * @return Description guaranteed without newline.
   */
  String get_description(ZbnfParseResultItem parent)
  { String sDescription = null;
    ZbnfParseResultItem itemDescription = parent.getChild("description");
    if(itemDescription != null)
    { sDescription = itemDescription.getChild("text").getParsedString();
      int posNewline = sDescription.indexOf('\n');
      int posDot = sDescription.indexOf(". ");
      int posEnd = posDot > 0 && posDot < posNewline ? posDot : posNewline > 0 ? posNewline : -1;
      if(posEnd > 0){ sDescription = sDescription.substring(0, posEnd); }
    }
    return sDescription;
  }

  
  /**generates constants values from parse result of the syntaxterm simpleValue. 
   * It is called if a < simpleValue> is expected, but the simpleValue should or may be a constant.
   * At example generation a <code>#define NAME VALUE</code> expression in C where a 
   * <code>final static int NAME = VALUE;</code> is given in Java.  
   * 
   * @param item The part of a simpleValue what is  
   * <pre>
   * [ <""?simpleStringLiteral>
   * | <''?simpleCharLiteral>
   * | 0x<#x?hexNumber>
   * | - 0x<#x?hexNumberNegative>
   * | <#-?intNumber>[?\.]
   * | <#f?floatNumber>F
   * | <#f?doubleNumber>
   * ]</pre>
   *
   * @return null if it is not any of the named part, than it may be a non constant value, see
   *         syntax description in Java.zbnf,
   *         return the constant expression in C if it is a constant one.
   *         
   */
  //protected String genConstantValue(ZbnfParseResultItem item, ClassData[] typeValue)
  protected CCodeData genConstantValue(ZbnfParseResultItem item)
  { final ClassData typeValue;
    String simpleValue;
    String sSemantic = item.getSemantic();
    if(sSemantic.equals("value"))
    { ZbnfParseResultItem zbnfChild = item.firstChild();
      return genConstantValue(zbnfChild);
    }
    else
    { if(sSemantic.equals("simpleStringLiteral"))
      { simpleValue = "\"" + item.getParsedString() + "\"";
        typeValue = Java2C_Main.singleton.standardClassData.clazz_s0;
      }
      else if(sSemantic.equals("booleanConst"))
      { simpleValue = item.getParsedString();
        typeValue = Java2C_Main.singleton.standardClassData.clazz_bool;
      }
      else if(sSemantic.equals("nullRef"))
      { simpleValue = item.getParsedString();
        typeValue = Java2C_Main.singleton.standardClassData.clazzObjectJc;
      }
      else if(sSemantic.equals("simpleCharLiteral"))
      { simpleValue = "\'" + item.getParsedString() + "\'";
        typeValue = Java2C_Main.singleton.standardClassData.clazz_char;
      }
      else if(sSemantic.equals("hexNumber"))
      { simpleValue = "0x" + Long.toHexString(item.getParsedInteger());
        typeValue = Java2C_Main.singleton.standardClassData.clazz_int32;
      }
      else if(sSemantic.equals("hexNumberNegative"))
      { simpleValue = "-0x" + Long.toHexString(item.getParsedInteger());
        typeValue = Java2C_Main.singleton.standardClassData.clazz_int32;
      }
      else if(sSemantic.equals("intNumber"))
      { simpleValue = "" + item.getParsedInteger();
        typeValue = Java2C_Main.singleton.standardClassData.clazz_int32;
      }
      else if(sSemantic.equals("floatNumber"))
      { simpleValue = "" + item.getParsedFloat()+"F";
        typeValue = Java2C_Main.singleton.standardClassData.clazz_float;
      }
      else if(sSemantic.equals("doubleNumber"))
      { simpleValue = "" + item.getParsedFloat();
        typeValue = Java2C_Main.singleton.standardClassData.clazz_double;
      }
      else
      { simpleValue = null;
        typeValue = null;
      }
      if(simpleValue == null)
      { return null;
      }
      else
      { final CCodeData codeInfo = new CCodeData(simpleValue, typeValue.classTypeInfo);
        return codeInfo;
      }
    }  
  }
  

  
  
  /**generates a String with the correct indentation. 
   * 
   * @param indent nr of indentation, as counter for nesting
   * @return "\n" and the appropriate number of spaces.
   */
  protected String genIndent(int indent)
  { return sNewLine.substring(0, 2*indent +1);
  }



  /**Generates an identifier in C from a identifier with a possibly given environment class in Java.
   * At example <code>Math.PI</code> in Java: Math is the environment class. 
   * The result in C is <code>Math_PI</code>.
   * Another example: <code>Outerclass.Innerclass</code>.
   * 
   * @param parent The result of a syntax term < envIdent>
   * @param sSep Separator string. It may be mostly a underline "_". Note: In C it should be
   *        a simple identifier.
   * @return The identifier for C.
   */
  protected String xxxgen_EnvIdent(ZbnfParseResultItem parent, String sSep, LocalIdents localIdents)
  { String sRet = "";
    ZbnfParseResultItem itemEnv = parent;
    while( (itemEnv=parent.getChild("envIdent"))!= null)
    { sRet += parent.getChild("name").getParsedString() + sSep;
      parent = itemEnv;
    }
    sRet += parent.getChild("name").getParsedString();
    ClassData classData = localIdents.getType(sRet);
    if(classData != null)
    { sRet = classData.getClassIdentName(); //The C-name
    }
    else
    { stop();
    }
    return sRet;
  }



  /**Provides the {@link ClassData} for the given type. The type is given in form of a parse result item
   * with semantic 
   * <pre>
   * type::=
   * [ [<?name>List|LinkedList|ArrayList|Iterator|Class][ \< <type?ContainerElementType> \>] 
   * | [<?name>Map|TreeMap|ArrayList|Iterator|Class][ \< <type?ContainerElementType> \>\< <type?ContainerElementType2> \>] 
   * | <envIdent?typeIdent>
   * ] 
   * [{<?typeArray> \[ \]}].
   * </pre>. 
   * Especially because the possibility of <code>&lt;envIdent?typeIdent></code> the requested type
   * may be a type inside another class (an inner class, <code>Outerclass.Innerclass</code>), 
   * so this class should be provided before, and the type should be searched there.
   * <ul>
   * <li>If the type is known, it means the {@link ClassData} with the given key are found,
   *     starting in the {@link LocalIdents} of the current context, further searching in the 
   *     {@link Java2C_Main#userTypes} and the {@link Java2C_Main#standardClassData}, 
   *     if necessary searched as inner type. It are one or some simple accesses to the appropriate 
   *     {@link LocalIdents}. 
   * <li>If the type isn't found, or the environment type (<code>&lt;envIdent?typeIdent></code>)
   *     isn't found, it is possible that this type isn't translated yet, but it is a part of the
   *     users <code>type.java</code> sources to translate. In this case a <code>type.java</code>-file
   *     should be part of input files, given with cmd line arguments <code>-i:FILE</code> or
   *     inside the <code>-if:INPFILE</code>. Therefore 
   *     {@link RunRequiredFirstPass_ifc#runRequestedFirstPass(String type)} is called.
   *     This routine either processes the parsing and first pass of translation for that file,
   *     or it parses the appropriate <code>type.stc</code> File and provides therewith
   *     the {@link ClassData}. The parsing is a deep recursively process, because other types
   *     are need too. This routine is called than recursively.
   * <li>If a file with the requested type isn't found, it may be either an not provided standard type
   *     (this is a deficiency of this translator, it may be any yet not considered Java standard class
   *     is used), or it may be an external type which is provided in the C-handwritten environment, 
   *     where a only emulating <code>type.java</code>-File in the Java sources exists. In both cases
   *     a new {@link ClassData} with attribute "external" is created.    
   * </ul>
   * In all cases a useable {@link ClassData}-instance is returned.
   *  
   * @param zbnfType The zbnf-component with syntax <code>type::=...</code>. 
   * @param localIdents The idents of this level.
   * @return The ClassData of the type.
   */
  protected ClassData getType
  ( ZbnfParseResultItem zbnfType
  , LocalIdents localIdents
  )
  { String sRet = "";
    ClassData classDataType = null;
    ZbnfParseResultItem itemEnv = zbnfType;
    ZbnfParseResultItem itemTypeIdent = zbnfType.getChild("typeIdent");
    String sType;
    if(itemTypeIdent != null)
    {
      while( (itemEnv=itemTypeIdent.getChild("envIdent"))!= null)
      { sType = itemTypeIdent.getChild("name").getParsedString();
        itemTypeIdent = itemEnv;
        classDataType = localIdents.getType(sType);
        if(classDataType ==null)
        {
          classDataType = runRequiredFirstPass.runRequestedFirstPass(sType);  //may be null if it is an external type.
          //If file not found, than classDataType is always still null. It is an external used type. 
          if(classDataType == null)
          { classDataType = new ClassData(sType, sType, sType, '.', sType, "+extern+");
            allClassData.putClassType(sType, sType, classDataType, "C", null);
          }
        }
   
        localIdents = classDataType.classLevelIdents;
      }
      sType = itemTypeIdent.getChild("name").getParsedString();
    }
    else
    { //containertype
      sType = zbnfType.getChild("name").getParsedString();
      ZbnfParseResultItem containerElementType = zbnfType.getChild("ContainerElementType");
    }  
    if(sType.equals("void"))
      stop();

    /**Search to appropriate ClassData. If not found, run first pass or create new ClassData for extern type. */
    if(sType.equals("SetValueGenerator"))
      stop();
    classDataType = localIdents.getType(sType);
    if(classDataType ==null)
    {
      classDataType = runRequiredFirstPass.runRequestedFirstPass(sType);  //may be null if it is an external type.
      //If file not found, than classDataType is always still null. It is an external used type. 
      if(classDataType == null)
      { classDataType = new ClassData(sType, sType, sType, '.', sType, "+extern+");
        allClassData.putClassType(sType, sType, classDataType, "C", null);
      }
    }
    return classDataType;
  }


  
  public int getDimensionsArray(ZbnfParseResultItem zbnfType)
  {
    int dimensionArray;
    /**Check if it is an array type. */
    if(zbnfType.getChild("typeArray") != null)
    { List<ZbnfParseResultItem> zbnfArray = zbnfType.listChildren("typeArray");
      dimensionArray = zbnfArray.size();
    }
    else{ dimensionArray = 0; }
    return dimensionArray;
  }
  
  
  
  protected String[] getJava2CTags(ZbnfParseResultItem zbnfDescription)
  { final String[] sTags;
    if(zbnfDescription != null)
    { List<ZbnfParseResultItem> listTags = zbnfDescription.listChildren("Java2CTag");
      if(listTags != null)
      { sTags = new String[listTags.size()];
        int idxTag = 0;
        for(ZbnfParseResultItem tag: listTags)
        { sTags[idxTag++] = tag.getParsedString();
        }
      }
      else
      { sTags = null;  //no Java2CTag 
      }
    }
    else
    { sTags = null;  //zbnfDescription == null
    }
    return sTags;
  }
  

  
  protected boolean isJava2CTag(String[] tags, String tag)
  { boolean bOk;
    if(tags == null) { return false; }         //return false: no tags given
    else
    { for(String tag1: tags)
      { if(tag1.equals(tag)){ return true;}    //return true: found
      }
      return false;                            //return false: not found
    }
  }
  
  
  
  /**It's a debug helper. The method is empty, but it is a mark to set a breakpoint. */
  protected void stop()
  { //debug
  }

}
