/****************************************************************************
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @author JcHartmut: hartmut.schorrig@vishia.de
 * @version 2008-04-06  (year-month-day)
 * list of changes:
 * 2008-04-06 JcHartmut: some correction
 * 2008-03-15 JcHartmut: creation
 *
 ****************************************************************************/
package org.vishia.java2C;

import java.io.File;
import java.text.ParseException;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map;

//import org.vishia.java2C.Java2C_Main.ReadStructure.Zbnf_Field;
import org.vishia.mainCmd.Report;
import org.vishia.util.FileSystem;


import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.UnsupportedCharsetException;

import org.vishia.util.StringPart;
import org.vishia.util.StringPartFromFileLines;
import org.vishia.zbnf.ZbnfJavaOutput;
import org.vishia.zbnf.ZbnfParseResultItem;
import org.vishia.zbnf.ZbnfParser;


/**This class is the main class of translation Java2C. 
 * It is used from the command line call-implementation class {@link Java2C}.  
 * @author JcHartmut
 *
 */
public class Java2C_Main implements RunRequiredFirstPass_ifc
{

  /**Aggregation to the Console implementation class.*/
  Report console;

  /**One file entry to translate. Instances of this class are created while processing 
   * {@link org.vishia.util.FileSystem#addFileToList(String, org.vishia.util.FileSystem.AddFileToList)}
   * calling {@link }
   * called in {@link #addInputFilemask(String sMask, String, String)}
   * evaluating the input arguments of command line call.
   * 
   * @author JcHartmut
   *
   */
  private static class FileIn
  { final File file; 
    final String sPrefix, sPostfix;
    
    private boolean bTranslated = false;
    
    FileIn(File file, String sPrefix, String sPostfix){ this.file = file; this.sPrefix = sPrefix; this.sPostfix = sPostfix; }
    
    public void setTranslated(){ bTranslated = true; }
    
    public boolean isTranslated(){ return bTranslated; }
  }
  
  
  /**Helper class to create instances of {@link Java2C_Main.FileIn} for each detect input file. 
   * The class holds informations for pre- and postfixes for the C-filenames.
   * This class implements the {@link org.vishia.util.FileSystem.AddFileToList}-interface.
   * <br>
   * using:
   * <ul>
   * <li>Only one private instances {@link Java2C_Main#listFileIn} is created in the outer class 
   * {@link Java2C_Main}. 
   * <li>The instance is used  processing all input files. That is either on any <code>-i:INPUT</code>-argument
   * or evaluating a given <code>-if:INPFILE</code> inside {@link #addInputFromFile(String)}.
   * <li>Both routines call {@link #addInputFilemask(String sMask, String sCPrefix, String sCPostfix)}.
   * <li>Inside that routine 
   * {@link org.vishia.util.FileSystem#addFileToList(String, org.vishia.util.FileSystem.AddFileToList)}
   * is called with this instance. 
   * <li>Inside <code>addFileToList</code> the interface method 
   * {@link org.vishia.util.FileSystem.AddFileToList#add(File)} is called. 
   * This routine is implemented here and adds instances of {@link Java2C_Main.FileIn} per detected file.
   * 
   * @author JcHartmut
   *
   */
  private static class ListFileIn implements FileSystem.AddFileToList
  {
    /**List of all files, which are inputs for Java2C-translation. */
    private List<FileIn> listFileIn = new LinkedList<FileIn>();
    
    /**Prefix and Postfix for actual parsing. It is temporary used. The Prefix and Postfix 
     * for translated C-Files is stored in {@link FileIn#sCName} using this informations.
     */
    private String sPrefix = "", sPostfix = "";
    
    
    /**Adds a file called in {@link org.vishia.util.FileSystem#addFileToList(String, org.vishia.util.FileSystem.AddFileToList)}.
     * @see org.vishia.util.FileSystem.AddFileToList#add(java.io.File)
     */
    public void add(File file)
    { String sJavaFileName = file.getName();
      //String sCName = sPrefix + sJavaFileName + sPostfix;
      listFileIn.add(new FileIn(file, sPrefix, sPostfix));
    }

    /**Sets the pre- and postfix for the next detected files. */
    public void setPrePostfix(String sPrefix, String sPostfix)
    { this.sPrefix = sPrefix == null ? "" : sPrefix; 
      this.sPostfix = sPostfix == null ? "" : sPostfix;
    }
    
  }
  
  
  /**Instance of {@link Java2C_Main.ListFileIn}. */
  private final ListFileIn listFileIn = new ListFileIn();

  /**Path to file given with cmd line argument <code>-if:INPFILE</code>.*/
  private String sInputFile;
  
  
  /**Sets the file given with cmd line argument <code>-if:INPFILE</code>.*/
  public void setInputFile(String s){ sInputFile = s; }
  
  
  /**The parser for *.java input files. The parser is initialized with syntax 
   * from file <code>Java2C.zbnf</code> in directory given with cmd line args 
   * <code>-syntax:SYNTAXDIR</code> in the routine {@link #init()} 
   * called in {@link #execute()}. */
  final ZbnfParser parser;
  
  /**The parser for *.stc input files. The parser is initialized with syntax 
   * from file <code>Structures.zbnf</code> in directory given with cmd line args 
   * <code>-syntax:SYNTAXDIR</code> in the routine {@link #init()} 
   * called in {@link #execute()}. */
  final ZbnfParser parserStruct;

  /**Writer for a file containing all enhanced references. */
  BufferedWriter fileRef;

  /**List of known types. */
  final TreeMap<String, ClassData> xxxallClassData = new TreeMap<String, ClassData>();
  //final AllData allClassData;

  /**List of all files processed in the first pass to consider in the second pass. */
  final List<GenerateFile> allFileParseResults = new LinkedList<GenerateFile>();

  /**Path to the *.c, *.h and *.stc output files, given with cmd line argument <code>-o:OUTPUT</code>. */
  private String sPathOut;
  
  
  private String sAllReferenceFilePath;

  /**This class contains all standard types. It is instanciated as singleton. */
  public final static class CRuntimeJavalikeClassData
  {
    /**This type identifies a place-holder for variable arguments. */
    public final ClassData clazz_va_argRaw = new ClassData("va_argRaw","va_argRaw", "v", '.', null, "+final+primitive+");
    /**This type identifies the va_list type defined in stdarg.h of standard-C. */
    public final ClassData clazz_va_list = new ClassData("Va_list","va_list", "v", '.', null, "+final+primitive+");
    
    public final ClassData clazz_void = new ClassData("void","void", "V", '.', "stdarg", "+final+primitive+");
    public final ClassData clazz_bool = new ClassData("boolean", "bool", "b", 'Z', null, "+final+primitive+");
    public final ClassData clazz_int8 = new ClassData("byte", "int8", "i", 'B', null, "+final+primitive+");
    public final FieldData ident_byte_AY = new FieldData("byte_AY", clazz_int8, '.', '%', 1, null, '%', clazz_int8); 
    public final ClassData clazz_int16 = new ClassData("short", "int16", "i", 'S', null, "+final+primitive+");
    public final ClassData clazz_int32 = new ClassData("int", "int32", "i", 'I', null, "+final+primitive+");
    public final ClassData clazz_int64 = new ClassData("long", "int64", "l", 'J', null, "+final+primitive+");
    public final ClassData clazz_int = new ClassData("int", "int", "i", 'I', null, "+final+primitive+");
    public final ClassData clazz_s0 = new ClassData("s0", "s0", "z", 'z', null, "+final+primitive+");   //String-Literal "xyz"
    public final ClassData clazz_char = new ClassData("char", "char", "c", 'C', null, "+final+primitive+");
    public final ClassData clazz_float = new ClassData("float", "float", "f", 'F', null, "+final+primitive+");
    public final ClassData clazz_double = new ClassData("double", "double", "f", 'D', null, "+final+primitive+");
    public final static ClassData clazzObjectJc = new ClassData("Object", "ObjectJc", "O", 'L', "Jc/ObjectJc", "");
    public final ClassData clazzExceptionJc = new ClassData("Exception", "ExceptionJc", "Exc", 'x', "Fwc/fw_Exception", "");
    public final ClassData clazzLogMessageFW = new ClassData("LogMessage", "LogMessageFW_i", "LogM", 'L', "Fwc/fw_LogMessage", "+final+ObjectJc+");
    public final ClassData clazzLogMessageConsole = new ClassData("LogMessageConsole", "LogMessageConsole", "Logc", 'L', "MsgDisp/LogMessageConsole", "+final++ObjectJc+");
    public final ClassData clazzVaArgBuffer = new ClassData("VaArgBuffer", "VaArgBuffer", "Vargb", 'L', "MsgDisp/VaArgBuffer", "+final+");
    public final ClassData clazzInteger = new ClassData("Integer", "IntegerJc", "Intg", 'L', "Jc/ObjectJc", "");
    public final ClassData clazzDouble = new ClassData("Double", "DoubleJc", "Double", 'L', "Jc/ObjectJc", "");
    public final ClassData clazzMath = new ClassData("Math", "MathJc", "Math", 'L', "Jc/ObjectJc", "");
    public final ClassData clazzArrays = new ClassData("Arrays", "ArraysJc", "Arrays", 'L', "Jc/ArraysJc", "");
    public final ClassData clazzDate = new ClassData("Date", "DateJc_s", "Date", 'L', "Jc/DateJc", "+ObjectJc+");
    public final ClassData clazzSimpleDateFormat = new ClassData("SimpleDateFormat", "SimpleDateFormatJc_s", "Dateformat", 'L', "Jc/DateJc", "+ObjectJc+");
    public final ClassData clazzTextFieldPositionJc = new ClassData("FieldPosition", "TextFieldPositionJc_s", "FieldPos", 'L', "Jc/DateJc", "+ObjectJc+");
    public final ClassData clazzFormatterJc = new ClassData("Formatter", "FormatterJc_s", "Fm", 'L', "Jc/FormatterJc", "+ObjectJc+");
    public final ClassData clazzListJc = new ClassData("List", "ListJc", "Li", 'L', "Jc/ListJc", "");
    public final ClassData clazzIteratorJc = new ClassData("Iterator", "IteratorJc", "Iter", 'L', "Jc/ListJc", "");
    public final ClassData clazzCollectionJc = new ClassData("Collection", "CollectionJc", "Collect", 'L', "Jc/ListJc", "");
    public final ClassData clazzConcurrentLinkedQueue = new ClassData("ConcurrentLinkedQueue", "ConcurrentLinkedQueueJc", "Clq", 'L', "Jc/ConcurrentLinkedQueueJc", "");

    
    public final ClassData clazzStringBufferJc = new ClassData("StringBuilder", "StringBufferJc", "Sb", 'b', "Jc/StringBufferJc", "+thx++ObjectJc+");
    public final ClassData clazzStringJc = new ClassData("String", "StringJc", "S", 's', "Jc/StringJc", "+final+embedded+xxxprimitive+");
    public final ClassData clazzMemC = new ClassData("MemC", "MemC", "M", 'm', "Fwc/fw_MemC", "+final+embedded+xxxprimitive+");
    public final ClassData clazzOS_TimeStamp = new ClassData("OS_TimeStamp", "OS_TimeStamp", "t", 't', "OSAL/inc/os_time", "+final+embedded+xxxprimitive+");
    
    public final ClassData clazzSystemJc = new ClassData("System", "SystemJc", "System", 'L', "Jc/SystemJc", "+final+");
      
    public final ClassData clazzPrintStreamJc = new ClassData("PrintStream", "PrintStreamJc", "PrintSt", 'L', "Jc/SystemJc", "+final+");
    public final ClassData clazzFileWriterJc = new ClassData("FileWriter", "FileWriterJc_s", "fWr", 'L', "Jc/FileIoJc", "+ObjectJc+");

    
    /**Typical 1-param arrays. */
    private final ClassData[] param_s0 = { clazz_s0};
    private final ClassData[] param_int8 = { clazz_int8};
    private final ClassData[] param_int16 = { clazz_int16};
    private final ClassData[] param_int = { clazz_int};
    private final ClassData[] param_int32 = { clazz_int32};
    private final ClassData[] param_char = { clazz_char};
    private final ClassData[] paramStringJc = { clazzStringJc};
    private final ClassData[] param_float = { clazz_float};
    private final ClassData[] param_double = { clazz_double};
    
    //private final FieldData field_voidP = new FieldData("void", clazz_void, '.', '*', 0, null, '.', null);
    private final FieldData field_raw = new FieldData("raw", clazz_void, '.', '%', 0, null, '.', null);
    private final FieldData field_boolean = new FieldData("boolean", clazz_bool, '.', '%', 0, null, '.', null);
    private final FieldData field_int = new FieldData("int", clazz_int, '.', '%', 0, null, '.', null);
    private final FieldData field_int16 = new FieldData("int", clazz_int16, '.', '%', 0, null, '.', null);
    private final FieldData field_int32 = new FieldData("int", clazz_int32, '.', '%', 0, null, '.', null);

    private final FieldData field_intY = new FieldData("int", clazz_int, '.', '%', 1, null, '%', null);
    private final FieldData field_charY = new FieldData("char", clazz_char, '.', '%', 1, null, '%', null);
    private final FieldData fieldStringJc = new FieldData("StringJc", clazzStringJc, '.', '%', 0, null, '.', null);
    private final FieldData fieldObjectY = new FieldData("ObjectJc", clazzObjectJc, '.', '*', 1, null, '*', null);
    
    /**This field type identifies a place-holder for variable arguments. */
    public final FieldData field_va_argRaw = new FieldData("va_argRaw", clazz_va_argRaw, '.', '%', 1, null, '%', null);
    
    /**The type MemC is used in C only as embedded type. */
    private final FieldData field_MemC = new FieldData("MemC", clazzMemC, '.', '%', 0, null, '.', null);
    private final FieldData fieldOS_TimeStamp = new FieldData("OS_TimeStamp", clazzOS_TimeStamp, '.', '%', 0, null, '.', null);

    public final ClassData.Method methodASSERT = new ClassData.Method(null, "assert", "assert", "ASSERT", ClassData.Method.modeNoThCxt + ClassData.Method.modeStatic, field_boolean, null);
    
    public final FieldData field_int_va_arg = new FieldData("int_va_arg", clazz_int, '.', '%', 1, null/*new String[]{"11"}*/, '%', null);
    
    CRuntimeJavalikeClassData(LocalIdents stdTypes)
    { 
      stdTypes.putClassType(clazz_void);
      stdTypes.putClassType(clazz_bool);
      stdTypes.putClassType(clazz_int32);
      
      clazz_int32.addCast(clazz_int8, "?");
      clazz_int32.addCast(clazz_int16, "?");
      clazz_int32.addCast(clazz_int, "?");
      clazz_int.addCast(clazz_int8, "?");
      clazz_int.addCast(clazz_int16, "?");
      clazz_int.addCast(clazz_int32, "?");
      
      clazz_int64.addCast(clazz_int8, "?");
      clazz_int64.addCast(clazz_int16, "?");
      clazz_int64.addCast(clazz_int32, "?");
      clazz_int64.addCast(clazz_int32, "?");
      stdTypes.putClassType("long", "int64", clazz_int64, "%", null);
      
      stdTypes.putClassType("short", "int16", clazz_int16, "%", null);
      stdTypes.putClassType("byte", "int8", clazz_int8, "%", null);
      stdTypes.putClassType("float", "float", clazz_float, "%", null);
      stdTypes.putClassType("double", "double", clazz_double, "%", null);
      stdTypes.putClassType("char", "char", clazz_char, "%", null);
      
      
      { //clazz_va_list.addMethod("get", "get_Va_list", 0, clazz_va_list.classTypeInfo);
        /*Note: get() isn't a method, it is the access immediately. In Java a method is necessary: */
        clazz_va_list.addMethod("get", "", ClassData.Method.modeNoThCxt, clazz_va_list.classTypeInfo);
        stdTypes.putClassType(clazz_va_list);
      }
      { 
        stdTypes.putClassType("Object", clazzObjectJc);
      }
      
      { clazzExceptionJc.addMethod("getMessage", "getMessage_ExceptionJc", 0, clazzStringJc.classTypeInfo);
        stdTypes.putClassType("Exception", clazzExceptionJc);
      }
      
      stdTypes.putClassType("LinkedList", "LinkedListJc", null, "@", null);
      
      stdTypes.putClassType("List", "LinkedListJc", clazzListJc, "@", null);
      clazzListJc.addMethod("add", "add_Collection_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32, clazzObjectJc});
      clazzListJc.addMethod("add", "add_ListJc", clazz_int32.classTypeInfo, new ClassData[]{});
      clazzListJc.addMethod("addAll", "addAll_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazzCollectionJc});
      clazzListJc.addMethod("addAll", "addAll_i_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32, clazzCollectionJc});
      clazzListJc.addMethod("clear", "clear_ListJc", 0, clazz_int32.classTypeInfo);
      clazzListJc.addMethod("contains", "contains_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazzObjectJc});
      clazzListJc.addMethod("containsAll", "containsAll_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazzCollectionJc});
      clazzListJc.addMethod("equals", "equals_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazzObjectJc});
      clazzListJc.addMethod("get", "get_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32});
      clazzListJc.addMethod("hashCode", "hashCode_ListJc", 0, clazz_int32.classTypeInfo);
      clazzListJc.addMethod("indexOf", "indexOf_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazzObjectJc});
      clazzListJc.addMethod("isEmpty", "isEmpty_ListJc", 0, clazz_int32.classTypeInfo);
      clazzListJc.addMethod("iterator", "iterator_ListJc", 0, clazz_int32.classTypeInfo);
      clazzListJc.addMethod("lastIndexOf", "lastIndexOf_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazzObjectJc});
      clazzListJc.addMethod("listIterator", "listIterator_ListJc", 0, clazz_int32.classTypeInfo);
      clazzListJc.addMethod("listIterator", "listIterator_i_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32});
      clazzListJc.addMethod("remove", "remove_i_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32});
      clazzListJc.addMethod("remove", "remove_O_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazzObjectJc});
      clazzListJc.addMethod("removeAll", "removeAll_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazzCollectionJc});
      clazzListJc.addMethod("retainAll", "retainAll_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazzCollectionJc});
      clazzListJc.addMethod("set", "set_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32, clazzObjectJc});
      clazzListJc.addMethod("size", "size_ListJc", 0, clazz_int32.classTypeInfo);
      clazzListJc.addMethod("subList", "subList_ListJc", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32, clazz_int32});
      clazzListJc.addMethod("toArray", "toArray_ListJc", 0, clazz_int32.classTypeInfo);
      clazzListJc.addMethod("toArray", "toArray_Array_ListJc", clazz_int32.classTypeInfo, new ClassData[]{});
      
      
      stdTypes.putClassType("Iterator", "IteratorJc", clazzIteratorJc, "@", null);
      clazzIteratorJc.addMethod("hasNext", "hasNext_IteratorJc", 0, clazz_bool.classTypeInfo);
      clazzIteratorJc.addMethod("next", "next_IteratorJc", 0, clazzObjectJc.classTypeInfo);
      clazzIteratorJc.addMethod("remove", "remove_IteratorJc", 0, clazz_void.classTypeInfo);
      
      
      stdTypes.putClassType("ArrayList", "ListJc", null, "@", null);
      stdTypes.putClassType("LinkedList", "ArrayListJc", null, "@", null);
      stdTypes.putClassType("ConcurrentLinkedQueue", "ConcurrentLinkedQueueJc", null, "@", null);

      { clazzStringBufferJc.addMethod("ctorO", "ctorO_i_StringBufferJc", clazzStringBufferJc.classTypeInfo, new ClassData[]{ clazz_int32} );
        clazzStringBufferJc.addMethod("ctorO", "ctorO_S_StringBufferJc", clazzStringBufferJc.classTypeInfo, new ClassData[]{ clazzStringJc} );
        clazzStringBufferJc.addMethod("toString", "toString_StringBufferJc", ClassData.Method.modeNoThCxt, fieldStringJc);
        clazzStringBufferJc.addMethod("length", "length_StringBufferJc", 0, clazz_int32.classTypeInfo);
        clazzStringBufferJc.addMethod("setLength", "setLength_StringBufferJc", 0, clazz_void.classTypeInfo, param_int);
        clazzStringBufferJc.addMethod("append", "append_s0_StringBufferJc", clazzStringBufferJc.classTypeInfo, new ClassData[]{ clazz_s0} );
        clazzStringBufferJc.addMethod("append", "append_s_StringBufferJc", clazzStringBufferJc.classTypeInfo, new ClassData[]{ clazzStringJc} );
        clazzStringBufferJc.addMethod("append", "append_Sb_StringBufferJc", clazzStringBufferJc.classTypeInfo, new ClassData[]{ clazzStringBufferJc} );
        clazzStringBufferJc.addMethod("append", "append_c_StringBufferJc", clazzStringBufferJc.classTypeInfo, param_char );
        clazzStringBufferJc.addMethod("append", "append_i_StringBufferJc", clazzStringBufferJc.classTypeInfo, param_int32 );
        clazzStringBufferJc.addMethod("append", "append_f_StringBufferJc", clazzStringBufferJc.classTypeInfo, param_float );
        clazzStringBufferJc.addMethod("insert", "insert_s0_StringBufferJc", clazzStringBufferJc.classTypeInfo, new ClassData[]{clazz_int, clazz_s0} );
        clazzStringBufferJc.addMethod("insert", "insert_s_StringBufferJc", clazzStringBufferJc.classTypeInfo, new ClassData[]{clazz_int, clazzStringJc } );
        clazzStringBufferJc.addMethod("insert", "insert_c_StringBufferJc", clazzStringBufferJc.classTypeInfo, new ClassData[]{clazz_int, clazz_char } );
        clazzStringBufferJc.addMethod("insert", "insert_i_StringBufferJc", clazzStringBufferJc.classTypeInfo, new ClassData[]{clazz_int, clazz_int32 } );
        clazzStringBufferJc.addMethod("insert", "insert_f_StringBufferJc", clazzStringBufferJc.classTypeInfo, new ClassData[]{clazz_int, clazz_float } );
        clazzStringBufferJc.addMethod("insert", "insert_sii_StringBufferJc", clazzStringBufferJc.classTypeInfo, new ClassData[]{clazz_int, clazzStringJc, clazz_int, clazz_int} );
        clazzStringBufferJc.addMethod("replace", "replace_StringBufferJc", clazzStringBufferJc.classTypeInfo, new ClassData[]{clazz_int, clazz_int, clazzStringJc} );
        stdTypes.putClassType("StringBuilder", clazzStringBufferJc);
        stdTypes.putClassType("StringBuffer", clazzStringBufferJc);
      }
      { ClassData[] params = { clazz_int32};
        //clazzStringJc.addMethod("append", "append_i_thcStringJc", clazzStringBufferJc.classTypeInfo, params );
        clazzStringJc.addMethod("new", "new_bYiiCharset_StringJc", 0, clazzStringJc.classTypeInfo, new FieldData[]{ ident_byte_AY, clazz_int32.classTypeInfo, clazz_int32.classTypeInfo, clazzStringJc.classTypeInfo} );
        clazzStringJc.addMethod("new", "new_cY_StringJc", 0, clazzStringJc.classTypeInfo, new FieldData[]{ field_charY} );
        clazzStringJc.addMethod("new", "new_cYi_StringJc", 0, clazzStringJc.classTypeInfo, new FieldData[]{ field_charY, field_int, field_int} );
        clazzStringJc.addMethod("charAt", "&charAt_StringJc", clazz_char.classTypeInfo, param_int32 );
        clazzStringJc.addMethod("equals", "&equals_StringJc", clazz_int32.classTypeInfo, new ClassData[]{ clazzStringJc} );
        clazzStringJc.addMethod("indexOf", "&indexOf_StringJc", clazz_int32.classTypeInfo, new ClassData[]{ clazz_char} );
        clazzStringJc.addMethod("indexOf", "&indexOf_i_StringJc", clazz_int32.classTypeInfo, new ClassData[]{ clazz_char, clazz_int32} );
        clazzStringJc.addMethod("indexOf", "&indexOf_S_StringJc", clazz_int32.classTypeInfo, new ClassData[]{ clazzStringJc} );
        clazzStringJc.addMethod("indexOf", "&indexOf_Si_StringJc", clazz_int32.classTypeInfo, new ClassData[]{ clazzStringJc, clazz_int32} );
        clazzStringJc.addMethod("startsWith", "&startsWith_StringJc", clazz_bool.classTypeInfo, new ClassData[]{ clazzStringJc} );
        clazzStringJc.addMethod("lastIndexOf", "&lastIndexOf_StringJc", clazz_int32.classTypeInfo, new ClassData[]{ clazz_char} );
        clazzStringJc.addMethod("lastIndexOf", "&lastIndexOf_i_StringJc", clazz_int32.classTypeInfo, new ClassData[]{ clazz_char, clazz_int32} );
        clazzStringJc.addMethod("lastIndexOf", "&lastIndexOf_S_StringJc", clazz_int32.classTypeInfo, new ClassData[]{ clazzStringJc} );
        clazzStringJc.addMethod("lastIndexOf", "&lastIndexOf_Si_StringJc", clazz_int32.classTypeInfo, new ClassData[]{ clazzStringJc, clazz_int32} );
        clazzStringJc.addMethod("length", "length_StringJc", ClassData.Method.modeNoThCxt, clazz_int32.classTypeInfo);
        clazzStringJc.addMethod("size", "size_StringJc", ClassData.Method.modeNoThCxt, clazz_int32.classTypeInfo);
        clazzStringJc.addMethod("substring", "substring_StringJc", clazzStringJc.classTypeInfo, new ClassData[]{ clazz_int32, clazz_int32} );
        clazzStringJc.addMethod("substring", "substring_i_StringJc", clazzStringJc.classTypeInfo, param_int32 );
        clazzStringJc.addMethod("replace", "replace_StringJc", clazzStringJc.classTypeInfo, new ClassData[]{ clazz_char, clazz_char} );
        clazzStringJc.addMethod("getChars", "getChars_StringJc", 0, clazz_void.classTypeInfo, new FieldData[]{ field_int, field_int, field_charY, field_int} );
        clazzStringJc.addCast(clazz_s0, "s0_StringJc(?)");
        clazzStringJc.addCastInitializer(clazz_s0, "CONST_s0_StringJc(?)");
        stdTypes.putClassType("String", clazzStringJc);
      }
      {
        clazzConcurrentLinkedQueue.addMethod("ctorO", "ctorO_Clq_ConcurrentLinkedQueueJc", clazzConcurrentLinkedQueue.classTypeInfo, new ClassData[]{clazzConcurrentLinkedQueue});
        clazzConcurrentLinkedQueue.addMethod("ctorO", "ctorO_MemC_ConcurrentLinkedQueueJc", 0, clazzConcurrentLinkedQueue.classTypeInfo, new FieldData[]{field_MemC});
        clazzConcurrentLinkedQueue.addMethod("add", "add_ConcurrentLinkedQueueJc", clazz_bool.classTypeInfo, new ClassData[]{clazz_void});
        stdTypes.putClassType("ConcurrentLinkedQueue", clazzConcurrentLinkedQueue);
      }
      
      
      
      {
        clazzMemC.addMethod("alloc", "&alloc_MemC", field_MemC, param_int);
        stdTypes.putClassType("MemC", clazzMemC);
      }
      
      {
        clazzOS_TimeStamp.addMethod("os_getDateTime", "os_getDateTime", ClassData.Method.modeNoThCxt, fieldOS_TimeStamp);
        clazzOS_TimeStamp.addField("time_sec", field_int32);
        stdTypes.putClassType("OS_TimeStamp", clazzOS_TimeStamp);
      }
      
      { clazzPrintStreamJc.addMethod("println", "println_s0_PrintStreamJc", clazz_void.classTypeInfo, new ClassData[]{ clazz_s0});
        stdTypes.putClassType("PrintStream", "PrintStreamJc", clazzPrintStreamJc, "@", null);
      }  
      { clazzFileWriterJc.addMethod("ctorO", "ctorO_FileWriterJc", 0, clazzFileWriterJc.classTypeInfo);
        clazzFileWriterJc.addMethod("open", "open_FileWriterJc", 0, clazzFileWriterJc.classTypeInfo, paramStringJc);
        clazzFileWriterJc.addMethod("isOpen", "isOpen_FileWriterJc", ClassData.Method.modeNoThCxt, clazzFileWriterJc.classTypeInfo);
        clazzFileWriterJc.addMethod("write", "write_FileWriterJc", 0, clazz_void.classTypeInfo, paramStringJc);
        clazzFileWriterJc.addMethod("flush", "flush_FileWriterJc", 0, clazz_void.classTypeInfo);
        clazzFileWriterJc.addMethod("close", "close_FileWriterJc", 0, clazz_void.classTypeInfo);
        stdTypes.putClassType(clazzFileWriterJc);
      }  
      { clazzSystemJc.classLevelIdents.putClassElement("out", "PrintStreamJc", clazzPrintStreamJc, 's', '@', "@", 0, null, '.', clazzSystemJc);
        clazzSystemJc.addMethod("arraycopy", "arraycopy_SystemJc", clazz_void.classTypeInfo, new ClassData[]{clazzObjectJc, clazz_int, clazzObjectJc, clazz_int, clazz_int});
        stdTypes.putClassType("System", "SystemJc", clazzSystemJc, "%", null);
      }
      { clazzLogMessageFW.addMethod("sendMsgVaList", "sendMsgVaList_LogMessageFW", ClassData.Method.modeOverloadable, clazz_void.classTypeInfo, new ClassData[]{clazz_int32, clazzOS_TimeStamp, clazzStringJc, clazz_va_list});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_s0_LogMessageFW", clazz_void.classTypeInfo, new ClassData[]{clazz_int32, clazz_s0});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_s_LogMessageFW", clazz_void.classTypeInfo, new ClassData[]{clazz_int32, clazzStringJc});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iDateSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, clazzOS_TimeStamp.classTypeInfo, fieldStringJc, field_va_argRaw});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iDateSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, clazzOS_TimeStamp.classTypeInfo, fieldStringJc, field_va_argRaw, field_va_argRaw});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iDateSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, clazzOS_TimeStamp.classTypeInfo, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iDateSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, clazzOS_TimeStamp.classTypeInfo, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iDateSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, clazzOS_TimeStamp.classTypeInfo, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
        clazzLogMessageFW.addMethod("sendMsg", "sendMsg_iDateSv_LogMessageFW", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, clazzOS_TimeStamp.classTypeInfo, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
        clazzLogMessageFW.addMethod("report", "write_S_LogMessageFW", clazz_void.classTypeInfo, new ClassData[]{clazz_int32, clazzStringJc});
        clazzLogMessageFW.addMethod("reportln", "write_iS_LogMessageFW", clazz_void.classTypeInfo, new ClassData[]{clazz_int32, clazz_int32, clazzStringJc});
        clazzLogMessageFW.addMethod("reportln", "write_S_LogMessageFW", clazz_void.classTypeInfo, new ClassData[]{clazz_int32, clazzStringJc});
        //clazzLogMessageFW.addMethod("convertFromMsgDispatcher", "convertFromMsgDispatcher_LogMessageFW", clazzLogMessageFW.classTypeInfo, new ClassData[]{clazzMsgDispatcher});
        stdTypes.putClassType("LogMessage", "LogMessageJc", clazzLogMessageFW, "$", null);
        stdTypes.putClassType("Report", "LogMessageJc", clazzLogMessageFW, "$", null);
      }
      { clazzLogMessageConsole.addMethod("create", "create_LogMessageConsole", ClassData.Method.modeStatic, clazzLogMessageFW.classTypeInfo);
        stdTypes.putClassType("LogMessageConsole", "LogMessageConsole", clazzLogMessageConsole, "$", null);
      }
      { stdTypes.putClassType(clazzVaArgBuffer);
      }
      { stdTypes.putClassType("Integer", "IntegerJc", clazzInteger, "S", null);
        clazzInteger.classLevelIdents.putClassElement("MAX_VALUE", "int32", clazz_int32, 's', '%', "%", 0, null, '.', clazzInteger);
        clazzInteger.classLevelIdents.putClassElement("MIN_VALUE", "int32", clazz_int32, 's', '%', "%", 0, null, '.', clazzInteger);
        //clazzInteger.addMethod("")
        
      }
      { stdTypes.putClassType("Double", "DoubleJc", clazzDouble, "S", null);
        clazzDouble.addMethod("toString", "toString_DoubleJc", clazzStringJc.classTypeInfo, new ClassData[]{clazz_double});
        //clazzInteger.addMethod("")
        
      }
      { stdTypes.putClassType("Math", "MathJc", clazzMath, "S", null);
        //clazzInteger.addMethod("")
        
      }
      { clazzArrays.addMethod("binarySearch", "binarySearch_int_ii_ArraysJc", 0, field_int, new FieldData[]{field_intY, field_int, field_int, field_int});        
        stdTypes.putClassType("Arrays", "ArraysJc", clazzArrays, "S", null);
      }
      { clazzDate.addMethod("ctorO", "ctorO_DateJc", 0, clazzDate.classTypeInfo);        
        stdTypes.putClassType("Date", clazzDate);
      }
      { clazzSimpleDateFormat.addField("DATE_FIELD", field_int);        
        clazzSimpleDateFormat.addMethod("ctorO", "ctorO_SimpleDateFormatJc", 0, clazzSimpleDateFormat.classTypeInfo);        
        clazzSimpleDateFormat.addMethod("ctorO", "ctorO_S_SimpleDateFormatJc", 0, clazzSimpleDateFormat.classTypeInfo, paramStringJc);        
        clazzSimpleDateFormat.addMethod("applyPattern", "applyPattern_SimpleDateFormatJc", 0, clazz_void.classTypeInfo, paramStringJc);        
        clazzSimpleDateFormat.addMethod("format", "format_tSb_SimpleDateFormatJc", 0, clazzStringBufferJc.classTypeInfo, new FieldData[]{fieldOS_TimeStamp, clazzStringBufferJc.classTypeInfo, clazzTextFieldPositionJc.classTypeInfo });        
        stdTypes.putClassType("SimpleDateFormat", clazzSimpleDateFormat);
      }
      { clazzTextFieldPositionJc.addMethod("ctorO", "ctorO_TextFieldPositionJc", 0, clazzTextFieldPositionJc.classTypeInfo, param_int);        
        stdTypes.putClassType(clazzTextFieldPositionJc);
      }
      { clazzFormatterJc.addMethod("ctorO", "ctorO_FormatterJc", 0, clazzFormatterJc.classTypeInfo);        
        clazzFormatterJc.addMethod("ctorO", "ctorO_Sb_FormatterJc", 0, clazzFormatterJc.classTypeInfo, new ClassData[]{ clazzStringBufferJc });        
        clazzFormatterJc.addMethod("format", "format_FormatterJc", 0, clazzFormatterJc.classTypeInfo, new ClassData[]{ clazzStringJc, clazz_va_list });        
        stdTypes.putClassType(clazzFormatterJc);
      }
    
    }
  
    //because there is no parent, the basicly types have to be defined:
    
  } 
  
  /**Base of all types contained in {@link LocalIdents} and therefore {@link ClassData},
   * which are defined in {@link Java2C_Main.CRuntimeJavalikeClassData}. This types and identifier
   * are accessible in all scopes. See also {@link #userTypes}.
   */
  private final LocalIdents stdTypes;

  /**Assignment between an import statement and a Headerfile. */
  private Map<String, String> importHeaders = new TreeMap<String, String>();
  
  /**That are all types contained in {@link LocalIdents} and therefore {@link ClassData},
   * which are processed in the Java2C translation. This types and identifier
   * are accessible in all scopes. If any type is necessary but not known yet, 
   * the translation of this depended type will be started using 
   * {@link RunRequiredFirstPass_ifc#runRequestedFirstPass(String)}.
   */
  private final LocalIdents userTypes;
  
  /**Instance with all {@link ClassData} and {@link LocalIdents} for the classes 
   * appropriated in the CRuntimeJavalike C runtime environment. All this {@link ClassData}
   * are stored in {@link #stdTypes}, which is the base of all {@link LocalIdents}, 
   * therefore this types are accessible in all scopes.
   * <br>
   * The content of this class is used directly if {@link ClassData} of this standard types are need, 
   * the access is done via the {@link Java2C_Main#singleton}.standardClassData.  
   */
  final Java2C_Main.CRuntimeJavalikeClassData standardClassData;
  
  /**A CCodeInfo instance useable as dummy for local variable. There havn't a reference. */
  public final CCodeData localReferenceDummy = new CCodeData("", null, '%', 0);
  
  /**A CCodeInfo instance useable as dummy for static variable. There havn't a reference. */
  public final CCodeData staticReferenceDummy = new CCodeData("", null, '%', 0);
  
  public void setPathOut(String path)
  { path = path.replace("\\", "/");
    if(!path.endsWith("/")){ path+= "/"; }
    sPathOut = path;
  }


  /**Path to the syntax-zbnf files, given with cmd line argument <code>-syntax:SYNTAXDIR</code>. */
  private String sSyntaxPath;

  public void setSyntaxPath(String path)
  { sSyntaxPath = path;
  }



  public boolean addInputFilemask(String sMask, String sCPrefix, String sCPostfix)
  { boolean bOk = true;
    listFileIn.setPrePostfix(sCPrefix, sCPostfix);
    try{ bOk = FileSystem.addFileToList(sMask, listFileIn); }
    catch(FileNotFoundException exc)
    { bOk = false;
    }
    return bOk;
  }

  /**An instance of this class is filled with the result of parsing an input file 
   * given with <code>-if:INPFILE</code> cmd line calling argument. The instance is created and used
   * inside the routine {@link Java2C_Main#addInputFromFile(String sFileName)}. 
   * That routine parses the result and calls 
   * {@link org.vishia.zbnf.ZbnfJavaOutput#setOutputStrict(Object result, ZbnfParseResultItem, Report)}.
   * The result is this instance.
   * <br>
   * All inner elements follows the syntax.zbnf:
   * <pre>
   * input::={ import: <set?importReplacement> ; | translate: <set> ;} \\e.
   * set::=<* ;=?inputpath>[ =: [[ <* ;*?Cname_prefix>]*] <* ;?Cname_postfix>].
   * </pre>.
   * 
   */ 
  public static final class InputFileParseResult
  { 
    /**Subclass to pour in the result of the <code>set::=...</code> subsyntax.
     * Any instance is created if a <code>&lt;set></code> is parsed with syntax 
     * <pre>
     * set::=<* ;=?inputpath>[ =: [[ <* ;*?Cname_prefix>]*] <* ;?Cname_postfix>].
     * </pre>.
     */
    public static final class Set
    { /**Ingests semantic for inputpath. &lt;* ;=?inputpath>. */
      public String inputpath; 
      /**Ingests semantic for Cname_prefix. &lt;* ;*?Cname_prefix>. */
      public String cname_prefix;
      /**Ingests semantic Cname_postfix. &lt;* ;?Cname_postfix>. */
      public String cname_postfix; 
    }
    
    /**Set from semantic. */
    public String fileAllReferences = "allReferences.h";
    
    /**Creates an appropriate instance  for semantic set. <code>&lt;set></code> */
    public Set new_set(){ return new Set(); }
    /**Adds the instance after it is filled with the content of a <code>&lt;set></code>. */ 
    public void add_set(Set value){ set.add(value); }
    
    /**Creates an appropriate instance for semantic importReplacement. <code>&lt;set?importReplacement></code> */
    public Set new_importReplacement(){ return new Set(); }
    /**Adds the instance after it is filled. <code>&lt;set?importReplacement></code>. */ 
    public void add_importReplacement(Set value){ importReplacement.add(value); }
    
    /**List to hold all <code>&lt;set></code>. */
    private final List<Set> set = new LinkedList<Set>();
    /**List to hold all importReplacement. <code>&lt;set?importReplacement></code>. */
    private final List<Set> importReplacement = new LinkedList<Set>();
  }
      
  
  /**The singleton instance of this class. */
  public static Java2C_Main singleton;
  
  /**The create routine for the singleton. */
  public static void instanciateSingleton(Report report)
  { singleton = new Java2C_Main(report);
  }
  
  
  /**Private Constructor for singleton.  */
  private Java2C_Main(Report report)
  { this.console = report;
    singleton = this;
    stdTypes = new LocalIdents(null, null);
    userTypes = new LocalIdents(stdTypes, null);
    standardClassData = new CRuntimeJavalikeClassData(stdTypes);
  
    parser = new ZbnfParser(console, 10);
    parser.setReportIdents(Report.error, Report.info, Report.debug, Report.fineDebug);

    parserStruct = new ZbnfParser(console, 10);
    parserStruct.setReportIdents(Report.error, Report.info, Report.debug, Report.fineDebug);

    
    //allClassData = new AllData();
  }


  /**Parses the given File and build the input file information for translating.
   * This method is called in the phase of evaluating the calling parameters. 
   * The content of the file will be parsed with ZBNF parsing with the fix syntax:
   * <pre>
   * input::={ translate: <set> ;}. 
   * set::=<* ;=?inputpath>[ =: [*<* ;?Cname_prefix>|<**?Cname_postfix>*]].</pre>
   * A example of file content:
   * <pre>
   * srcJava/org/vishia/exampleJava2C/java4c/*.java = *_PosCtrl; //the files for position control
   * ..//TODO
   * </pre>
   * @param sFileName
   * @return true if successful, false if any error.
   * @throws ParseException 
   */
  public boolean addInputFromFile(String sFileName) 
  throws ParseException
  { boolean bOk = true;
    File fInput = new File(sFileName);
    StringPart spInput = null;
    try{ spInput = new StringPartFromFileLines(fInput,10000, null, null); }
    catch(Exception exc)
    { console.writeError("fault input file", exc);
      bOk = false;
    }
    if(bOk)
    { ZbnfParser parserInput = new ZbnfParser(console, 10);
      parserInput.setSkippingEndlineComment("#", false);
      String syntaxInput = "input::={ fileAllReferences : <* ;?fileAllReferences> ; | import : <set?importReplacement> ; | translate : <set> ;} \\e. "
                         + "set::=<* ;=?inputpath>[ =: [[ <* ;*?Cname_prefix>]*] <* ;?Cname_postfix>]."
                         ;
      try{ parserInput.setSyntax(syntaxInput);} 
      catch(ParseException exc){ throw new RuntimeException(exc); }
      bOk = parserInput.parse(spInput);
      if(bOk)
      { InputFileParseResult parseResult = new InputFileParseResult();
        parserInput.reportStore(console, Report.debug);
        try{ ZbnfJavaOutput.setOutputStrict(parseResult, parserInput.getFirstParseResult(), console);}
        catch(Exception exc){ throw new RuntimeException(exc); }
        sAllReferenceFilePath = parseResult.fileAllReferences;
        for(InputFileParseResult.Set set: parseResult.set)
        { 
          bOk = addInputFilemask(set.inputpath, set.cname_prefix, set.cname_postfix);
          if(!bOk)
          { console.writeError("not found: " + set.inputpath);
          }
        }
        for(InputFileParseResult.Set set: parseResult.importReplacement)
        { String sImport = set.inputpath;
          String sHeader= set.cname_postfix;
          importHeaders.put(sImport, sHeader);
        }
      }
      else throw new ParseException(parserInput.getSyntaxErrorReport(),0);
    }
    return bOk;
  }
  
  
  /**Executes the translation for all files. 
   * @throws Exception
   */
  public void execute() throws Exception
  { if(init())
    { for(FileIn file: listFileIn.listFileIn)
      { if(file.isTranslated())
        { console.writeInfoln("* input file, first pass already processed, was depended: " + file.file.getName()); 
        }
        else
        { //console.writeInfoln( file.file.getAbsolutePath() + "* processing..."); 
          runFirstPass(file.file, file.sPrefix, file.sPostfix, true);
          file.setTranslated();
        }
      }
      for(GenerateFile genFile : allFileParseResults)
      { genFile.runSecondPass(console, userTypes); //allClassData);
      }
      writeFileRef();
      console.writeInfoln("...done.");
    }
  }



  /**Writes the file <code>fileAllReferences.h</code>. This file contains all reference types of user types.
   * It calls {@link #writeFileRefClass(LocalIdents)} for all {@link #userTypes}.
   * The TODO allref is determined by input cfg file.
   * @throws IOException
   */
  private void writeFileRef()
  throws IOException
  {
    fileRef = new BufferedWriter(new FileWriter(sPathOut + "/" + Java2C_Main.singleton.getFileAllReferences() + ""));
    fileRef.write("/* This file, generated by Java2C, contains all enhanced reference definitions.");
    fileRef.write("\n * It needs no further includes, because all user types are forward declared.");
    fileRef.write("\n * This file is included overall to know the requested enhanced reference types.");
    fileRef.write("\n * Enhanced reference types are used for all interfaces and dynamic instances.");
    fileRef.write("\n */");
    fileRef.write("\n#ifndef __" + Java2C_Main.singleton.getIdentAllReferences() + "__");
    fileRef.write("\n#define __" + Java2C_Main.singleton.getIdentAllReferences() + "__");
    
    writeFileRefClass(userTypes);  
    
    fileRef.write("\n#endif  //__" + Java2C_Main.singleton.getIdentAllReferences() + "__");
    fileRef.close();
    fileRef = null;
  }



  /**writes all reference types of the given types in localIdent. Calls itself recursively for inner types. 
   * <br>
   * The {@link #fileRef} is used to output.
   * @param localIdents The types from there are used.
   * @throws IOException 
   */
  private void writeFileRefClass(LocalIdents localIdents) throws IOException  
  {   //if(classData.isReferenceNeeded())
    Set<Map.Entry<String,ClassData>> listInnerClassData = localIdents.getTypeSet();
    for(Map.Entry<String,ClassData> entry:listInnerClassData)
    { ClassData classData = entry.getValue();
      String sName = classData.getClassIdentName();
      fileRef.write("\ntypedef struct " + sName + "REF_t { ObjectRefValuesJc refbase; struct " + sName + "_t* ref; } " + sName + "REF;");
      if(classData.isneed_YP())
      { fileRef.write("\ntypedef struct " + sName + "_YP_t { ObjectJc object; int32 length; int16 sizeElement; int16 mode; struct " + sName + "_t* data[50]; } *" + sName + "_YP;");
      }
      fileRef.write("\n");
      writeFileRefClass(classData.classLevelIdents);  
    }
  }
    

  
  /**Gets the header file name for an import statement. The association is stored in {@link #importHeaders}.
   * @param sImport
   * @return
   */
  String getHeaderReplacementForImport(String sImport)
  { return importHeaders.get(sImport);
  }


  String getFileAllReferences(){ return sAllReferenceFilePath; }
  
  String getIdentAllReferences()
  {  String sIdent1 = sAllReferenceFilePath.replaceAll("\\W", "_");
     return sIdent1;
  }
  
  String getPathOut(){ return sPathOut; }
  
  
  /**Initializes before {@link #execute()}.
   * <ul>
   * <li>Reads the {@link #sInputFile} if an cmd line argument <code>-if:INPFILE</code> is given.
   * <li>Initializes the {@link #parser} with the syntax file <code>Java2C.zbnf</code>
   *     using the path from cmd line argument <code>-syntax:PATH</code>.
   * <li>Initializes the {@link #parserStruct} with the syntax file <code>Structures.zbnf</code>
   *     using the path from cmd line argument <code>-syntax:PATH</code>.
   * </ul>       
   * @return true if successful.
   * @throws ParseException
   * @throws IllegalCharsetNameException
   * @throws UnsupportedCharsetException
   * @throws FileNotFoundException
   * @throws IOException
   */
  boolean init()
  throws ParseException, IllegalCharsetNameException, UnsupportedCharsetException, FileNotFoundException, IOException
  { boolean bOk = true;
    if(sInputFile != null)
    { bOk = addInputFromFile(sInputFile);
    }
    if(bOk)
    { File fileSyntaxJava = null;
      fileSyntaxJava= new File(sSyntaxPath + "/Java2C.zbnf"); //"../../srcJava/org/vishia/Java2C/Java2C.zbnf");
      String sSyntaxJava;
      sSyntaxJava = FileSystem.readFile(fileSyntaxJava);
      if(sSyntaxJava == null)
      { console.writeError("syntaxfile not found, error arg -syntax:" + sSyntaxPath + "/Java2C.zbnf");
        bOk = false;
      }
      else
      { parser.setSyntax(sSyntaxJava);
        //parser.reportSyntax(console);
      }
    }  
    if(bOk)
    { File fileSyntaxStruct = null;
      fileSyntaxStruct= new File(sSyntaxPath + "/Structures.zbnf"); //"../../srcJava/org/vishia/Java2C/Java2C.zbnf");
      String sSyntaxStruct;
      sSyntaxStruct = FileSystem.readFile(fileSyntaxStruct);
      if(sSyntaxStruct == null)
      { console.writeError("syntaxfile not found, error arg -syntax:" + sSyntaxPath + "/Structures.zbnf");
        bOk = false;
      }
      else
      { parserStruct.setSyntax(sSyntaxStruct);
        //parser.reportSyntax(console);
      }
    }  
    return bOk;
  }



  /**parses and translates one java file. It is called 
   * <ul>
   * <li> either from {@link execute()} for any given input file,
   * <li> or from {@link runRequestedFirstPass(String sClassName)} for any depended file.
   * </ul>
   * The following actions are done:
   * <ul>
   * <li>Tests whether the destination file exists and the source isn't newer. In this case 
   *     {@link readStructToClassData(String)} is called, and nothing else. This produces the 
   *     {@link ClassData for using the informations of this file.}
   * <li>If the destination file doesn't exists or the source is newer:
   * <li>Reads the file content in a {@link org.vishia.util.StringPartFromFileLines}.
   * <li>Parses the file with the {@link #parser}.
   * <li>Creates an instance of {@link GenerateFile} and adds it to {@link allFileParseResults}.
   *     This list will be processed for second path of translation.
   * <li>Calls {@link GenerateFile#runFirstPass(ZbnfParseResultItem, LocalIdents, String)}
   *     to build the {@link ClassData} of all contained classes.
   * </ul>            
   * @throws InstantiationException 
   * @throws IllegalAccessException 
   * @throws IllegalArgumentException 
   * */
  void runFirstPass(File javaFile, String sPrefix, String sPostfix, boolean bPrimaryInput)
  throws FileNotFoundException, IOException, ParseException, IllegalArgumentException, IllegalAccessException, InstantiationException
  {
    long dateJavaFile = javaFile.lastModified();
    String sFileName = javaFile.getName();
    int posDot = sFileName.indexOf('.');
    sFileName = sFileName.substring(0, posDot);
    File fileC = new File(sPathOut + sPrefix + sFileName + sPostfix + ".c");
    File fileH = new File(sPathOut + sPrefix + sFileName + sPostfix + ".h");
    File fileS = new File(sPathOut + sPrefix + sFileName + sPostfix + ".stc");
    final boolean shouldTranslate;
    if(fileC.exists() && fileH.exists() && fileS.exists())
    { long dateCfile = fileC.lastModified();
      long dateHfile = fileH.lastModified();
      long dateSfile = fileS.lastModified();
      shouldTranslate = dateJavaFile > dateCfile || dateJavaFile > dateHfile || dateJavaFile > dateSfile;
    }
    else{ shouldTranslate = true; } //because file not exists
    if(shouldTranslate)
    {
      console.writeInfoln( (bPrimaryInput ? "* input file" : "* depending type") +", first pass, parse: " + javaFile.getCanonicalPath() + " ... ");
      StringPart spInput = null;
      //if(allClassData.get(sFileName) == null)
      if(sFileName.equals("PID_controller"))
        stop();
      if(userTypes.getType(sFileName) == null)
      { spInput = new StringPartFromFileLines(javaFile,100000, null, null);
        boolean bOk = parser.parse(spInput);
        if(!bOk)
        { String sError = parser.getFoundedInputOnError();
          console.writeError(sError);
          sError = parser.getExpectedSyntaxOnError();
          console.writeError(sError);
          sError = parser.getSyntaxErrorReport();
  
          console.writeError(sError);
          stop();
  
        }
        else
        { parser.reportStore(console, Report.fineInfo);
          ZbnfParseResultItem resultItem = parser.getFirstParseResult();
          if(resultItem.getSemantic().equals("JavaSrc"))
          { GenerateFile genFile = new GenerateFile(sFileName, sPrefix, sPostfix, this);
            allFileParseResults.add(genFile);
            //parsing and first pass:
            console.writeInfo("OK.");
            genFile.runFirstPass(resultItem, userTypes, sPathOut);
          }
        }
      }
      else
      { //the file is translated before because it is needed by a type.
        stop();
      }
    }
    else
    { //not to translate
      readStructToClassData(sFileName);  
    }
  }


  
  /**An instance of this class is filled with the result of parsing an <code>*.stc</code> file
   * which contains the structure informations of a already translated file.
   * <br>
   * Such a file is parsed inside {@link #readStructToClassData(String sClassFileName)} 
   * called inside {@link #runFirstPass(File javaFile, boolean)},
   * if the destination file exists and the source <code>*.java</code> file isn't newer than the
   * already translated destination.
   * That routine parses the result and calls 
   * {@link org.vishia.zbnf.ZbnfJavaOutput#setOutputStrict(Object result, ZbnfParseResultItem, Report)}.
   * The result is this instance.
   * <br>
   * The syntax of parsing is given in the outside file <code>Structures.zbnf</code> in form:
   * <pre>
   * Structure::= Java2C: structure of file-class: <$?fileName> { class <ClassData> } \e.
   * 
   * ClassData::= <*; ?nameJava> ; <$?nameC> ; 
   * \{ [ fieldIdents \{ [{ field; <field> }] \}]
   *    [ typeIdents \{ [{ <type> | class <ClassData?InnerClassData> }] \}]
   *    [ methods \{ [{ <method> }] \}]
   *    [ InnerClass \{ [{ class <ClassData?InnerClassData> }] \}]
   * \}.
   * 
   * field::= <1*?modeAccess><1*?modeArrayElement><1*?modeStatic>;
   *          [{\[ [\?\?<?arrayDimension>|<*\]?fixArraySizes>]\]}] ;
   *          <*; ?typeJava> ; <$?typeC> ; <$?name> ; 
   *          .
   *          
   * type::= type; <$?typeName>.
   * 
   * method::= method ; <*;?keyName> ; <$?javaName> ; <$?CName> ;
   *           return ; <field?returnType> 
   *           [{ param ( <field?param> ) }]
   *           .
   * </pre>.
   * All syntax/semantic elements have a representation in this class, because
   * {@link org.vishia.zbnf.ZbnfJavaOutput#setOutputStrict(Object result, ZbnfParseResultItem, Report)}.
   * sets the result of parsing directly in the instance <code>result</code> of this class.
   */ 
  public class ReadStructure
  {
    public String sFileName;
    
    /**Subclass to pour in the result of the <code>field::=...</code> subsyntax.
     * Only some singleton instances are created:
     * <ul>
     * <li>{@link Java2C_Main.ReadStructure.Zbnf_Method#returnType1} to save the return type.
     *     It is an own instance because the return type will be processed after alle param are processed.
     * <li>{@link Java2C_Main.ReadStructure.Zbnf_Method#param} to save the current param.
     * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#field} to save the current field.
     * </ul>
     * They are reused, calling {@link #init()} before, if 
     * <ul>
     * <li>{@link Java2C_Main.ReadStructure.Zbnf_Method#new_param()}
     * <li>{@link Java2C_Main.ReadStructure.Zbnf_Method#new_returnType()} 
     * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#new_field()} 
     * </ul>
     * are called because any <code>&lt;field?...></code> were parsed with syntax 
     * <pre>
     * field::= <1*?modeAccess><1*?modeArrayElement><1*?modeStatic>;
     *          [{\[ [\?\?<?arrayDimension>|<*\]?fixArraySizes>]\]}] ;
     *          <*; ?typeJava> ; <$?typeC> ; <$?name> ; 
     * </pre>.
     * If the appropriate methods
     * <ul>
     * <li>{@link Java2C_Main.ReadStructure.Zbnf_Method#add_param(org.vishia.java2C.Java2C_Main.ReadStructure.Zbnf_Field)}
     * <li>{@link Java2C_Main.ReadStructure.Zbnf_Method#add_returnType(org.vishia.java2C.Java2C_Main.ReadStructure.Zbnf_Field)} 
     * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#add_field(org.vishia.java2C.Java2C_Main.ReadStructure.Zbnf_Field)} 
     * </ul>
     * are called, the content of the singleton is read out and stored 
     * in the appropriate destination data structure, that is {@link FieldData}.
     * This temporary instance is used straighten out, because the structure of syntax 
     * is not so opportune for storing in {@link FieldData} directly.
     */
    public class Zbnf_Field
    { private char modeAccess, modeStatic, modeArrayElement;
      
      /**Stores the parsed result
       * inside {@link org.vishia.zbnf.ZbnfJavaOutput#setOutputStrict(Object result, ZbnfParseResultItem, Report)}.
       * To access the field from there via reflection, it should be public.
       */
      public String name, typeJava, typeC;
      
      /**If <code>&lt;*\]?fixArraySizes></code> is parsed, the String result is added to this list
       * inside {@link org.vishia.zbnf.ZbnfJavaOutput#setOutputStrict(Object result, ZbnfParseResultItem, Report)}.
       * To access the field from there via reflection, it should be public.
       */
      public List<String> fixArraySizes;
      
      private int dimensionArray;
      
      private Zbnf_Field()
      { modeAccess= '?'; dimensionArray = 0;
      }
      
      /**Initializes the instance to use.*/
      void init(){modeArrayElement = modeStatic = modeAccess= '?'; typeC = typeJava = null; name = null;}

      /**Sets the parsed <code>modeAccess</code>. Only the first character is used and valid. */
      public void set_modeAccess(String value){ modeAccess = value.charAt(0);}
      /**Sets the parsed <code>modeArrayElement</code>. Only the first character is used and valid. */
      public void set_modeArrayElement(String value){ modeArrayElement = value.charAt(0);}
      /**Sets the parsed <code>modeStatic</code>. Only the first character is used and valid. */
      public void set_modeStatic(String value){ modeStatic = value.charAt(0);}
      
      //public void set_name(String value){ name = value; }

      /**Counts the {@link #dimensionArrayOrFixSize} if <code>[??&lt;?arrayDimension>]</code> is parsed.  */
      public void set_arrayDimension(){ dimensionArray +=1; }
    }
    
    /**Subclass to pour in the result of the <code>type::=...</code> subsyntax.
     * Only a singleton instance is created. It is reused, calling {@link #init()} before, if 
     * <ul>
     * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#new_type()()} 
     * </ul>
     * is called because any <code>&lt;type></code> were parsed with syntax 
     * <pre>
     * type::= type; <$?typeName>.
     * </pre>.
     * If the appropriate method
     * <ul>
     * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#add_type(org.vishia.java2C.Java2C_Main.ReadStructure.Type)} 
     * </ul>
     * is called, the content of the singleton is read out and stored 
     * in the appropriate destination data structure, that is {@link ClassData}.
     * This temporary instance is used straighten out, because the structure of syntax 
     * is not so opportune for storing in {@link ClassData} directly.
     */
    public class Type
    { public String typeName;
      void init(){typeName = null; }
    }
    
    /**Subclass to pour in the result of the <code>method::=...</code> subsyntax.
     * Only a singleton instance is created. It is reused, calling {@link #init()} before, if 
     * <ul>
     * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#new_method()} 
     * </ul>
     * is called because any <code>&lt;method></code> were parsed with syntax 
     * <pre>
     * method::= method ; <*;?keyName> ; <$?javaName> ; <$?CName> ;
     *           return ; <field?returnType> 
     *           [{ param ( <field?param> ) }]
     * </pre>.
     * If the appropriate methods
     * <ul>
     * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#add_method(org.vishia.java2C.Java2C_Main.ReadStructure.Zbnf_Method)} 
     * </ul>
     * is called, the content of the singleton is read out and stored 
     * in the appropriate destination data structure, that is {@link ClassData.Method}.
     * This temporary instance is used straighten out, because the structure of syntax 
     * is not so opportune for storing in {@link ClassData.Method} directly.
     */
    public class Zbnf_Method
    {
      private String sNameJava; String sCName; String sKeyName; 
      
      private final List<FieldData> listParams = new LinkedList<FieldData>();
      
      private final Zbnf_Field returnType1 = new Zbnf_Field();
      
      private final Zbnf_Field param = new Zbnf_Field();
      
      public Zbnf_Field new_returnType(){ returnType1.init(); return returnType1; }
      public void add_returnType(Zbnf_Field dummy){} //NOTE: nothing to do, let it filled.
      
      public Zbnf_Field new_param(){ param.init(); return param; }
      
      public void add_param(Zbnf_Field param)
      {
        ClassData typeClass = userTypes.getTypeContingentlyRunFirstPass(param.typeJava, Java2C_Main.this);
        FieldData identInfo = new FieldData
        ( param.name, typeClass, param.modeStatic, param.modeAccess, 0, null, param.modeArrayElement, null);
        listParams.add(identInfo);
        param.init();
      }
      
      public void set_keyName(String value){ sKeyName = value; }
      
      public void set_javaName(String value){ sNameJava = value; }
      
      public void set_CName(String value){ sCName = value; }
      
      
      Zbnf_Method()
      { 
      }
      
      private void init()
      { returnType1.init();
        param.init();
        listParams.clear();
        sNameJava = sCName = sKeyName = null;
      }
    }
    
    final Zbnf_Method zbnfMethod = new Zbnf_Method();
    
    /**Subclass to pour in the result of the <code>ClassData::=...</code> subsyntax.
     * Only a singleton instance {@link Java2C_Main.ReadStructure#zbnfClassData} is created. 
     * It is reused, calling {@link #init()} before, if 
     * <ul>
     * <li>{@link Java2C_Main.ReadStructure#new_ClassData()} 
     * </ul>
     * is called because any <code>&lt;ClassData>></code> were parsed with syntax 
     * <pre>
     * ClassData::= <*; ?nameJava> ; <$?nameC> ; 
     * \{ [ fieldIdents \{ [{ field; <field> }] \}]
     *    [ typeIdents \{ [{ <type> | class <ClassData?InnerClassData> }] \}]
     *    [ methods \{ [{ <method> }] \}]
     *    [ InnerClass \{ [{ class <ClassData?InnerClassData> }] \}]
     * \}.
     * </pre>.
     * If the appropriate method
     * <ul>
     * <li>{@link Java2C_Main.ReadStructure#add_ClassData(org.vishia.java2C.Java2C_Main.ReadStructure.Zbnf_ClassData)} 
     * </ul>
     * is called, the content of the singleton is read out and stored 
     * in the appropriate destination data structure, that is {@link ClassData}.
     * This temporary instance is used straighten out, because the structure of syntax 
     * is not so opportune for storing in {@link ClassData} directly.
     */
    public class Zbnf_ClassData
    { 
      private String sFileName; boolean bIsStaticInstance; boolean bFinal;
      
      public String nameC, nameJava;
      
      /**temporary list of IdentInfos to add to ClassData. */
      private final List<FieldData> listFields = new LinkedList<FieldData>();
      
      /**temporary list of Method to add to ClassData. */
      private final List<ClassData.Method> listMethods = new LinkedList<ClassData.Method>();
      
      /**The ClassData were set if the first field, method is set.
       * it is because the ClassData of the own class may be necessary in the own fields, methods.
       */
      private ClassData classData;
      
      private final Zbnf_Field field = new Zbnf_Field();
      
      Zbnf_ClassData zbnfInnerClassData; //NOTE: don't create initially, because it will be recursively without end.
  
      /**The classData of the top level are to add to userTypes, inner classes are add to classLevelIdents. */
      final LocalIdents dstToAddClassData;
      
      public Zbnf_ClassData(LocalIdents dstToAddClassData)
      { this.dstToAddClassData = dstToAddClassData;
      }
      
      public Zbnf_Field new_field()
      { field.init();
        return field;
      }
      
      public void add_field(Zbnf_Field field)
      { createClassDataIfNotDone(null, standardClassData.clazzObjectJc); 
        if(field.name.equals("msg") && classData.sClassNameJava.equals("MainController"))
          stop();
        ClassData typeClass = userTypes.getTypeContingentlyRunFirstPass(field.typeJava, Java2C_Main.this);
        if(typeClass == null)
          stop();
        FieldData identInfo = new FieldData
        ( field.name, typeClass, field.modeStatic, field.modeAccess, 0, null, field.modeArrayElement, classData);
        listFields.add(identInfo);      
      }

      public Zbnf_Method new_method()
      { createClassDataIfNotDone(null, standardClassData.clazzObjectJc); zbnfMethod.init(); return zbnfMethod; }
      
      public void add_method(Zbnf_Method m)
      {
        ClassData typeClass = userTypes.getTypeContingentlyRunFirstPass(m.returnType1.typeJava, Java2C_Main.this);
        FieldData returnType = new FieldData
        ( m.returnType1.name, typeClass, m.returnType1.modeStatic, m.returnType1.modeAccess, 0, null, m.returnType1.modeArrayElement, null);
        int sizeParams = m.listParams.size();
        FieldData[] params = new FieldData[sizeParams];
        int idxParam = 0;
        for(FieldData param: m.listParams)
        { params[idxParam++] = param;
        }
        ClassData.Method method = new ClassData.Method(typeClass, m.sKeyName, m.sNameJava, m.sCName, 0, returnType, params);
        listMethods.add(method);
      }
      
      public Type new_type(){ return new Type(); } //TODO
      
      public Zbnf_ClassData new_InnerClassData()
      { createClassDataIfNotDone(null, standardClassData.clazzObjectJc);
        if(zbnfInnerClassData == null) 
        { zbnfInnerClassData = new Zbnf_ClassData(classData.classLevelIdents); }
        zbnfInnerClassData.init(sFileName);
        return zbnfInnerClassData;
      }

      /**Invoked from ZbnfJavaOutput, creates a new ClassData instance, fills it with given Input
       * and add it to the userTypes.
       * @param zbnfInput
       */
      public void add_InnerClassData(Zbnf_ClassData zbnfInput)
      { createClassDataIfNotDone(null, standardClassData.clazzObjectJc);
        for(FieldData field: zbnfInput.listFields)
        { classData.classLevelIdents.putClassElement(field.getName(), field);
        }
        for(ClassData.Method method: zbnfInput.listMethods)
        { classData.addMethod(method);
        }
      }

      public void add_type(Type value){}; //TODO
      
      private void createClassDataIfNotDone(ClassData outerClassData ,ClassData superClassData)
      { if(classData == null)
        { classData = new ClassData(sFileName, nameJava, nameC, outerClassData ,superClassData, null, bIsStaticInstance, bFinal, userTypes);
          dstToAddClassData.putClassType(classData.sClassNameJava, classData.sClassNameC, classData, "C", null);
        }
      }
      
      //public void set_className(String value){ sName = value; }
      
      private void init(String sFileName)
      { this.sFileName = sFileName; nameC = nameJava = null; bIsStaticInstance = false; bFinal = false;
        listFields.clear();
      }
    }
    
    final private Zbnf_ClassData zbnfClassData = new Zbnf_ClassData(userTypes); 
    
    public Zbnf_ClassData new_ClassData()
    { //ZbnfClassData obj = new ZbnfClassData();
      //obj.sFileName = sFileName;
      //return obj;
      zbnfClassData.init(sFileName);
      return zbnfClassData;
    }
    
    /**set content, able to call from ZbnfJavaOutput. */
    public void set_fileName(String value){ sFileName= value; }
    
    /**Invoked from ZbnfJavaOutput, creates a new ClassData instance, fills it with given Input
     * and add it to the userTypes.
     * @param zbnfInput
     */
    public void add_ClassData(Zbnf_ClassData zbnfInput)
    { zbnfInput.createClassDataIfNotDone(null, standardClassData.clazzObjectJc);
      for(FieldData field: zbnfInput.listFields)
      { zbnfInput.classData.classLevelIdents.putClassElement(field.getName(), field);
      }
      for(ClassData.Method method: zbnfInput.listMethods)
      { zbnfInput.classData.addMethod(method);
      }
    }
    
    
    
  }
  
  

  /**reads the structure of a class from a *.stc-File and save it to the {@link #userTypes}-{@link ClassData}.
   * @param sClassFileName The name of the Java class.
   * @throws FileNotFoundException
   * @throws IOException
   * @throws IllegalArgumentException
   * @throws IllegalAccessException
   * @throws InstantiationException
   */
  void readStructToClassData(String sClassFileName) 
  throws FileNotFoundException, IOException, IllegalArgumentException, IllegalAccessException, InstantiationException
  { //???
    //ClassData classForwardOrExternal = new ClassData(sClassName, sClassName, sClassName, null, "+extern+");
    //The put class will be replaced if it is found.
    //userTypes.putClassType(sClassName, sClassName, classForwardOrExternal, "C", null);
    
    File fileStruct = new File(sPathOut + sClassFileName + ".stc");
    if(fileStruct.exists())
    { console.writeInfoln("read structure from " + fileStruct.getAbsolutePath() + ":");
      StringPart spStruct = new StringPartFromFileLines(fileStruct, 100000, null, null);
      boolean bOk = parserStruct.parse(spStruct);
      if(!bOk)
      { console.writeError("error Structure file:" + parserStruct.getSyntaxErrorReport());
      }
      else
      {
        parserStruct.reportStore(console, Report.fineInfo);
        ZbnfParseResultItem zbnfStruct = parserStruct.getFirstParseResult();
        ReadStructure structure = new ReadStructure();
        ZbnfJavaOutput.setOutputStrict(structure, zbnfStruct, console);
        stop();
      }
    } else
    { console.writeInfoln("file not found: " + fileStruct.getCanonicalPath());
    }
  }
  






  /**It's a debug helper. The method is empty, but it is a mark to set a breakpoint. */
  void stop()
  { //debug
  }


  /* (non-Javadoc)
   * @see org.vishia.java2C.RunRequiredFirstPass_ifc#runRequestedFirstPass(java.lang.String)
   */
  public ClassData runRequestedFirstPass(final String sClassName)
  { //search whether there is a file with the required name:
    if(sClassName.equals("Target"))
        stop();
    ClassData classData = userTypes.getType(sClassName);
    if(classData == null)
    { final String sFileRequ = sClassName + ".java";
      FileIn fileRequ = null;
      for(FileIn file: listFileIn.listFileIn)
      { String sFile = file.file.getName();
        /*
        int posDot = sName.lastIndexOf('.');
        if(posDot > 0){ sName = sName.substring(0,posDot); }
        */
        if(sFile.equals(sFileRequ))
        { fileRequ= file; 
          file.setTranslated();  //to mark in List of all input files, it is found as depended.
          break;
        }
      }
      if(fileRequ != null)
      { try
        { runFirstPass(fileRequ.file, fileRequ.sPrefix, fileRequ.sPostfix, false);
          //there may be more as one class in the file, all are translated,
          //get the classData of the requested class.
          classData = userTypes.getType(sClassName);
        }
        catch(ParseException exc)
        { console.writeInfoln("* parse exception.");
        }
        catch(Exception exc)
        { console.writeInfoln("* any exception.");
        }
      }
      else
      { console.writeInfoln("* depending type, file not found: " + sClassName + ".java, it may be external.");
      }
    }  
    return classData;  //null on external unknown class
  }

}