/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @author JcHartmut: hartmut.schorrig@vishia.de
 * @version 2008-04-06  (year-month-day)
 * list of changes:
 * 2008-04-06 JcHartmut: some correction
 * 2008-03-15 JcHartmut: creation
 *
 ****************************************************************************/
package org.vishia.java2C;

import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.vishia.mainCmd.Report;
import org.vishia.zbnf.ZbnfParseResultItem;
import org.vishia.zbnf.ZbnfSyntaxPrescript;


/**This class handle the second pass of the translation.
 * An instance is built only temporary to run the pass. All Data are stored in the {@link ClassData}.
 * Instances of ClassData will be created while running the first pass, there are stored in 
 * the singleton {@link AllData}. Than they are used to run the Second Pass.
 * 
 * @author JcHartmut
 *
 */
public class SecondPass extends GenerateClass
{

  /**This class holds the common values of a statement block and handles the generation of a statement block.
   * 
   * {@link StatementBlock#gen_simpleMethodCall(ZbnfParseResultItem, String, LocalIdents.IdentInfos, ClassData[] retType, LocalIdents) }
   */
  public class StatementBlock
  {
    /**The number of call of <code>new</code> in this blockStatment. 
     * The call of new are counted, generates some special statements in C.
     * {@link }
     */
    public int nrofNew; 
    
    /**The deepness of block nesting. */
    public final int blockNestingCnt;
  
    /**Saves, wether the last generated statement is a return. If it is not so, 
     * some instructions at end of block have to be done.
     */
    public boolean lastWasReturn = false;
  
    /**The identifiers valid at this block level. There are not final because may defined new.*/
    public LocalIdents localIdents;
   
    /**The parent statement block. It is necessary at ex. for gen_ActivateGarbageCollection()
     * called at return statement.
     * @param parentIdents
     */
    final StatementBlock parent;
    
    StatementBlock(StatementBlock parent)
    { //without any own variable of the block, it are equal to parent idents.
      //if any variable definition is found, a new LocalIdents(parent) is called.
      this.parent = parent;
      if(parent != null){
        blockNestingCnt = parent.blockNestingCnt +1;
        nrofNew = 0; //(parent.nrofNew & 0xff) + 0x100;
        localIdents = parent.localIdents;
      }
      else {
        blockNestingCnt = 0;
        nrofNew = 0;
        localIdents = null;
      }
    }
    
    StatementBlock(LocalIdents parentLocalIdents, boolean bClassLevel)
    { //without any own variable of the block, it are equal to parent idents.
      //if any variable definition is found, a new LocalIdents(parent) is called.
      this.parent = null;
      blockNestingCnt = 0;
      nrofNew = 0;
      localIdents = parentLocalIdents;
    }
    
    /**generates the statement for <code>activateGarbageCollectorAccess_BlockHeapJc()</code>.
     * This is placed on end of a statement block or before an <code>return</code>-statement,
     * if inside new-statements are generated.
     * 
     * @param indent number of indentations. 2 spaces per indentation are produced.
     * @return The statement as string with indentation +1, but without newline.
     */
    public String gen_ActivateGarbageCollection(int indent, boolean bRet)
    { String ret = "";
      StatementBlock blockLevel = this;
      do
      { int ctNrofNew = 0;
        while(++ctNrofNew <= blockLevel.nrofNew)
        { ret += genIndent(indent) + "activateGarbageCollectorAccess_BlockHeapJc(newObj" + blockLevel.blockNestingCnt + "_"  + ctNrofNew + ");";
        }
      } while(bRet && (blockLevel = blockLevel.parent) != null);  //repeat it only if return for all block levels.
      return ret;
    }
    
    
    
    
    /**generates a statement from given parse result item < statement>.
     * It may be a simple call, an assignment, an if-statement and so on or a < statementBlock>
     * <ul>
     * <li>A comment in /*...* / found in Java-code before the statement are generated also in C
     *     before the statement in an extra line calling {@link get_description(ZbnfParseResultItem)}
     * <li>On < assignment> in ZBNF, {@link gen_assignment(ZbnfParseResultItem, int indent, LocalIdents)} is be called immediate.    
     * <li>On < statementBlock> in ZBNF, {@link gen_statementBlock(ZbnfParseResultItem, int indent, LocalIdents)} 
     *     is be called immediate with a new instance of {@link StatementBlock}.    
     * <li>An < if_statement> in ZBNF consists of a < condition>, a < statement> and optional a < elseStatement>.
     *    The condition will be translated in C with {@link gen_value(ZbnfParseResultItem, LocalIdents, char)}.
     *    As well as the condition occupies more as one line in Java, in C it is written yet in one line.
     *    Typically it is a suitable behavior. But if the condition expression is complex,
     *    it is not so ideal in C. The reason for this behavior: The parser ignore new lines, 
     *    all are white spaces. The C-code is correct, but possibly not ideal readable. It is a good style
     *    for writing comprehensible code separating a complex expressions in smaller parts. 
     *    To divide a conditional expression in smaller parts, some local boolean variable may be used.
     *    It is anyhow effective at machine level.
     *    <br>
     *    If the < statement> is really a simple statement, it will be written after the <code>if(...)</code> in the same line.
     *    But typically it is a statement block, and will be written in extra lines with the correct indentation.
     *    <br>
     *    If a construct <code>else if(...)</code> is given in Java, the <code>if(...)</code> after <code>else</code>
     *    is a simple statement. Therefore it was be produced as <code>else if(...)</code> also in C.
     *    It can be understand as a chain of if. 
     * <li>The behavior of the other control statements are adequate.
     * <li>On < return> statement some additional statements to finish a subroutine are generated,
     *     especially the handling of the <code>StacktraceJc</code>. 
     *     The {@link gen_ActivateGarbageCollection(int)} is called 
     *     because the statement block may contain one or some new-Statements.
     * <li>on < methodCall> in Java2C.zbnf-script, the {@link gen_simpleValue(ZbnfParseResultItem, LocalIdents, char intension)}
     *    is called with intension='m'. The methodcall is translated like a simple value, 
     *    it is syntactically the same. (void-value).
     * <li>On < throwNew> in Java2C.zbnf-script, the {@link gen_throwNew(ZbnfParseResultItem, LocalIdents)} is be called.
     * <li>On < break_statement> in Java2C.zbnf-script, a simple <code>break</code> is generated. 
     *    It are the same relations in Java likewise in C.
     * </ul>    
     * 
     * @param parent The ZBNF parse result item which is a < statement>
     * @param indent Number of nesting level of the block to generate indentations of a line.
     * @param localIdents The indentation of the block: TODO use it as class element.
     * @param typeReturn The return type of the superior method if it contains a return statement.
     * @param intension Intension of call: 'c'-constructor body, 'm'-method body, 'b'-internal block, 'z'-part of if, while etc., 'f'-finalize body. 
     * @return The statement in C stated without indentation and ended without newline, but with the end-semicolon.
     *         If the statement occupies more as one line, the indentation is generated correctly using the indent argument. 
     * @throws ParseException
     */
    public String gen_statement
    ( ZbnfParseResultItem parent
    , int indent
    , LocalIdents localIdents
    , ClassData typeReturn
    , char intension
    )
    throws ParseException
    { String ret = "";
      ClassData[] typeValue = new ClassData[1];  //classData of the part of expression
      //String ret = genIndent(indent);
      Iterator<ZbnfParseResultItem> iterStatement= parent.iterChildren();  //may consist of some parts one after another, not a loop.
      //NOTE: a <statement> has only one child because a statement syntax component is a alternative without loop.
      while(iterStatement.hasNext())
      { ZbnfParseResultItem itemStatement = iterStatement.next();
        lastWasReturn = false;  //default, set to true if it is a return.
        String sDescription = get_description(itemStatement);
        if(sDescription != null)
        { ret += "/*" + sDescription + "*/" + genIndent(indent);
        }
        
        String semanticStatement = itemStatement.getSemantic();
        if(semanticStatement.equals("assignment"))
        { ret += gen_assignment(itemStatement, indent, localIdents, intension);
        }
        else if(semanticStatement.equals("return"))
        { ret += "{ STACKTRC_LEAVE;";
          ret += gen_ActivateGarbageCollection(indent+1, true) + genIndent(indent+1);
          
          ZbnfParseResultItem itemExpr = itemStatement.firstChild();
          if(itemExpr != null && itemExpr.getSemantic().equals("value"))
          { String retValue = gen_value(itemExpr, typeValue, localIdents, 'e');
            if(typeReturn==Java2C_Main.singleton.standardClassData.clazz_bool)
              stop();
            ret += "return " + typeReturn.testAndcast(typeValue[0], retValue, 'r') + ";";
          }
          else
          { ret += "return;";
          }
          ret += genIndent(indent) + "}";
          //to indicate that no additional 'STACKTRC_LEAVE;' should be generate:
          lastWasReturn = true;  
        }
        else if(semanticStatement.equals("statementBlock"))
        { ret += gen_statementBlock(itemStatement, indent, this, typeReturn, 'b');
        }
        else if(semanticStatement.equals("if_statement"))
        { ZbnfParseResultItem itemCondition = itemStatement.getChild("condition");
          String sCondition = gen_value(itemCondition, typeValue, localIdents, 'e');
          ret += "if(" + sCondition + ") ";
          ZbnfParseResultItem ifStatement = itemStatement.getChild("statement");
          String sIfStatement = ifStatement != null ? gen_statement(ifStatement, indent, localIdents, typeReturn, 'z') : ";";
          ret += sIfStatement;
          ZbnfParseResultItem elseStatement = itemStatement.getChild("elseStatement");
          if(elseStatement != null)
          { ret += genIndent(indent) + "else ";
            String sElseStatement = gen_statement(elseStatement, indent, localIdents, typeReturn, 'z');
            ret += sElseStatement;
          }
        }
        else if(semanticStatement.equals("switch_statement"))
        { ZbnfParseResultItem itemSwitchValue = itemStatement.getChild("switchValue");
          String sSwitchValue = gen_value(itemSwitchValue, typeValue, localIdents, 'e');
          ret += "switch(" + sSwitchValue + "){";
          List<ZbnfParseResultItem> listCases = itemStatement.listChildren("case");
          if(listCases != null)for(ZbnfParseResultItem itemCase: listCases)
          { ZbnfParseResultItem zbnfCaseValue = itemCase.getChild("value");
            String sCaseValue = gen_value(zbnfCaseValue, typeValue, localIdents, 'e');  //should be a constant
            ret += genIndent(indent+1) + "case " + sCaseValue + ": ";
            List<ZbnfParseResultItem> zbnfCaseStatements = itemCase.listChildren("statement");
            if(zbnfCaseStatements != null)for(ZbnfParseResultItem zbnfCaseStatement: zbnfCaseStatements)
            { String sCaseStatement = gen_statement(zbnfCaseStatement, indent+1, localIdents, typeReturn, 'z');
              ret += sCaseStatement;
            }
          }
          ZbnfParseResultItem zbnfDefault = itemStatement.getChild("default");
          if(zbnfDefault != null)
          { String sDefaultStatement = gen_statement(zbnfDefault, indent+1, localIdents, typeReturn, 'z');
            ret += "default:" + sDefaultStatement;
          }  
          ret += genIndent(indent) + "}/*switch*/;";
        }
        else if(semanticStatement.equals("while_statement"))
        { ZbnfParseResultItem itemCondition = itemStatement.getChild("condition");
          String sCondition = gen_value(itemCondition, typeValue, localIdents, 'e');
          ret += genIndent(indent) + "while(" + sCondition + ")";
          ZbnfParseResultItem itemWhileStatement = itemStatement.getChild("statement");
          String sWhileStatement = itemWhileStatement != null 
                                 ? gen_statement(itemWhileStatement, indent+1, localIdents, typeReturn, 'z') 
                                 : ";";
          ret += sWhileStatement;
        }
        else if(semanticStatement.equals("dowhile_statement"))
        { ZbnfParseResultItem itemCondition = itemStatement.getChild("condition");
          String sCondition = gen_value(itemCondition, typeValue, localIdents, 'e');
          ZbnfParseResultItem itemWhileStatement = itemStatement.getChild("statement");
          ret += "do ";
          String sWhileStatement = itemWhileStatement != null ? gen_statement(itemWhileStatement, indent+1, localIdents, typeReturn, 'z') : ";";
          ret += sWhileStatement;
          ret += "while(" + sCondition + ");";
                    
        }
        else if(semanticStatement.equals("for_statement")) 
        { ret += gen_for_statement(itemStatement, indent, this, typeReturn);
        }
        else if(semanticStatement.equals("comment"))
        { String sComment = itemStatement.getParsedString();
          ret += "/*" + sComment + "*/" + genIndent(indent);
        }
        else if(semanticStatement.equals("methodCall"))
        { CCodeData methodCall = gen_simpleValue(itemStatement, localIdents, 'm');
          ret += methodCall.cCode + ";";
        }
        else if(semanticStatement.equals("throwNew"))
        { String sMethodCall = gen_throwNew(itemStatement, localIdents);
          ret += sMethodCall + ";";
        }
        else if(semanticStatement.equals("try_statement"))
        { String sTry = gen_try_statement(itemStatement, indent, localIdents);
          ret += sTry;
        }
        else if(semanticStatement.equals("break_statement"))
        { ret += "break;";
        }
        else if(semanticStatement.equals("emtypStatementBlock"))
        { ret += genIndent(indent) + "{ }";
        }
        else if(semanticStatement.equals("emptyStatement"))
        { ret += genIndent(indent) + ";";
        }
        else if(semanticStatement.equals("variable"))
        { //it may be an assignment or such as i++
          CCodeData codeVariable = gen_simpleValue(itemStatement, localIdents, 'm');
          ret += codeVariable.cCode + ";";
        }
        else if(semanticStatement.equals("description"))
        { String sDescription1 = itemStatement.getChild("text").getParsedString();
          ret += "/*" + sDescription1 + "*/" + genIndent(indent);
        }
        else
        { ret += "unknownStatement(); /*unknown statement with semantic: " + semanticStatement + "*/";
          //throw new ParseException("Java2C-ERROR:unknown statement semantic: " + semanticStatement, 0);
        }
      }
      return ret;
    }
  
    
    
    //String gen_initEmbeddedInstance(CCodeData reference, int indent)
    String genInitEmbeddedInstance(ZbnfParseResultItem zbnfNewObject, FieldData fieldInfo, String sCCodeInstance, int indent) 
    throws ParseException
    { String ret = "";
      /*The reference to the embedded instance: */
      String sInstanceRef = null;  //TODO if the instance is a non static inner class, in Java: instance.new(...) 
      CCodeData reference = new CCodeData(sInstanceRef, fieldInfo);
      /*Generate init of the ObjectJc part: */
      if(fieldInfo.modeArrayElement == 'B')
      { /*A fix StringBuffer is to be init: final Java StringBuffer buffer = new StringBuffer(123); */
        String sInit = 
          "init_ObjectJc(&" + sCCodeInstance + ".sb.object, sizeof(StringBufferJc) + " 
        +         fieldInfo.dimensionArrayOrFixSize + " - 4"   //it contains the size of the StringBuffer
        +         ", 0);"
        + genIndent(indent)
        + "ctorO_i_StringBufferJc(&" + sCCodeInstance + ".sb.object, "
        +         fieldInfo.dimensionArrayOrFixSize   //it contains the size of the StringBuffer
        +         ", _thCxt);"
        ;
        ret += sInit;
      }
      else
      { /*All other cases, not a @java2c=fixStringBuffer. : */
        final CCodeData codeCtor;
        String sAccessObject = fieldInfo.typeClazz.getRefObjectJc(); //the .object
        if(sAccessObject == null || !sAccessObject.endsWith("object"))
        { /*the embedded instance doesn't base on Object: */
          String sReference = "build_MemC(&" + sCCodeInstance + ", sizeof(" + sCCodeInstance + "))";
          codeCtor = gen_InternalMethodCall(zbnfNewObject, "ctor", reference, sReference, localIdents);
        }
        else
        { String sReference = sCCodeInstance;
          String sRefObject = "&" + sReference + "." + sAccessObject;
          ret += "init_ObjectJc(" + sRefObject + ", sizeof(" + sReference + "), 0); ";
          ret += genIndent(indent);
          //String sObject = "(ObjectJc*)&(" + leftVariable.cCode + ")";
          codeCtor = gen_InternalMethodCall(zbnfNewObject, "ctorO", reference, sRefObject, localIdents);
        }
        ret += codeCtor.cCode + ";";
      }  
      return ret;
    }
    
    
    
  
    /**generates an assignment-statement from given parse result item < statement>.
     * The < assignment> consist of a < ?leftValue>, an < assignOperator> and a < value>.
     * <br>
     * The < leftValue> is translated via call of 
     * {@link gen_variable(ZbnfParseResultItem, LocalIdents , char intension, LocalIdents.IdentInfos[] retIdentInfo)}
     * with intension='='. The argument retIdentInfo is a call by returned reference. 
     * The returned object contains informations about the kind of the left-value variable, 
     * especially if it is an enhanced reference.
     * <br>
     * The value is translated via call of {@link gen_value(ZbnfParseResultItem, LocalIdents, char intension)}
     * with intension='e'.
     * <br>
     * If the left value is an enhanced reference, a call of <code>clearBackRefJc(variable)</code>
     * is produced before the new reference is set to it, and <code>setBackRefJc(variable)</code>
     * after it is set. This subroutines implement the necessities of Garbage Collection for that enhanced references.
     * 
     * @param parent The ZBNF parse result item which is a < assignment>
     * @param indent Number of nesting level of the block to generate indentations of a line.
     * @param localIdents The indentation of the block: TODO use it as class element.
     * @param intension Intension of call: 'c'-constructor body, 'm'-method body, 'b'-internal block, 'z'-part of if, while etc., 'f'-finalize body. 
     * @return
     * @throws ParseException
     */
    public String gen_assignment(ZbnfParseResultItem parent, int indent, LocalIdents localIdents, char intension) 
    throws ParseException
    { //String ret = genIndent(indent);
      String ret = "";
      ClassData[] typeValue = new ClassData[1];  //classData of the part of expression
      //LocalIdents.IdentInfos typeLeftValue[] = new LocalIdents.IdentInfos[1];
      //String sVariableC;
      final CCodeData leftVariable;
      String sAssignOperatorC;
      { ZbnfParseResultItem zbnfLeftValue = parent.getChild("leftValue");  //##a
        if(zbnfLeftValue == null)
        { throw new ParseException("leftValue expected",0);
        }
        { //test
          String sName = zbnfLeftValue.getChild("variableName").getParsedString();
          if(sName.equals("ij"))
            stop();
        }
        
        leftVariable = gen_variable(zbnfLeftValue, localIdents, '=', classData.thisCodeInfo); //, typeLeftValue); 
      }
      { ZbnfParseResultItem zbnfAssignOperator = parent.getChild("assignOperator");;
        sAssignOperatorC = zbnfAssignOperator.getParsedText();
      }
      ZbnfParseResultItem zbnfValue = parent.getChild("value");
      ZbnfParseResultItem zbnfNewObject = zbnfValue.getChild("newObject");
      if(leftVariable.dimensionArrayOrFixSize == 0 && leftVariable.getTypeName().equals("StringJc"))
      {
        ret = gen_StringAssignment(indent, leftVariable.cCode, leftVariable, sAssignOperatorC, zbnfValue, localIdents, intension);  
      }
      else if(leftVariable.modeAccess == '$' && zbnfNewObject != null)
      { /*variable = new Type(), variable is embedded:
          generate no new, but call of constructor. 
          The Type of new should be the same as the type of the embedded instance.
          It is so, because an embedded instance is only used, if this condition is met. */
        ClassData typeOfNew = getType(zbnfNewObject.getChild("className"), localIdents);  //itemNewObject.getChild("className").getParsedString();
        /*induce to generate the necesarry include statement:
          typeOfNew may be null if the type is a external type. */
        if(typeOfNew != null)
        { writeContent.addIncludeC(typeOfNew.sFileName, "embedded ctor");
        }
        ret += genInitEmbeddedInstance(zbnfNewObject, leftVariable.identInfo, leftVariable.cCode, indent);
      }
      else if(leftVariable.modeAccess == 'Y' && zbnfNewObject != null)
      { //variable = new Type(), variable is embedded:
        //generate no new, but call of constructor.
        ClassData typeOfNew = getType(zbnfNewObject.getChild("className"), localIdents);  //itemNewObject.getChild("className").getParsedString();
        //induce to generate the necesarry include statement:
        //typeOfNew may be null if the type is a external type.
        if(typeOfNew != null)
        { writeContent.addIncludeC(typeOfNew.sFileName, "embedded ctor-Y");
        }
        String sInstanceRef = null;  //TODO if the instance is a non static inner class, in Java: instance.new(...) 
        CCodeData reference = new CCodeData(sInstanceRef, typeOfNew.classTypeInfo);
        //ClassData[] retTypeValue = new ClassData[1];
        String sObject = "(ObjectJc*)&(" + leftVariable.cCode + ")";
        CCodeData codeCtor = gen_InternalMethodCall(zbnfNewObject, "ctorO", reference, sObject, localIdents);
        ret += codeCtor.cCode + ";";
      }
      else 
      {
        String sValueC = gen_value(zbnfValue, typeValue, localIdents, 'e');
        //if(typeLeftValue[0] != null && typeLeftValue[0].sModifier.indexOf('@')>=0)
        if(leftVariable.modeAccess == '@')
        { 
          //it is an enhanced reference:
          //ret += "/*" + sVariableC + " =  ...for an enhanced reference*/" + genIndent(indent);
          ret += "if(" + leftVariable.cCode + ".ref != null) clearBackRefJc(&(" + leftVariable.cCode + ".refbase));" + genIndent(indent);
          ret += leftVariable.cCode + ".ref = " + sValueC + ";";
          if(!sValueC.equals("null"))
          { ret += genIndent(indent) + "setBackRefJc(&(";
            //ret += sVariableC + ".refbase), (ObjectJc*)(" + sVariableC + ".ref));";
            ret += leftVariable.cCode + ".refbase)/*addr of ref*/, " + leftVariable.cCode + ".ref /*The refed Object as void* */);";
          }
          ret += genIndent(indent);
        }
        else
        { //normal assignment, at ex. a numerical value to a int variable
          ret += leftVariable.cCode + sAssignOperatorC + sValueC + ";"; 
        }
      }  
      return ret;
  
    }
  
  
    
    /**
     * @param indent
     * @param sVariableC
     * @param typeLeftValue
     * @param sAssignOperatorC
     * @param sValueC
     * @param typeValue
     * @param intension Intension of call: 'c'-constructor body, 'm'-method body, 'b'-internal block, 'z'-part of if, while etc., 'f'-finalize body. 
     * @return
     * @throws ParseException
     */
    public String gen_StringAssignment
    ( int indent
    , String sVariableC  
    , CCodeData leftValue  //, LocalIdents.IdentInfos typeLeftValue
    , String sAssignOperatorC
    , ZbnfParseResultItem zbnfValue
    , LocalIdents localIdents
    , char intension
    ) throws ParseException
    { String ret = "";
      //if(typeLeftValue.sModifier.indexOf('@')>=0 && intension != 'c')
      if(leftValue.modeAccess == '@' && intension != 'c')
      { ret = "if(" + sVariableC + ".ref != null) clearBackRefJc(&(" + sVariableC + ".refbase));" + genIndent(indent);
      }
      if(sAssignOperatorC.equals("+="))
      { //concatenation required!
        String sValueC  = gen_ConcatenatedStrings
        ( sVariableC    //sFirstString
        //, typeLeftValue.typeClazz  //firstType
        , leftValue.identInfo.typeClazz  //firstType
        , zbnfValue.iterChildren()  //iterZbnf
        , localIdents
        , intension
        );
        ret = sVariableC + " = " + sValueC + ";";
      }
      else
      { ClassData[] retType = new ClassData[1];
        String sValueC = gen_value(zbnfValue, retType, localIdents, intension);
        String sType = retType[0].getClassIdentName();
        if(sValueC.equals("null"))
        { ret += sVariableC + "= null_StringJc;";
        }
        else if(sType.equals("s0") && sAssignOperatorC.equals("="))
        { //A simple String
          ret += sVariableC + "= s0_StringJc(" +sValueC + ");"; 
        }
        else if(sType.equals("s0") && sAssignOperatorC.equals("+="))
        { ret += sVariableC + "= add_s0_StringJc(" + sVariableC + ", " + sValueC + ");"; 
        }
        else if(sType.equals("StringBufferJc") && sAssignOperatorC.equals("="))
        { ret += sVariableC + "= toString_StringBufferJc(" +sValueC + ");"; 
        }
        else if(sType.equals("StringBufferJc") && sAssignOperatorC.equals("+="))
        { ret += sVariableC + "= add_StringJc(" + sVariableC + ", toString_StringBufferJc(" +sValueC + ", _thCxt), _thCxt);"; 
        }
        else if(sType.equals("StringJc") && sAssignOperatorC.equals("="))
        { ret += sVariableC + "= " +sValueC + ";"; 
        }
        else if(sType.equals("StringJc") && sAssignOperatorC.equals("+="))
        { ret += sVariableC + "= add_StringJc(" + sVariableC + ", " + sValueC + ");"; 
        }
        else if(sAssignOperatorC.equals("="))
        { //An Object, use the toString Method.
          ret += sVariableC + "= /*#*/toString_"+ sType + "(" + sValueC + ", _thCxt);"; 
        }
        else if(sAssignOperatorC.equals("+="))
        { //An Object, use the toString Method.
          ret += sVariableC + "= add_StringJc(" + sVariableC + ", toString_"+ sType + "(" + sValueC + ", _thCxt), _thCxt);"; 
        }
        else throw new ParseException("unexpected syntax",0);
      }    
      return ret;
    }
    
  
    /**generates an access to a value in a variable or a using of a variable as left value. 
     * The variable may be referenced. It means, it is a variable in the referenced object.
     * <br>
     * If no reference is given, it may be either it a local variable or a class variable. 
     * With help of the argument localIdents the variable can be found in the context.
     * <ul><li>If it is a class variable, In C <code>ythis-></code> is written before.
     *     <li>If it is a variable of a super class, In C <code>ythis->super.</code> is written before.
     *     <li>If it is a variable of a outer class, In C <code>ythis->outer-></code> is written before.
     *     <li>If it is a local stack variable, the name is directly used in C like in Java.
     * </ul>    
     * <br>
     * If it is a referenced variable, the identifier info of the referencing object is used,
     * The reference is build correctly either with <code>-></code> or <code>.</code>,
     * depended on the reference kind. It may be an embedded struct.
     * The LocalIdents of the referencing object are used to desire it.
     * <br>
     * If the variable is an enhanced reference, the code <code>.ref</code> is generated to get
     * the stored reference as value, if the intension is ones of "Rrmex". 
     * Especially if it is left value, the code of enhanced reference itself is generated.  
     * <br>
     * A variable may have a pre- or post- increment or -decrement like --x or y++.
     * 
     * @param itemVariable The ZBNF parse result item of the < ?variable>-semantic.
     *                     It is a part of a simpleValue-syntax-prescript.
     * @param localIdents The Identifier info of the environment.
     * @param intension Info about the location respectively cause to call this method.
     *                  e-expression R-first reference r-nested reference =:leftvalue ...
     * @param retIdentInfo information about the variable in its context.
     * @return generated String of variable access for the C-code
     * @throws ParseException
     */
    public CCodeData gen_variable(ZbnfParseResultItem itemVariable, LocalIdents localIdents
        , char intension, CCodeData cCodeReferenceInput) //, LocalIdents.IdentInfos[] xxxretIdentInfo) 
    throws ParseException
    { //String ret = "";
      String cCode = "";
      
      final FieldData identInfo; // = retIdentInfo[0];  
      
      String sIdent = itemVariable.getChild("variableName").getParsedString();
      if(sIdent.equals("sFilenameBuffer"))
        stop();
      if(itemVariable.getChild("preDecrement")!= null)
      { cCode +="--"; 
      }
      else if(itemVariable.getChild("preIncrement")!= null)
      { cCode +="++";
      }
      ZbnfParseResultItem itemReference = itemVariable.getChild("reference");
      final CCodeData reference;
      boolean bFinish = false;
      if(itemReference != null)
      { 
        reference = gen_reference(itemReference, localIdents, classData.thisCodeInfo, intension); //, retIdentInfoRef);
        if(reference.dimensionArrayOrFixSize >0 && sIdent.equals("length"))
        { //the length of an array is to be get dependend from the modeAccess: 
          identInfo = Java2C_Main.singleton.standardClassData.clazz_int32.classTypeInfo;
          switch(reference.modeAccess)
          { case '%': cCode += "ARRAYLEN(" + reference.cCode + ")"; break; //simple array
            case '$': cCode += reference.cCode + ".length";         break; //array struct embedded
            case '&': cCode += "ARRAYLEN(" + reference.cCode + ")"; break; //simple array of pointer
            case '*': cCode += reference.cCode + "->length";        break; //array struct simple referenced
            case '@': cCode += reference.cCode + ".ref->length";    break; //array struct enhanced referenced
            default: assert(false);
          }
          bFinish = true;
        }
        else
        { localIdents = reference.getClassLevelIdents(); //retIdentInfoRef[0].typeClazz.classLevelIdents;
          identInfo = localIdents.get(sIdent);  //search it. localIdents are them from a reference,
        }
      }
      else
      { if(sIdent.equals("this"))
        { identInfo = classData.classTypeInfo;
          cCode = "ythis";
          bFinish = true;
        }
        else
        {
          identInfo = localIdents.get(sIdent);  //search it. localIdents are from input,
        }
        
        //if(cCodeReferenceInput == null || cCodeReferenceInput.modeAccess == 'C')
        { if(identInfo == null)
          { reference = null;
          }
          else if("Ssd".indexOf(identInfo.modeStatic)>=0)
          { reference = Java2C_Main.singleton.staticReferenceDummy;
          }
          else if(identInfo.getClassLevel()>0)
          { reference = cCodeReferenceInput;  //it is a ident of the environment.
                           //classData.thisCodeInfo;
          }
          else
          { reference = Java2C_Main.singleton.localReferenceDummy;
          }
        }
        //else
        { //reference = cCodeReferenceInput;
        }
      }
      //Info about the identifier of the variable in local context:
      if(!bFinish)
      { /*bFinish means, the code generation of the base access is done. */
        //the ident Info should be exists, otherwise an exception is thrown here.
        int nClassLevel;
        assert(identInfo != null); // || sIdent.equals("exc"));  //TODO exc in CATCH
        if(identInfo == null)
        { //it is a referenced variable.
          cCode+= sIdent + "/*unknown ident*/";  
          nClassLevel = 0;
          //identInfo = Java2C_Main.singleton.standardClassData.clazz_void.classTypeInfo;
          //throw new ParseException("unknown ident: " + sIdent,0);
        }
        else
        { nClassLevel = identInfo.getClassLevel();
          //char cModifier = identInfo.sModifier.charAt(0);
          if("Ssd".indexOf(identInfo.modeStatic)>=0)
          { //the identifier is static or defined with #define
            cCode += sIdent + "_" + identInfo.clazz.getClassIdentName();
          }
          else
          { /*
            if(nClassLevel > 0 && !bReferenced)
            { //a variable of the own class should be referenced with 'ythis'.
              cCode += "ythis->";
            }
            while(--nClassLevel > 0)
            { cCode += "super.";  //member of a super class. It is an embedded struct named 'super'
            }
            int nOuterLevel = identInfo.getOuterLevel();
            while(--nOuterLevel > 0)
            { cCode += "outer->";  //member of a outer class. It is referenced with 'outer'
            }
            */
            String referenceSeparator;
            switch(reference.modeAccess)
            { case '@': referenceSeparator = ".ref->"; break; //the identifier of this scope is an enhanced reference.
              case '$':  //the identifier of this scope is an embedded struct.
              case 'Y':  //the identifier of this scope is an embedded array element
              case 't':  //the identifier of this scope is a StringJc.
                        referenceSeparator = "."; break;
              case '*': referenceSeparator = "->"; break;
              //case '%': referenceSeparator = ""; assert(reference.cCode.equals("")); break;
              case '%': referenceSeparator = reference.cCode.equals("") ? "" : "."; 
                stop();  //note: OS_TimeStamp as reference was here, time_sec ?
                break;
              case 'C': //class static variable 
                referenceSeparator = ""; assert(reference.cCode.equals("")); 
                break;
              default: throw new ParseException("unexpected modeAccess", reference.modeAccess);
            }
            cCode += reference.cCode + referenceSeparator + sIdent;
            if(identInfo.modeArrayElement == 'B')
            { cCode += ".sb";  //access to the StringBufferJc inside the struct.
            }
            //TODO test wether it is a local overwritten variable against a class element.
          }
        }
      }  
      char cModeAccess = identInfo.modeAccess;
      int dimensionArray = identInfo.dimensionArrayOrFixSize;
      List<ZbnfParseResultItem> listArrayIndices = itemVariable.listChildren("arrayIndex");
      if(listArrayIndices != null) 
      { 
        switch(identInfo.modeAccess)
        { case '%':                       break; //simple array
          case '$': cCode += ".data";     break; //array struct embedded
          case '&':                       break; //simple array of pointer
          case '*': cCode += "->data";    break; //array struct simple referenced
          case '@': cCode += ".ref->data";break; //array struct enhanced referenced
          default: throw new ParseException("unexpected modeAccess", reference.modeAccess);
        }      
        for(ZbnfParseResultItem item: listArrayIndices)
        { ClassData[] typeIndex = new ClassData[1];  //classData of the part of expression
          String sIdxValue = gen_value(item, typeIndex, localIdents, 'e');
          cCode += "[" + sIdxValue + "]";  //fix array
          dimensionArray -=1;
        }
        if(dimensionArray == 0)
        { cModeAccess = identInfo.modeArrayElement; //access to the element.
        }
      }
      if(itemVariable.getChild("postDecrement")!= null)
      { cCode +="--"; 
      }
      else if(itemVariable.getChild("postIncrement")!= null)
      { cCode +="++";
      }
      //The modeAccess is the access to an array element if the variable is so. identInfo is the info of the array variable type.
      final CCodeData retData = new CCodeData(cCode, identInfo, cModeAccess, dimensionArray); //identInfo.modeArray);
      return retData;
    }
  
  
  
    /**generates an reference to a variable or method call from parse result < reference>. 
     * <ul>
     * <li>If <code>this.</code> is found in Java, designed with < this> in zbnf parse result,
     *  <code>ythis-></code> is generated.
     *  
     * <li>If <code>super.</code> is found in Java, designed with < super> in zbnf parse result,
     *  <code>ythis->super.</code> is generated.
     *  
     * <li>If a < referenceAssociation> is found, the access to it is generated in C calling
     *   {@link gen_variable(ZbnfParseResultItem, LocalIdents, char intension, FieldData[])}.
     
     *   It follows either by <code>.</code> or <code>-></code> or the <code></code>
     *   The type of the variable returned in the FieldData[] 
     *   is stored as retIdentInfo and is used to determine if <code>.</code> or <code>-></code>
     *   follows after the variable. The variable may be an embedded reference,
     *   than <code>.</code> should be following. 
     *   The <code>.ref</code> part of an enhanced reference is generated in gen_variable().
     *    
     * <li>If a < referenceMethod> is found, {@link gen_simpleMethodCall(ZbnfParseResultItem, String sInstanceRef, FieldData, ClassData, LocalIdents)}
     *   is called to produce an call of the method in C. The type of return-value of the method
     *   supplied in the FieldData-arg is used as retIdentInfo.
     * </ul>
     * 
     * A reference can be referenced again, writing at ex.<code>myRef.itsRef.</code> or 
     * <code>myRef.method().itsRef.</code>. In Java2C.zbnf the references are understand syntactically 
     * as repetition. Therefore here all references are concatenated. The following separator
     * <code>.</code> or <code>-></code> is determined always from type of the reference before.
     * The type-info of the last reference is returned in retInfo. 
     * 
     * @param zbnfReferences The ZBNF parse result item of the < reference>.
     * @param localIdents The Identifier info of the environment.
     * @param intension Info about the location respectively cause to call this method.
     *                  e-expression R-first reference r-nested reference =:leftvalue ...
     * @param retIdentInfo information about the last reference type.
     * @return String representing the reference in C.
     * @throws ParseException
     */
    CCodeData gen_reference
    ( ZbnfParseResultItem zbnfReferences
    , final LocalIdents localIdentsParent
    , CCodeData envClassCode
    , char intension
    ) 
    throws ParseException
    { LocalIdents refIdents = localIdentsParent;
      int countNestedRef = 0; //to test
      Iterator<ZbnfParseResultItem> iterReferences = zbnfReferences.iteratorChildren();
      intension = 'R';  //the left element of a reference.
      
      CCodeData referenceCode = envClassCode;
      //The initial values are returned if hasNext() fails at begin.
      while(iterReferences.hasNext())
      { if(++countNestedRef >= 2)
          stop();
        ZbnfParseResultItem itemRef = iterReferences.next();
        String semantic = itemRef.getSemantic();
        if(semantic.equals("this"))
        { referenceCode = classData.thisCodeInfo;
        }
        else if(semantic.equals("super"))
        { referenceCode = classData.inheritanceInfo.superClass.classData.thisCodeInfo;
        }
        else if(semantic.equals("referenceMethod"))
        { //the reference is built from a method call.
          referenceCode = gen_simpleMethodCall
                ( itemRef
                , referenceCode //envInstance             //the generated instance reference, it is the reference up to now.
                , localIdentsParent  //the idents for build values for method arguments 
                );
        }
        else if(semantic.equals("referenceAssociation"))
        { String sAssociationName = itemRef.getChild("variableName").getParsedString();
          //sAssociationName may be a type.
          if(sAssociationName.equals("singleton"))
            stop();
          ClassData typeClass = refIdents.getType(sAssociationName);
          if(typeClass != null)
          { //it is a type, not a variable:
            referenceCode = typeClass.typeCodeInfo;
          }
          else
          {
            referenceCode = gen_variable(itemRef, refIdents, intension, referenceCode); //, retIdentInfo); //itemRef.getParsedString();
            //NOTE: retIdentInfo[0] = setted in gen_variable with the type info of the detected variable.
            if(referenceCode.cCode.equals("targets"))
              stop();
          }

        } //if referenceAssociation

        //if(referenceCode.type != null)
        { String sFileName = referenceCode.getTypeHeaderfilename(); //retIdentInfo[0].typeClazz.sFileName;
          if(sFileName != null)
          { writeContent.addIncludeC(sFileName, "reference-ssociation");
          }
          refIdents = referenceCode.getClassLevelIdents();  
        }
        //else
        { //a type without typeClass is not recognize in Java2C, it will be an external type.
          //refIdents = null;  //there don't have to use in a next nested reference, because the type is unknown. 
          //stop();
        }
        intension = 'r'; //for the next nested reference.
      } //while, all references are evaluated one after another.
      return referenceCode;
    }

    
    public String gen_value(ZbnfParseResultItem parent, ClassData[] retType, LocalIdents localIdents, char intension) 
    throws ParseException
    { CCodeData codeValue = gen_value(parent, intension);
      retType[0] = codeValue.identInfo.typeClazz;
      return codeValue.cCode;
    }
  
    /**generates the expression to get a value. From syntax item <...?value>.
     * In the syntax of < value> it is a repetion of <code>{ [|< unaryOperator>] < simpleValue?> ? < binaryOperator> }</code>.
     * Therefore {@link #gen_simpleValue(ZbnfParseResultItem, LocalIdents, char)} is called inside.
     * All components of the < value> are processed in one while-loop because it is stored one after another.
     * That includes also the <code>[< ?conditional>...</code> construct 
     * in which is this method is called recursively.
     * <br>
     * The priority of operators is checked in the Java context. The input to the Java2C-translator
     * should be a well compiled java source code. Therefore it needn't considered here.
     * 
     * @param parent The parse result item which has the semantic <...?value>.
     * @param intension intension of generating: 
     *        <ul><li>'e'-value in an expression, enhanced references are taken with .ref
     *            <li>'l'-left value, 
     *            <li>'a'-argument
     *        </ul>    
     * @return
     * @throws ParseException 
     */
    public CCodeData gen_value(final ZbnfParseResultItem parent, final char intension) 
    throws ParseException
    { String expr = ""; //"/*gen_value=" + intension + "*/";
      Iterator<ZbnfParseResultItem> iter= parent.iterChildren();
      ClassData typeVal = null;
      boolean isBoolean = false;
      char modeAccess ='.', modeStatic = '.', modeArrayElement = '.';
      int dimensionArray = 0;
      String sSemantic;;
      String sLiteral = null;
      CCodeData codeValue = null;
      while(iter.hasNext())
      { ZbnfParseResultItem item = iter.next();
        sSemantic = item.getSemantic();
        /**Because the Java.zbnf-script defines a < value> respectively < simpleValue?> in a flatten way,
         * the < simpleValue> is not an extra result item. All items of a value expression are disposed
         * one after another: an optional < unaryOperator> is always followed by a peculiarity of a 
         * < simpleValue>, after them a < binaryOperator> may be followed. 
         * The < conditional> is the last one in this disposal. 
         * But this correct order of items should not be tested here. It is dedicated by the syntax of Java
         * and the syntax of ZBNF script.
         */
        if(sSemantic.equals("StringLiteralMethod"))
        { /* ZBNF-syntax: simpleValue::=<?> ... <""?simpleStringLiteral>[ \. <simpleMethodCall?StringLiteralMethod> ]
           * Because a simpleValue doesn't create a own ZBNF-component, its components are visible here. 
           * Inside {@link #gen_simpleValue(ZbnfParseResultItem, ClassData[], LocalIdents, char) 
           * the <""?simpleStringLiteral> isn't known. It is the sInstance.
           */
          String sInstance = "s0_StringJc(" + sLiteral + ")";
          CCodeData envInstance = new CCodeData(sInstance, Java2C_Main.singleton.standardClassData.clazzStringJc.classTypeInfo);  
          CCodeData codeExpr =
          gen_simpleMethodCall( item, envInstance, localIdents);     //localIdents used for method arguments. 
          expr += codeExpr.cCode;
          typeVal = codeExpr.identInfo.typeClazz;  //the return type of the method, at ex. "xyz".indexOf(char) returns int.
          modeAccess = codeExpr.modeAccess;
          sLiteral = null;
        }
        else
        { if(sLiteral != null)
          { expr += sLiteral;
            sLiteral = null;
          }
          
          if(sSemantic.equals("unaryOperator"))
          { String operator = item.getParsedText();
            expr += operator;
          }
          else if(sSemantic.equals("binaryOperator"))
          { String operator = item.getParsedText();
            if(  typeVal != null 
              && operator.equals("+")         
              && ( !typeVal.isPrimitiveType()  //+ on not primitive type can only be a String concatenation.
                 || typeVal.getClassIdentName().equals("s0")
                 )
              )
            { expr = gen_ConcatenatedStrings(expr, typeVal, iter, localIdents, intension);
              typeVal = Java2C_Main.singleton.standardClassData.clazzStringJc;
            }
            else
            {
              expr += " " + operator + " ";
            }  
          }
          else if(sSemantic.equals("cmpOperator"))
          { String operator = item.getParsedText();
            if( codeValue != null 
              && (  operator.equals("==")        //NOTE: compare to null
                 || operator.equals("!=")
                 )
              && codeValue.getTypeName().equals("StringJc")
              && codeValue.dimensionArrayOrFixSize == 0
              )
            { //normally a StringJc comes as struct, here the pointer to the chars are compared.
              //StringJc is equal the osal-oriented OS_ValuePtr.
              //expr = "getPtr_OS_ValuePtr((" + expr + "), char const*) " + operator + " ";  
              expr = expr + ".ptr__" + operator + " ";  
            }
            else
            {
              expr += " " + operator + " ";
            }
            typeVal = Java2C_Main.singleton.standardClassData.clazz_bool;
            modeAccess = '%';  //boolean immediate value.
            isBoolean = true;
          }
          else if(sSemantic.equals("booleanOperator"))
          { String operator = item.getParsedText();
            expr += " " + operator + " ";
            typeVal = Java2C_Main.singleton.standardClassData.clazz_bool;
            modeAccess = '%';  //boolean immediate value.      
            isBoolean = true;
          }
          else if(sSemantic.equals("conditional")) // condition ? truevalue : falsevalue
          { //it should be the last one in the disposal of parts of value.
            expr += " ? ";
            ZbnfParseResultItem zbnfTrueValue = item.getChild("trueValue");
            CCodeData trueValue = gen_value(zbnfTrueValue, intension);
            expr += trueValue.cCode;
            ZbnfParseResultItem zbnfFalseValue = item.getChild("falseValue");
            CCodeData falseValue = gen_value(zbnfFalseValue, intension);
            expr += " : ";
            expr += falseValue.cCode;
            //the trueValue and the falseValue should have the same properties.
            typeVal = trueValue.identInfo.typeClazz;
            modeAccess = trueValue.modeAccess;
            modeArrayElement = trueValue.identInfo.modeArrayElement;
            isBoolean = false;   
          }    
          else if(sSemantic.equals("assignment"))
          { expr += " = /*? assignment*/";
          }
          else if(sSemantic.equals("simpleStringLiteral"))
          { sLiteral = "\"" + item.getParsedString() + "\"";
            if(sLiteral.startsWith("\"integral"))
              stop();
            typeVal = Java2C_Main.singleton.standardClassData.clazz_s0;
            modeAccess = 't';
          }  
          else //it is any peculiarity of simpleValue, need not: if(semantic.equals("simpleValue"))
          { codeValue = gen_simpleValue(item, localIdents, intension);
            expr += codeValue.cCode;
            if(!isBoolean)
            { //the right operator wins, TODO test both!
              typeVal = codeValue.identInfo.typeClazz;
              modeAccess = codeValue.modeAccess;
            }
          }
        }//not "StringLiteralMethod"  
      }//while
      //the expression is a repetition of the parts in while loop, 
      //they are concatenated together in expr.
      if(sLiteral != null)
      { expr += sLiteral;  //the first and only was a simpleStringLiteral
      }
      assert(modeAccess != '.');
      assert(typeVal != null);
      switch(modeAccess)
      { case '$':
        { //the result is an embedded struct. because a value should be either an immediate value
          //or a pointer, it should be referenced.
          modeAccess = '*'; //a simple reference
          expr = "&(" + expr + ")";  //build the pointer.
        }break;
        case '@':
        { stop();
          
        }break;
      }  
      FieldData valueInfo = new FieldData
        ("$value", typeVal, modeStatic, modeAccess, dimensionArray, null, modeArrayElement, null); 
      return new CCodeData(expr, valueInfo);
    }
  
  
    /**This routine
     * 
     * @param sFirstString
     * @param firstType
     * @param iterZbnf
     * @return
     * @throws ParseException 
     */
    String gen_ConcatenatedStrings
    ( String sFirstString
    , ClassData firstType
    , Iterator<ZbnfParseResultItem> iterZbnf
    , LocalIdents localIdents, char intension
    ) throws ParseException
    { String sString;
      ClassData type = firstType;
      String sType;
      /**The first part of concatenation is given, test the type and build a new StringBuffer to concatenate.
       * The new StringBuffer should be managed in 'newObjx' like all new Objects, see {@link nrofNew}.
       */
      this.nrofNew +=1;
      //NOTE: the newObj is of type ObjectJc*, cast it directly, it is the same mem location.
      sString = "(StringBufferJc*)(" + gen_newObj() + " = (ObjectJc*)";
      sType = type.getClassIdentName();
      if     (sType.equals("s0"))            {  sString += "s0BXJc";  }
      else if(sType.equals("StringJc"))      {  sString += "sBXJc";  }
      else if(sType.equals("StringBufferJc")){  sString += "sbBXJc";  }
      else                                   {  sString += "xxBXJc/*" + sType + "*/";  }
      sString += "(" + sFirstString + ", _thCxt))";
      while(iterZbnf.hasNext())
      { ZbnfParseResultItem item = iterZbnf.next();
        String expr;
        //ClassData[] retType1 = new ClassData[1];  //classData of the part of expression
        String sSemantic = item.getSemantic();
        
        if(sSemantic.equals("binaryOperator"))
        { //there is only possible a '+'
          //String operator = item.getParsedText();
            
        }
        else
        { if(sSemantic.equals("conditional"))
          { //it should be the last one in the disposal of parts of value.
            expr = " ? ";
            ZbnfParseResultItem zbnfTrueValue = item.getChild("trueValue");
            CCodeData trueValue = gen_value(zbnfTrueValue, intension);
            expr += trueValue.cCode;
            ZbnfParseResultItem zbnfFalseValue = item.getChild("falseValue");
            CCodeData falseValue = gen_value(zbnfFalseValue, intension);
            expr += " : ";
            expr += falseValue.cCode;
            type = falseValue.identInfo.typeClazz;
          }    
          else //it is any peculiarity of simpleValue, need not: if(semantic.equals("simpleValue"))
          { CCodeData methodCode = gen_simpleValue(item, localIdents, intension);
            expr = methodCode.cCode;
            type = methodCode.identInfo.typeClazz;
          }
          //TODO calculate retType
          //type = retType1[0];  //the simple calc
          sType = type == null ? "void" : type.getClassIdentName();
          if     (sType.equals("s0"))            {  sString = "s0XJc(" + sString + ", "+ expr + ", _thCxt)";  }
          else if(sType.equals("StringJc"))      {  sString = "sXJc("  + sString + ", "+ expr + ", _thCxt)";  }
          else if(sType.equals("StringBufferJc")){  sString = "bXJc(" + sString + ", "+ expr + ", _thCxt)";  }
          else if(sType.equals("int16"))         {  sString = "iXJc(" + sString + ", "+ expr + ", _thCxt)";  }
          else if(sType.equals("int32"))         {  sString = "iXJc(" + sString + ", "+ expr + ", _thCxt)";  }
          else if(sType.equals("int64"))         {  sString = "i8XJc(" + sString + ", "+ expr + ", _thCxt)";  }
          else if(sType.equals("float"))         {  sString = "fXJc(" + sString + ", "+ expr + ", _thCxt)";  }
          else if(sType.equals("double"))        {  sString = "dXJc(" + sString + ", "+ expr + ", _thCxt)";  }
          else                                   {  sString = "xXJc/*" + sType + "*/(" + sString + ", " + expr + ", _thCxt)";  }
          
        }  
      }
      
      
      
      return "toString_StringBufferJc(" + sString + ")";
    }
  
  
  
  
    /**generates the code for ZBNF-< simpleValue>. 
     * A simpleValue is a value without operators, a non calculated value, in opposite to a < value>.
     * But a simpleValue may be an expression accessing a referenced value.
     * Examples are: var, this.var, super.var, ref.var, "xyz", method(a.b),
     *               "xyz".indexof(cc), method().val
     * <ul>
     * <li>A simpleValue may be a < variable>, converted with {@link #gen_variable(ZbnfParseResultItem, LocalIdents, char, org.vishia.java2C.FieldData[])},
     *     but a < variable> can be more as that, especially a left value. This is not considered here.
     * <li>A simple value may be a casted value, written in the syntax in Java2C.zbnf with < ?casting> ( < type> ) < value>.
     * <li>There is a special case of casting: (String)null will be converted to null_StringJc, because the simple null fails in C.    
     * <li>A simple value may be such as <code>this</code> or <code>super</code>.
     * <li>A simple value can be a new Object, generated with {@link #gen_newObject(ZbnfParseResultItem, LocalIdents)}. 
     * <li>A < methodCall> is also a variant of simple value. Thats why this method is also used to generate a < methodCall>
     *     in statements. A < methodCall> tests wether it is referenced, than {@link #gen_simpleMethodCall(ZbnfParseResultItem, String, org.vishia.java2C.FieldData, LocalIdents)}
     *     is called.
     * </ul>
     * @param zbnfItem One of the alternatives in <code>simpleValue::=...</code>
     * @param localIdents of the environment
     * @param intension calling intension.
     * @return
     * @throws ParseException
     */
    public CCodeData gen_simpleValue(ZbnfParseResultItem zbnfItem, final LocalIdents localIdents, final char intension)
    throws ParseException
    { CCodeData simpleValue; // = new CCodeData(); // = "";
      ClassData[] retType = new ClassData[1];
      String sSemantic = zbnfItem.getSemantic();
      simpleValue = genConstantValue(zbnfItem);  //simpleStringLiteral, hexValue etc.
      if(simpleValue != null)
      { //done
        retType[0] = simpleValue.identInfo.typeClazz;
      }
      else if(sSemantic.equals("casting"))
      { zbnfItem = zbnfItem.firstChild();    //<typeIdent>
        ClassData typeClazz = getType(zbnfItem, localIdents);
        String castType = typeClazz.getClassIdentName();
        simpleValue = new CCodeData("(" + castType + ")", typeClazz.classTypeInfo);
        zbnfItem = zbnfItem.next();                     //<simpleValue>
        String valueToCast = gen_value(zbnfItem, retType, localIdents, 'e');
        if(valueToCast.equals("null") && castType.equals("StringJc"))
        { //special case (StringJc)null
          simpleValue.cCode = "null_StringJc";
        }
        else
        { //common case: (castType) value
          simpleValue.cCode += valueToCast;
        }  
        retType[0] = typeClazz;  //overwrites the retType from simpleValue().
      }
      else if(sSemantic.equals("parenthesisExpression"))
      { String sValue = gen_value(zbnfItem, retType, localIdents, 'e');
        simpleValue = new CCodeData("(" + sValue + ")", retType[0].classTypeInfo);

      }
      else if(sSemantic.equals("variable"))
      { //FieldData typeLastVariable[] = new FieldData[1];
        simpleValue = gen_variable(zbnfItem, localIdents, intension, classData.thisCodeInfo); //, typeLastVariable);
        if(simpleValue.modeAccess == '@'){
          simpleValue.cCode += ".ref"; }  
        if(  simpleValue.identInfo.fixArraySizes != null 
          && simpleValue.identInfo.modeAccess == '$'
          && simpleValue.dimensionArrayOrFixSize >0
          )
        { //it is defined as an embedded special structure:
          //cast it to its known array pointer type. Otherwise it isn't useable.
          String sArrayPostifx = simpleValue.identInfo.modeArrayElement == '*' ? "_YP" : "_Y";
          simpleValue.cCode = "(" + simpleValue.identInfo.typeClazz.getClassIdentName() + sArrayPostifx + ")" 
                            + "&(" + simpleValue.cCode + ")";
          simpleValue.modeAccess = '*'; //it is now a reference to an array.
        }
        
        retType[0] = simpleValue.identInfo.typeClazz; //typeLastVariable[0].typeClazz;
      }
      else if(sSemantic.equals("newObject"))
      { simpleValue = gen_newObject(zbnfItem, localIdents);
      }
      else if(sSemantic.equals("newArray"))
      { String sValue = gen_newArray(zbnfItem, retType, localIdents, null);
        simpleValue = new CCodeData(sValue, retType[0].classTypeInfo);
      }
      else if(sSemantic.equals("methodCall"))
      { 
        { String sMethodName = zbnfItem.getChild("methodName").getParsedString();
          if(sMethodName.equals("requireWay1Sensor"))
            stop();
        }
        ZbnfParseResultItem itemReference = zbnfItem.getChild("reference");
        CCodeData reference;
        if(itemReference != null)
        { reference = gen_reference(itemReference, localIdents, classData.thisCodeInfo, 'm'); //, typeReference); // retIdentInfo);
        }
        else
        { //no reference before method call, either it is a static method or this-method.
          reference = classData.thisCodeInfo;
        }
        //if it is a static method, the sInstance will be ignored.
        simpleValue = gen_simpleMethodCall
                      ( zbnfItem
                      , reference //sInstanceRef
                      , localIdents     //used for method arguments. 
                      );
      }
      else
      { throw new ParseException("unexpected semantic:" + sSemantic,0);
      }
      return simpleValue;
    }
  
  
  
  
  
    /**generates a simple method call. It includes the evaluation of actual parameters 
     * using {@link #gen_value(ZbnfParseResultItem, ClassData[], LocalIdents, char)}.
     * The method-name is built with the Java-method-name, following by the class name as postfix.
     * In C all methods should have a unique name.
     * <br>
     * If it is a class method, the reference of the class is generated as the first argument.
     * Depended on the kind of the class reference, a <code>&(ref)</code> is generated if it is an embedded reference.
     * @param parent
     * @param sInstanceRef The generated C-Code for the reference to the methods class-instance. 
     *        It is the output from {@link #gen_reference(ZbnfParseResultItem, LocalIdents, char, org.vishia.java2C.FieldData[])}
     *        For static methods this parameter is null.
     *        If it is a constructor call, this  is either the reference to the outer class or null.
     * @param envInstanceInfo The type-info of the class from which the method is member of.
     * @param localIdents The local identifier of this statement block level used for parameter values.
     * @return
     * @throws ParseException
     */
    public CCodeData gen_simpleMethodCall
    ( ZbnfParseResultItem parent
    , CCodeData envInstance
    , LocalIdents localIdents
    )
    throws ParseException
    { String sMethodName = parent.getChild("methodName").getParsedString();
      if(sMethodName.equals("new"))
        stop();
      String sNewObject = null; //no new Object, for param nrofNew
      if(sMethodName.equals("_sizeof"))
      { 
        return new CCodeData("sizeof(" + envInstance.identInfo.getName() + ")", Java2C_Main.singleton.standardClassData.clazz_int.classTypeInfo);
      }
      else return gen_InternalMethodCall(parent, sMethodName, envInstance, sNewObject, localIdents);
    }
  

    
    
    
    /**generates either a new or a simple method call. It includes the evaluation of actual parameters 
     * using {@link #gen_value(ZbnfParseResultItem, ClassData[], LocalIdents, char)}.
     * The method-name is built with the Java-method-name, following by the class name as postfix.
     * In C all methods should have a unique name.
     * <br>
     * If it is a class method, the reference of the class is generated as the first argument.
     * Depended on the kind of the class reference, a <code>&(ref)</code> is generated if it is an embedded reference.
     TODO
     * @param parent
     * @param sMethodName The methodname from Java
     * @param envInstance Type and name of the reference to the instance, from which the method is called. 
     *        It is the output from {@link #gen_reference(ZbnfParseResultItem, LocalIdents, char, org.vishia.java2C.FieldData[])}
     *        For static methods this parameter is null.<br>
     *        If it is a constructor call, this  is either the reference to the outer class: 
     *        The constructor respectively new(...) is a method of the outer class: outer.new(...)
     *        Or null it should be null: The constructor respectively new(...) of not-inner classes is a static method.
     * @param sNewObject If it is a constructor call, the generated C-Code for access the new Object, else null.
     *        it is used as second argument of ctor(...) respectively first argument of ctor(...) if envInstance == null.
     * @param localIdents The local identifier of this statement block level used for parameter values.
     * @return
     * @throws ParseException
     */
    public CCodeData gen_InternalMethodCall
    ( ZbnfParseResultItem parent
    , String sMethodName
    , CCodeData envInstance
    , String sNewObject
    , LocalIdents localIdents
    )
    throws ParseException
    {
      String ret;
      boolean bEmbeddedType = false;
      final FieldData retInfo;
      String sMethodEnvType;
      if(envInstance != null) // it is not a implicit this access
      { //The reference type is defined in an other class, include it!
        boolean bTypeSign = true;
        switch(envInstance.modeAccess)       //envInstanceInfo.sModifier.charAt(0))
        { case '$': bEmbeddedType = true; break;
          default: bTypeSign = false;
        }
        if(bTypeSign)
        { //sMethodEnvType = sMethodEnvType.substring(1);
        }
        sMethodEnvType = envInstance.getTypeName();
      }
      else
      { //a static method, no environment instance is given.
        sMethodEnvType = classData.sClassNameC;
      }
      if(sMethodName.equals("alloc")) //&& sMethodEnvType.equals("Target"))
        stop();
      //List<String> paramsC;
      //List<ClassData> paramsType;
      final List<CCodeData> actParams;
      final FieldData[] actParamsArray;
      ZbnfParseResultItem args = parent.getChild("actualArguments");
      /**The actual arguments are evaluated in value AND in theire type.
       * The types are stored in paramsType. They are relevant for searching the correct method.
       */
      if(args != null)
      { //paramsC = new LinkedList<String>();
        //paramsType = new LinkedList<ClassData>();
        actParams = new LinkedList<CCodeData>();
        Iterator<ZbnfParseResultItem> iterArgs = args.iterChildren();
        while(iterArgs.hasNext())
        { ZbnfParseResultItem arg = iterArgs.next();
          String argSemantic = arg.getSemantic();
          ClassData[] typeValue = new ClassData[1];  //classData of the part of expression
          if(argSemantic.equals("value"))
          { CCodeData actParam = gen_value(arg, 'a');
            actParams.add(actParam);
            //String sValue = gen_value(arg, typeValue, localIdents, 'a');
            //paramsC.add(sValue);
            //paramsType.add(typeValue[0]);
          }
          else if(argSemantic.equals("objectAccess"))
          { actParams.add(new CCodeData("?objectAccess", Java2C_Main.singleton.standardClassData.clazz_void.classTypeInfo));
          }
          else
          { actParams.add(new CCodeData("?unknown", Java2C_Main.singleton.standardClassData.clazz_void.classTypeInfo));
          }
        }
        actParamsArray = new FieldData[actParams.size()];
        int idxActParams = 0;
        for(CCodeData actParam: actParams)
        { actParamsArray[idxActParams++] = actParam.identInfo; //NOTE: not used, searchMethod needs CCodeData, not only FieldData, because an array access should be recognized.
        }
      }
      else
      { //no parameter of the method call.
        //paramsC = null;
        //paramsType = null;
        actParams = null;
        actParamsArray = null;
      }
      FieldData[] formalParams;
      //if(envInstanceInfo.typeClazz != null)
      //if(envInstance.type != null)
      /**In Java the methods are recognized with their parameter types, so in C++ 
       * (overload methods, parameter sensitive method calls). 
       * But in C the method name is unambiguously assigned to one method.
       * Therefore the method name have to be built sensitive to the java parameter.
       * The method will be searched with knowledge of actual parameter types.
       */
      //ClassData.Method method = envInstanceInfo.typeClazz.searchMethod(sMethodName, paramsType);
      ClassData.Method method = envInstance.identInfo.typeClazz.searchMethod(sMethodName, actParams);
      if(method.sCName.equals("indexOf_StringJc")) // && envInstance.cCode.equals(""))
        stop();
      if(method == null)
      { //The are not method known, build the name simple.
        ret = sMethodName + "_" + sMethodEnvType;
        retInfo = Java2C_Main.singleton.standardClassData.clazz_void.classTypeInfo;  //their is no better choice.
        formalParams = null;
      }
      else
      { //Method name for C is found, it is the complete name inclusively type information.
        //at example append_i_StringBufferJc when searched StringBuffer.append(int)
        ret = method.sCName;
        if(ret.equals("alloc_MemC"))
          stop();
        retInfo = method.returnType;
        formalParams = method.paramsType;
      }
      ret += "(";
      String sParamSep;
      String sInstanceRef = envInstance.cCode;
      if( (method.mode & ClassData.Method.modeStatic) == 0   //mode cant't be a static
        &&( sInstanceRef != null && sInstanceRef.length()>0)
        )
      {
        switch(envInstance.modeAccess)
        { case '%': ret += sInstanceRef; break;
          case 't': ret += sInstanceRef; break;
          case '$': ret += "&(" + sInstanceRef + ")"; break;
          case '@': ret += sInstanceRef + ".ref"; break;
          case '*': ret += sInstanceRef; break;
          case 'C': ret += sInstanceRef; break;  //return type of a method
          default: assert(false);
        }
        sParamSep = ", ";
      }  
      else
      { //call of static method because no sInstanceRef or its a static method
        if(( sInstanceRef != null && sInstanceRef.length()>0))
          assert(sInstanceRef.equals("ythis")); //NOTE: ythis-reference is provided always, if no reference is given. 
        sParamSep = "/*static*/";
      }
      
      if(sNewObject != null)
      { //to generate new Object with ctor(alloc...)
        ret += sParamSep + sNewObject;
        sParamSep = ", ";
      }
      
      if(actParams != null)
      { int idxParam = 0;
        /*
        if(paramsType == null || formalParams == null)
        { for(String param: paramsC)
          { ret += sParamSep + param;
            sParamSep = ", ";
          }
        }
        else
        */
        { //Iterator<ClassData> iterActParam = paramsType.iterator();
          StringBuilder sTypeVaArg = null;
          StringBuilder sValueVaArg = null;
          //for(String param: paramsC)
          for(CCodeData actParam: actParams)
          { //ClassData actParam = iterActParam.next();
            if(sTypeVaArg != null)
            { //there are variable arguments.
              sValueVaArg.append(sParamSep).append(actParam.cCode);
              sTypeVaArg.append(actParam.identInfo.getTypeChar());
            }
            else if(formalParams == null)
            { //no method found
              ret += sParamSep + actParam.cCode;
            }
            else
            {
              FieldData formalParam = formalParams[idxParam++];
              if(formalParam == actParam.identInfo)
              { ret += sParamSep + actParam.cCode;
              }
              else if(formalParam == Java2C_Main.singleton.standardClassData.field_va_argRaw)
              { //all following arguments are variable.
                sTypeVaArg = new StringBuilder();
                sValueVaArg = new StringBuilder();
                sValueVaArg.append(sParamSep).append(actParam.cCode);
                sTypeVaArg.append(actParam.identInfo.getTypeChar());
              }
              else
              { String sCast = formalParam.testAndcast(actParam.identInfo.typeClazz, actParam.cCode);
                ret += sParamSep + sCast;
              }
            }  
            sParamSep = ", ";
          }
          if(sTypeVaArg != null)
          { //variable arguments
            ret += sParamSep + "&typedVaArg_VaArgBuffer, \"" + sTypeVaArg + '\"' + sValueVaArg; 
          }
        }  
      }  
      if(method.need_thCxt)
      { ret += sParamSep + "_thCxt)";
      }
      else
      { ret += ")";
      }
      return new CCodeData(ret, retInfo);
      
    }
    
    
    
  
    
    /**generates the expression for a new Type(...) expression.
     * For a new Object, a MemC-instance is necessary. It will be generated in
     * {@link #gen_statementBlock(ZbnfParseResultItem, int, LocalIdents)}.
     * The variable {@link #nrofNew} is used and incremented for that.
     * 
     * @param zbnfNewObject The zbnf parse result item of the < newObject>
     * @param idents The identifier of the environment.
     * @return generated C-code.
     * @throws ParseException
     */
    public CCodeData gen_newObject(ZbnfParseResultItem zbnfNewObject, LocalIdents idents) 
    throws ParseException
    {
      ClassData typeOfNew = getType(zbnfNewObject.getChild("className"), idents);  //itemNewObject.getChild("className").getParsedString();
      //retTypeValue[0] = typeOfNew;
      String sTypeNewObject = typeOfNew.getClassCtype_s();
      if(sTypeNewObject.equals("StringJc"))
      {
        CCodeData reference = new CCodeData(null, typeOfNew.classTypeInfo);//static method call but the method class is given here. 
        return gen_InternalMethodCall(zbnfNewObject, "new", reference, null, idents);
      }
      else
      { //induce to generate the necesarry include statement:
        //typeOfNew may be null if the type is a external type.
        if(typeOfNew != null)
        { writeContent.addIncludeC(typeOfNew.sFileName, "new object");
        }
        //retTypeValue[0] = typeOfNew;
        
        this.nrofNew +=1;
        String sInstanceRef = null;  //TODO if the instance is a non static inner class, in Java: instance.new(...) 
        String sNewObject = "(" + gen_newObj() + " = alloc_ObjectJc(sizeof(" + sTypeNewObject + "), 0, _thCxt))";
        CCodeData reference = new CCodeData(sInstanceRef, typeOfNew.classTypeInfo);
        return gen_InternalMethodCall(zbnfNewObject, "ctorO", reference, sNewObject, idents);
      }
    }
  
  

    /**generates the expression for a new Type[...] expression.
     * For a new Object, a Object-instance is necessary. It will be generated using
     * {@link #gen_newObj()}.
     * The variable {@link #nrofNew} is used and incremented for that.
     * <br>
     * If a variable is given, and it is an embedded instance, no new Object is allocated,
     * but the constructor for the given embedded instance is called.
     * 
     * @param zbnfNewArray The zbnf parse result item of the < newObject>
     * @param idents The identifier of the environment.
     * @param variable The variable to assign to, or null
     * @return generated C-code.
     * @throws ParseException
     */
    public String gen_newArray(ZbnfParseResultItem zbnfNewArray, ClassData[] retTypeValue, LocalIdents idents, FieldData variable) 
    throws ParseException
    { String sRet;
      ClassData typeOfNew = getType(zbnfNewArray.getChild("className"), idents);  //itemNewObject.getChild("className").getParsedString();
      retTypeValue[0] = typeOfNew;
      String sTypeNewObject = typeOfNew.getClassCtype_s();
      String sIdentNameNewObject = typeOfNew.getClassIdentName();
      //induce to generate the necesarry include statement:
      //typeOfNew may be null if the type is a external type.
      if(typeOfNew != null && typeOfNew.sFileName != null)
      { writeContent.addIncludeC(typeOfNew.sFileName, "new array");
      }
      //retTypeValue[0] = typeOfNew;
      List<ZbnfParseResultItem> listValues = zbnfNewArray.listChildren("value");
      int dimension = listValues.size();
      ZbnfParseResultItem zbnfValue = listValues.get(0);
      ClassData[] retType = new ClassData[1];
      String nrofElements = gen_value(zbnfValue, retType, idents, 'e');
      final String sInstance;
      if(variable != null && variable.modeAccess == '$')
      { //use the embedded given variable
        String sName = variable.getName();
        sInstance = "&" + sName + ".object";  //TODO: inherition, than not only .object
      }
      else
      { //create a new Object
        this.nrofNew +=1;
        //String sInstanceRef = null;  //TODO if the instance is a non static inner class, in Java: instance.new(...) 
        sInstance = "(" + gen_newObj() + " = alloc_ObjectJc( sizeof(ObjectArrayJc) + (" + nrofElements + ") * sizeof(" + sTypeNewObject + "), 0, _thCxt))";
      }
      //call the constructor
      String sReflection = (typeOfNew.isPrimitiveType() ? "REFLECTION_" : "&reflection_") + sTypeNewObject;  //reflections of primitive type are defined as simple constants.
      sRet = "(" + sIdentNameNewObject + "_Y)ctorO_ObjectArrayJc(" + sInstance + ", " + nrofElements + ", sizeof(" + sTypeNewObject + ")," + sReflection + ", 0 /*typeInstanceIdent*/);";
      return sRet;
    }
  
  

    /**generates the expression for a < try_Statment> .
     * 
     * @param zbnfThrowNew
     * @param localIdents
     * @return
     * @throws ParseException
     */
    public String gen_try_statement(ZbnfParseResultItem zbnfThrowNew, int indent, LocalIdents localIdents)
    throws ParseException
    { String expr = "";
      ClassData[] typeValue = new ClassData[1];  //classData of the part of expression
      
      ZbnfParseResultItem zbnfStatement = zbnfThrowNew.getChild("statementBlock");
      String statementBlock = 
        gen_statementBlock
        ( zbnfStatement, indent, this
        , Java2C_Main.singleton.standardClassData.clazz_void 
        , 'b'
        );
      expr += "TRY" + statementBlock + "_TRY";
      List<ZbnfParseResultItem> listZbnfCatch = zbnfThrowNew.listChildren("catchBlock");
      if(listZbnfCatch!=null) for(ZbnfParseResultItem zbnfCatch : listZbnfCatch)
      { String sExceptionType = zbnfCatch.getChild("ExceptionType").getParsedString();
        String sExceptionVariable = zbnfCatch.getChild("exceptionVariable").getParsedString();
        ZbnfParseResultItem zbnfExceptionStatement = zbnfCatch.getChild("statementBlock");
        LocalIdents catchIdents = new LocalIdents(localIdents, classData);
        catchIdents.putLocalElement(sExceptionVariable, Java2C_Main.singleton.standardClassData.clazzExceptionJc.classTypeInfo);
        StatementBlock statementBlockCatchFrame = new StatementBlock(catchIdents, true);
        String sExceptionStatement = 
          gen_statementBlock
          ( zbnfExceptionStatement, indent+1, statementBlockCatchFrame
          , Java2C_Main.singleton.standardClassData.clazz_void 
          , 'b'
          );
        expr += genIndent(indent) + "CATCH(" + sExceptionType + ", " + sExceptionVariable + ")"
              + genIndent(indent) + sExceptionStatement;
      }
      expr += genIndent(indent) + "END_TRY";
      return expr;
    }
  
  
  
  
  
    /**generates the expression for a <throwNew> .
     * 
     * @param zbnfThrowNew
     * @param localIdents
     * @return
     * @throws ParseException
     */
    public String gen_throwNew(ZbnfParseResultItem zbnfThrowNew, LocalIdents localIdents)
    throws ParseException
    { String expr = "";
      ClassData[] typeValue = new ClassData[1];  //classData of the part of expression
  
      String sExceptionClass = zbnfThrowNew.getChild("exceptionClass").getParsedString();
      ZbnfParseResultItem itemText = zbnfThrowNew.getChild("text");
      if(itemText != null)
      { //String sText = itemText.getParsedString();
        String sText = gen_value(itemText,typeValue, localIdents,'a');
        String sThrow;
        if(typeValue[0] == Java2C_Main.singleton.standardClassData.clazz_s0)
             { sThrow = "throw_s0Jc(ident_";}
        else { sThrow = "throw_sJc(ident_";}
        expr = sThrow + sExceptionClass + "Jc, " + sText + ", ";
      }
      else
      { expr = "THROW_s(" + sExceptionClass + ", xxx, "  ;
      }
      ZbnfParseResultItem itemValue = zbnfThrowNew.getChild("value2");
      String sValue;
      if(itemValue == null)
      {
        sValue = "0";
      }
      else
      {
        sValue = gen_value(itemValue, typeValue, localIdents, 'a');
      }
      expr += sValue + ", &_thCxt->stacktraceThreadContext, __LINE__)";
      return expr;
    }
  
  
  
  
  
    /**generates the initial assignments to variables. Called only inside a 
     * {@link #gen_statementBlock(ZbnfParseResultItem , int, StatementBlock, ClassData, char)}
     *
     * @param zbnfVariableDefinition Item of &lt; variableDefinition>
     * @throws IOException
     * @throws ParseException
     */
    public String gen_VariableInitAssignment(ZbnfParseResultItem zbnfVariableDefinition, int indent, LocalIdents idents)
    throws ParseException
    {
      String ret;
      ClassData[] typeValue = new ClassData[1];  //classData of the part of expression
  
      { //ZbnfParseResultItem itemAttrib = iterAttrib.next();
        String sName = zbnfVariableDefinition.getChild("name").getParsedString();
        FieldData infoVariable = localIdents.get(sName);
        if(sName.equals("aliases"))
          stop();
        assert(infoVariable != null);
        ZbnfParseResultItem zbnfAssignment = zbnfVariableDefinition.getChild("value");
        if(zbnfAssignment != null)
        { final String value = gen_value(zbnfAssignment, typeValue, idents, 'e');
          final FieldData typeVariable = idents.get(sName);
          final String dstValue = typeVariable.testAndcast(typeValue[0], value);
          //TODO: assignment to StringJc!
          ret = genIndent(indent) + sName + " = " + dstValue + ";";
        }
        else if( (zbnfAssignment = zbnfVariableDefinition.getChild("newObject")) != null)
        { //an assignment with new...
          if(infoVariable.modeAccess == '$')
          { //call the constructor
            String sCtor = genInitEmbeddedInstance(zbnfAssignment, infoVariable, sName, indent);
            ret = genIndent(indent) + sCtor;
          }
          else
          { CCodeData codeNewObject = gen_newObject(zbnfAssignment, idents);
            ret = genIndent(indent) + sName + " = " + codeNewObject.cCode + ";";
          }  
        }
        else if( (zbnfAssignment = zbnfVariableDefinition.getChild("newArray")) != null)
        { //an assignment with new...
          String sNewArray = gen_newArray(zbnfAssignment, typeValue, idents, infoVariable);
          if(infoVariable.modeAccess == '$')
          { ret = genIndent(indent) + sNewArray + ";";
          }
          else
          { ret = genIndent(indent) + sName + " = " + sNewArray + ";";
          }
        }
        else
        { //no asignment TODO use 3. param from gen_variableDefinition: variablesToInit, but not at classlevel
          //ret = genIndent(indent) + sName + " = 0; /*assignment not found*/";
          ret = "/*no initvalue*/";
        }
      }
      return ret;
    }



    CCodeData gen_VariableInitValue(ZbnfParseResultItem zbnfInitAssignment) 
    throws ParseException
    {
      String ret = "";
      final String semanticInitAssignment = zbnfInitAssignment.getSemantic();
      if(semanticInitAssignment.equals("constArray"))
      { String separator = "{ ";
        List<ZbnfParseResultItem> listValues = zbnfInitAssignment.listChildren();  //all are kind of simpleValue
        FieldData identInfoElement = null;
        for(ZbnfParseResultItem zbnfValue: listValues)
        { CCodeData codeValue = genConstantValue(zbnfValue);
          ret += separator + codeValue.cCode;
          separator = ", ";
          identInfoElement = codeValue.identInfo;  //the last wins. But there should be all the same.
        }
        ret += "}";
        return new CCodeData(ret, identInfoElement, '%', 1);
      }
      else if(semanticInitAssignment.equals("value"))
      { CCodeData codeValue = gen_value(zbnfInitAssignment, 'r');
        return codeValue;
      }
      else return null;
        
    }
  
    
    public String gen_newObj()
    { 
      return "newObj" + blockNestingCnt + "_"  + nrofNew;
    }
  
  }//class StatementBlock
  
  
  
  
  

  /**Initializes an instance to run the second pass of a given class.
   * The instance is generated only temporary to run the passes, see {@link GenerateClass}. 
   * @param writeContent Interface to write the content. Note: The content is not written immediately 
   *   in a file, but temporary stored in some StringBuilder. The reason is the order of parts of C-File and H-File.
   * @param allClassData singleton information of all classes there are knwon yet.
   * @param classData The data of this class saves between parsing, first and second pass. 
   *        This classData also contain the ZBNF parse result items.
   * @param runRequiredFirstPass Interface to cause the parsing and running of a first pass
   *        of a up-to-now unknown class. 
   *        This interface is implemented at the main level of the Java2C-translator
   */
  //SecondPass(iWriteContent writeContent, TreeMap<String, ClassData> allClassData, ClassData classData, RunRequiredFirstPass_ifc runRequiredFirstPass)
  SecondPass(iWriteContent writeContent, LocalIdents allClassData, ClassData classData, RunRequiredFirstPass_ifc runRequiredFirstPass)
  { super(writeContent, allClassData, runRequiredFirstPass);
    this.classData = classData;
  }


  /**writes the constructor-, method- and static variable- definition as C-method into the C-File.
   * Inside it is written and called:<ul>
   * <li>all definitions of static variables with there initialization or 0-initiatialization.
   * <li>{@link #write_constructorDefinition(ZbnfParseResultItem, String)}
   * <li>{@link #write_methodDefinition(ZbnfParseResultItem, String)}
   * </ul>
   *  
   * @param itemClass The class start item of zbnf parse result.
   * @param sOuterClassName Name of the environment class not used yet
   * @throws IOException
   * @throws ParseException
   */
  void runSecondPass(Report console, ZbnfParseResultItem itemClass, String sOuterClassName)
  throws IOException, ParseException
  {
    String sClassNameWithout_s = classData.getClassIdentName();
    String sClassName = classData.getClassCtype_s();
    console.writeInfoln("2. pass: " + sClassNameWithout_s);
    
    ZbnfParseResultItem itemDescription = itemClass.getChild("description");
    if(itemDescription != null)
    { write_Description(itemDescription);
    }

    StatementBlock staticQuasiBlock = new StatementBlock(classData.classLevelIdents, true);
    for(ClassData.VariablesToInit staticVariable : classData.staticVariables)
    { String sName = staticVariable.identInfos.getName();
      if(sName.equals("n10"))
        stop();
      FieldData identInfos = classData.classLevelIdents.get(sName);
      String sTypeClassIdent = staticVariable.identInfos.typeClazz.getClassIdentName();
      char kind = staticVariable.identInfos.modeAccess;
      String sStaticDef = "";
      //if(staticVariable.bConst)
      if(identInfos.modeStatic=='S')
      { sStaticDef += "const ";   //final type in java, in headerfile with const static 
      }
      if(kind=='@'){ sStaticDef += staticVariable.identInfos.typeClazz.getClassIdentName() + "REF"; }
      else { sStaticDef += staticVariable.identInfos.typeClazz.getClassCtype_s(); }
      //sStaticDef += sTypeClassIdent + (kind=='@' ? "REF " : " ") + sName + "_" + sClassNameWithout_s;
      sStaticDef += " " + sName + "_" + sClassNameWithout_s;
      for(int dimension1 = 0; dimension1 < staticVariable.identInfos.dimensionArrayOrFixSize; dimension1++)
      { final String sDimension = staticVariable.identInfos.fixArraySizes == null ? null
                                : staticVariable.identInfos.fixArraySizes[dimension1];
        sStaticDef += "[" + ( sDimension == null ? "" : sDimension) + "]"; 
      }
      
      if(staticVariable.itemValue == null)
      { //no init value
        if("@".indexOf(kind)>=0)
        { //embedded struct or enhanced reference, null-initializing
          sStaticDef += " = { 0, null };";
        }
        else if("$".indexOf(kind)>=0)
        { //embedded struct or enhanced reference, null-initializing
          sStaticDef += " = { 0 };";
        }
        else if("*".indexOf(kind)>=0)
        { //normal reference, null-initializing
          sStaticDef += " = null;";
        }
        else
        { sStaticDef += "= 0;";
        }
      }
      else
      { if(sName.equals("n10a"))
          stop();
        CCodeData codeValue = staticQuasiBlock.gen_VariableInitValue(staticVariable.itemValue); 
        
        String value = codeValue.testAndCast(staticVariable.identInfos, 'i');
        //String value = staticVariable.identInfos.typeClazz.testAndcast(codeValue.type, codeValue.cCode, 'i');
        
        if(true || kind == '$')
        { //embedded struct, call initializer
          sStaticDef += " = " + value + ";";
        }
        else if("@*%".indexOf(kind)>=0)
        { //all others
          ClassData[] typeValue = new ClassData[1];
          String sValue = staticQuasiBlock.gen_value(staticVariable.itemValue, typeValue, classData.classLevelIdents, 'e');
          sStaticDef += " = " + sValue + "; /*" + typeValue[0].getClassIdentName() + "*/";
        }
        else if(kind=='t')
        { 
          sStaticDef += " = " + value + ";";
        }
        else
        { throw new ParseException("unknown kind of static variable:" + kind, 0);
        }
      }
      writeContent.writeClassC("\n" + sStaticDef);      
    }
    
    for(ClassData.MethodWithZbnfItem methodDef: classData.listMethodWithZbnf)
    {
      if(methodDef.isConstructor())
      {
        write_constructorDefinition(methodDef, sClassNameWithout_s, classData.nrofConstructor >1);
      }
      else
      { write_methodDefinition(methodDef, sClassNameWithout_s, classData.classLevelIdents);
        
      }
    }
    
    
    /* NOTE: the write_finalizeDefinition() have to be called after all write_methodDefinition(),
     * because the ClassData.zbnfFinalizeMethod is set in write_methodDefinition().
     */
    write_finalizeDefinition(itemClass, sClassNameWithout_s, classData.classLevelIdents);
    
    writeContent.writeClassC("\nstruct Mtbl_" + sClassNameWithout_s);      
    writeContent.writeClassC("\n{ Mtbl_ObjectJc mtblObjectJc;  //methods from ObjectJc;");      
    writeContent.writeClassC(genMethodTableDeclaration(classData));
    writeContent.writeClassC("\n} const mtbl" + sClassNameWithout_s + " =");      
    writeContent.writeClassC("\n{ { clone_Object_F");      
    writeContent.writeClassC("\n  , equals_Object_F");      
    writeContent.writeClassC("\n  , finalize_" + sClassNameWithout_s);      
    writeContent.writeClassC("\n  , hashCode_Object_F");      
    writeContent.writeClassC("\n  , toString_Object_F");      
    writeContent.writeClassC("\n  }");      
    writeContent.writeClassC(genMethodTableContent(classData.inheritanceInfo));
    writeContent.writeClassC("\n};\n");      

    final ClassData superClass = classData.getSuperClassData();
    final String referenceReflectionSuperClass;
    if(superClass != null)
    { final String sNameSuper = superClass.getClassCtype_s();
      final String reflectionSuperClass = superClass == null ? null : "&reflection_";
      final String sFirstMethodSuper;
      if(sNameSuper.equals("ObjectJc")){ sFirstMethodSuper = "clone"; }
      else 
      { sFirstMethodSuper = "sign";
        
      }
      writeContent.writeClassC("\nextern ClassJc const reflection_" + sNameSuper + ";");      
      writeContent.writeClassC("\nstatic struct superClasses_" + sClassName + "_t");      
      writeContent.writeClassC("\n{ ObjectArrayJc head;");      
      writeContent.writeClassC("\n  ClassOffset_idxMtblJc data[1];");      
      writeContent.writeClassC("\n}superclasses_" + sClassName + " =");      
      writeContent.writeClassC("\n{ CONST_ObjectArrayJc(ClassOffset_idxMtblJc, 1, OBJTYPE_ClassOffset_idxMtblJc, null, null)");      
      writeContent.writeClassC("\n, { {&reflection_" + sNameSuper + ", OFFSET_Mtbl(struct Mtbl_" + sClassNameWithout_s + ", mtbl" + superClass.getClassIdentName() + ") }");      
      writeContent.writeClassC("\n  }");      
      writeContent.writeClassC("\n};\n");      
      referenceReflectionSuperClass = "(ClassOffset_idxMtblJcARRAY*)&superclasses_" + sClassName;
    }
    else
    { referenceReflectionSuperClass = "null";
    }

    
    final String sClassNameReflection;
    if(sClassName.length() > 28)
    { int len = sClassName.length();
      sClassNameReflection = sClassName.substring(0, 18) + "_" + sClassName.substring(len-9, len);
    }
    else
    { sClassNameReflection = sClassName;
    }
    writeContent.writeClassC("\nconst ClassJc reflection_" + sClassName + " = ");      
    writeContent.writeClassC("\n{ CONST_ObjectJc(OBJTYPE_ClassJc + sizeof(ClassJc), &reflection_ObjectJc, null) ");      
    writeContent.writeClassC("\n, \"" + sClassNameReflection + "\"");      
    writeContent.writeClassC("\n, 0 //position of ObjectJc");      
    writeContent.writeClassC("\n, sizeof(" + sClassName + ")");      
    writeContent.writeClassC("\n, null //attributes and associations");      
    writeContent.writeClassC("\n, null //method");      
    writeContent.writeClassC("\n, " + referenceReflectionSuperClass + " //superclass");      
    writeContent.writeClassC("\n, null //interfaces");      
    writeContent.writeClassC("\n, 0    //modifiers");      
    writeContent.writeClassC("\n, &mtbl" + sClassNameWithout_s + ".mtblObjectJc");      
    writeContent.writeClassC("\n};\n");      
    //writeContent.writeClassC("\n const ClassJc* clazz" + sClassName + " = &class" + sClassName + ";");      
    
    
  }

  
  
  
  /**returns the definition of the methodtable of the class and all superclasses.
   * @param classDataP
   * @return
   */
  String genMethodTableDeclaration(ClassData classDataP)
  { String ret = "";
    { ClassData superClassData = classDataP.getSuperClassData();
      if(superClassData != null)
      { ret += genMethodTableDeclaration(superClassData);  
      }
    }
    if(classDataP.inheritanceInfo != null && classDataP.inheritanceInfo.methodTable != null)
    { String sClassNameC = classDataP.getClassIdentName();
      ret += "\n  Mtbl_" + sClassNameC + " mtbl" + sClassNameC + ";  //methods from "  + sClassNameC;
    }
    return ret;
  }    
  
  /**returns the definition of the methodtable of the class and all superclasses.
   * @param classDataP
   * @return
   */
  String genMethodTableContent(ClassData.InheritanceInfo inheritanceInfo)
  { String ret = "";
    if(inheritanceInfo != null)
    { ClassData.InheritanceInfo superInheritance = inheritanceInfo.superClass;
      if(superInheritance != null)
      { ret += genMethodTableContent(superInheritance);  
      }
    }
    if(inheritanceInfo != null && inheritanceInfo.methodTable != null)
    { String sClassNameC = inheritanceInfo.classData.getClassIdentName();
      ret += "\n, { &sign_Mtbl_" + sClassNameC + "\n  ";
      for(String methodName: inheritanceInfo.methodTable)
      { ret += ", " + methodName + "\n  ";
      }
      ret += "}";
    }
    return ret;
  }    
  
  
  
  
  
  
  /**writes a constructor definition as C-method into the C-File.
   * The interface-method {@link iWriteContent#writeClassC(String)} is used to do it.
   * The content isn't written directly in the file, because some include-lines 
   * generated while running any methods and constructors of second pass should be written
   * before the text of this method.
   * <br>
   * Inside it is written and called:<ul>
   * <li><code>MemC rawMem</code> as first and <code>ThCxt* _thCxt</code>
   *   as last argument of constructor head.
   * <li>{@link gen_variableDefinition(ZbnfParseResultItem, LocalIdents, List, char intension)}
   *   to generate the arguments of the constructor, in Java2C.zbnf <code>< argumentList></code>
   *   and <code>< argument></code>
   * <li><code>{  StacktraceJc stacktrace...</code>-expressions.  
   * <li>Test of size of rawMem, RuntimeException if it is to less.
   * <li><code>memset(ythis, 0, sizeof(*ythis));</code> of the used mem area of the class (C-struct).
   *   Like in Java all class elements of this instance are set to zero in default.
   * <li>call of super constructors or call of <code>ctorc_ObjectJc(rawMem);</code>
   * <li><code>setReflection_ObjectJc(...)</code> to set reflection and jump table infos for the instance.       
   * <li>call of constructor of all initialized references and especially embedded instances.  
   * <li>{@link StatementBlock#gen_value(ZbnfParseResultItem, ClassData[], LocalIdents, char)}
   *   to generate the initial value assignment of class variable. The list {@link ClassData.variablesToInit}
   *   of the {@link classData} is used to get all variables to initialze. That list elements contain
   *   the reference to the parse result item elements of parsed initial values in Java code.
   * <li>{@link StatementBlock#gen_statementBlock(ZbnfParseResultItem, int, LocalIdents)}
   *   is used to generate the statement block of the constructor.  
   * </ul>
   * While generating the constructor it is possible that a type is used inside, which is unknown yet.
   * Than an include is generated. Because the internal structure of the type should be known,
   * the parsing of the Java Code and the run of its first pass is processed while running the method generation.
   * The second pass of this file is running later. This actions are done using 
   * {@link RunRequiredFirstPass_ifc#runRequestedFirstPass(String)}.
   *  
   * @param itemConstructor Parse result of <code>constructorDefinition::=...</code>
   *        If it is null, than this method is used to create a default constructor with all initializations.
   * @param sClassName Full Name of the class in C-Style where the constructor is member of.
   *        The class name is used as return type and as C-methodidentifier-postfix. 
   * @throws IOException
   * @throws ParseException
   */
  public void write_constructorDefinition
  ( ClassData.MethodWithZbnfItem methodDef
  , String sClassName
  , boolean bArgumentSensitive
  )
  throws IOException, ParseException
  { //LocalIdents localIdents = new LocalIdents(classData.classLevelIdents);

    ZbnfParseResultItem itemConstructor = methodDef.zbnfMethod;
    //StatementBlock statementBlockCtorFrame = new StatementBlock(new LocalIdents(classData.classLevelIdents, null), true);
    StatementBlock statementBlockCtorFrame = new StatementBlock(methodDef.method.getMethodIdents(), true);
        
    writeContent.writeClassC("\n\n/*Constructor */");
    if(itemConstructor == null)
    { writeContent.writeClassC("/**Java2C: autogenerated as default constructor. */");
    }
    //NOTE: is part of gen_methodHead(): else { writeContent.writeClassC("/**" + get_description(itemConstructor) + "*/"); }
      
    //ClassData.Method method = gen_methodHead(itemConstructor, classData, "ctorO", statementBlockCtorFrame.localIdents, classData.classTypeInfo, bArgumentSensitive, 'c'); 
    //String sMethodHead = method.getMethodHeadDefiniton();
    writeContent.writeClassC("\n" + methodDef.method.getMethodHeadDefiniton()); //sMethodHead);

    writeContent.writeClassC("\n{ " + sClassName + "_s* ythis = (" + sClassName + "_s*)yObj;  //upcasting to the real class.");
    writeContent.writeClassC("\n  int sizeObj = getSizeInfo_ObjectJc(yObj);");
    writeContent.writeClassC("\n  STACKTRC_TENTRY(\"ctorO_" + sClassName + "\");");

    //TODO: super(xyz)
    writeContent.writeClassC("\n    checkConsistence_ObjectJc(yObj, sizeof(" + sClassName + "_s), &reflection_" + sClassName + "_s, _thCxt);  ");
    //writeContent.writeClassC("\n  setReflection_ObjectJc(&ythis->object, clazz" + sClassName + ", sizeof(" + sClassName + "));  ");
    
    if(sClassName.equals("ReadTargetFromText"))
      stop();
    for(ClassData.VariablesToInit variableToInit : classData.variablesToInit)
    { String sName = variableToInit.identInfos.getName();
      String sType = variableToInit.identInfos.getTypeName();
      char kind = variableToInit.identInfos.modeAccess;
      if(variableToInit.identInfos.dimensionArrayOrFixSize >0 && variableToInit.identInfos.modeArrayElement != 'B')
      {
        stop();
        if(kind == '$')
        { //embedded array head, call constructor
          writeContent.writeClassC("\n  //Java2C: constructor for embedded array");
          String sCtor ="\n  ctorc_ObjectJc(&ythis->" + sName +".object); "
                       + "ythis->" + sName + ".length = " + variableToInit.identInfos.fixArraySizes[0] + "; "
                       + "ythis->" + sName + ".sizeElement = sizeof(" + variableToInit.identInfos.getTypeName() + ");";
          writeContent.writeClassC(sCtor); 
        }
      }
      else
      { //initialization of non array
        if(kind == '$')
        { //embedded struct, call constructor
          writeContent.writeClassC("\n  { //Java2C: constructor for embedded element");
          String sInstance = "(ythis->" + sName + ")";
          String sInit = statementBlockCtorFrame.genInitEmbeddedInstance(variableToInit.itemValue, variableToInit.identInfos, sInstance, 2);
          
          writeContent.writeClassC("\n    " + sInit);
          writeContent.writeClassC("\n  }");
        }
        else if(kind == 's')
        { //initialize static variable not here.
          stop();
        }
        else if(kind == 'S')
        { //final static ... do not initialize here, but at .c-level.
          stop();
        }
        else if("@*%t".indexOf(kind)>=0)
        { //all others
          if(sType.equals("StringJc"))
          { assert(kind=='t'|| kind=='%');
            CCodeData leftVariable = new CCodeData("ythis->" + sName, Java2C_Main.singleton.standardClassData.clazzStringJc.classTypeInfo);
            //leftVariable.type = Java2C_Main.singleton.standardClassData.clazzStringJc;
            leftVariable.modeAccess = 'C';
            String sAssign = statementBlockCtorFrame.gen_StringAssignment
            ( 1, "ythis->" + sName, leftVariable
            , "=", variableToInit.itemValue, statementBlockCtorFrame.localIdents,'c'
            );
            writeContent.writeClassC("\n  " + sAssign);          
          }
          else
          { ClassData[] typeValue = new ClassData[1];  //classData of the part of expression
            String sValue = statementBlockCtorFrame.gen_value(variableToInit.itemValue, typeValue, classData.classLevelIdents, 'e');
            if(sValue.equals("null"))
            {
              writeContent.writeClassC("\n  ythis->" + sName + " = NULL_"+ sType);
            }
            else
            { writeContent.writeClassC("\n  ythis->" + sName + " = " + sValue + "; /*" + sType + "*/");
            }
          }  
        }
        else
        { throw new ParseException("unknown kind of variable to init:" + kind, 0);
        }
      }  
    }


    if(itemConstructor != null)
    { //StatementBlock genBlockStatment = new StatementBlock();
      String sBody = gen_statementBlock(itemConstructor, 1, statementBlockCtorFrame, classData, 'c');  //it is a statementBlock
      writeContent.writeClassC(sBody);
    }
    writeContent.writeClassC("\n  STACKTRC_LEAVE;");
    writeContent.writeClassC("\n  return ythis;\n}\n\n");
  }

  
  
  
  public void write_finalizeDefinition
  ( ZbnfParseResultItem zbnfClass
  , String sClassName
  , LocalIdents idents
  ) throws ParseException
  { String sClassCtype_s = classData.getClassCtype_s();
    String sClassIdent = classData.getClassIdentName();
    sMethodNameCurrent = "finalize_" + sClassIdent;
    if(sClassIdent.equals("SetValueGenerator"))
      stop();
    writeContent.writeClassC("\n\n" + "void " + sMethodNameCurrent + "(ObjectJc* yObj, ThCxt* _thCxt)");
    writeContent.writeClassC("\n{ " + sClassCtype_s + "* ythis = (" + sClassCtype_s + "*)yObj;  //upcasting to the real class.");
    writeContent.writeClassC("\n   STACKTRC_TENTRY(\"" + sMethodNameCurrent + "\");");
    if(classData.zbnfFinalizeMethod != null)
    { ZbnfParseResultItem itemBody = classData.zbnfFinalizeMethod.getChild("methodbody");
      if(itemBody != null)
      { String sBody = gen_statementBlock(itemBody, 1, null, classData, 'f');  //it is a statementBlock
        writeContent.writeClassC(sBody);
      }
    }
    List<ZbnfParseResultItem> listVariables = zbnfClass.listChildren("variableDefinition");
    if(listVariables != null)for(ZbnfParseResultItem zbnfVariable : listVariables)
    { if(zbnfVariable.getChild("static")== null)
      { //only non static members.
        String sName = zbnfVariable.getChild("name").getParsedString();
        if(sName.equals("msg"))
          stop();
        ZbnfParseResultItem itemType = zbnfVariable.getChild("typeIdent");
        FieldData identInfo = idents.get(sName);
        char cModifier = identInfo.modeAccess;      
        if("@&".indexOf(cModifier) >=0)
        { //it is a reference, generate the test and clearBackRefJc
          writeContent.writeClassC("\n  if(ythis->" + sName + ".ref != null) clearBackRefJc(&(ythis->" + sName + ".refbase));");
        }
      }  
    }
    //if(!genBlockStatment.lastWasReturn)
    { writeContent.writeClassC("\n  STACKTRC_LEAVE;");
    }
    writeContent.writeClassC("\n}\n\n");
    
  }
  
  
  
  
  
  

  /**writes a method definition as C-method into the C-File.
   * The interface-method {@link iWriteContent#writeClassC(String)} is used to do it.
   * The content isn't written directly in the file, because some include-lines 
   * generated while running any methods and constructors of second pass should be written
   * before the text of this method.
   * <br>
   * Inside it is written or called:<ul>
   * <li><code>Type* ythis</code> as first and <code>ThCxt* _thCxt</code>
   *   as last argument of method head.  
   * <li>{@link gen_variableDefinition(ZbnfParseResultItem, LocalIdents, List, char intension)}
   *   to generate the arguments of the constructor, in Java2C.zbnf <code>< argumentList></code>
   *   and <code>< argument></code>
   * <li><code>{  StacktraceJc stacktrace...</code>-expressions.  
   * <li>{@link StatementBlock#gen_statementBlock(ZbnfParseResultItem, int, LocalIdents)}
   *   is used to generate the statement block of the constructor.  
   * </ul>
   * While generating the method it is possible that a type is used inside, which is unknown yet.
   * Than an include-line is generated. Because the internal structure of the type should be known,
   * the parsing of the Java Code and the run of its first pass is processed while running the method generation.
   * The second pass of this file is running later. This actions are done using 
   * {@link RunRequiredFirstPass_ifc#runRequestedFirstPass(String)}.
   *  
   * @param parent Parse result of <code>methodDefinition::=...</code>
   * @param sClassName Full Name of the class in C-Style where the method is member of.
   *        The class name is used as ythis-type and as C-method-identifier-postfix. 
   * @throws IOException
   * @throws ParseException
   */
  public void write_methodDefinition
  ( ClassData.MethodWithZbnfItem methodDef
  , String sClassName
  , LocalIdents parentIdents
  )
  throws IOException, ParseException
  { ZbnfParseResultItem parent = methodDef.zbnfMethod;
    //A statement block as frame for the whole method, especially the formal arguments are localIdents of it:
    //StatementBlock statementBlockMethodFrame = new StatementBlock(new LocalIdents(classData.classLevelIdents,null), true);
    StatementBlock statementBlockMethodFrame = new StatementBlock(methodDef.method.getMethodIdents(), true);
    //LocalIdents localIdents = new LocalIdents(classData.classLevelIdents);
    this.sMethodNameCurrent = methodDef.method.sCName;  //used for STACKTRC_ENTRY.
    ZbnfParseResultItem itemType = parent.getChild("type");
    ClassData typeClazz = getType(itemType, parentIdents);
    String sType = typeClazz.getClassIdentName();
    String sName = parent.getChild("name").getParsedString();
    if(sName.equals("finalize"))  //NOTE: finalize Method see write_finalizeDefinition()
    { classData.zbnfFinalizeMethod = parent;
    }
    else
    { 
      boolean bArgumentSensitive = classData.isAmbiguousnessMethod(sName);
      //ClassData.Method method = gen_methodHead(parent, classData, sName, statementBlockMethodFrame.localIdents, typeClazz.classTypeInfo, bArgumentSensitive, 'c');
      //String sMethodHead = method.getMethodHeadDefiniton();
      writeContent.writeClassC(methodDef.method.getMethodHeadDefiniton()); //sMethodHead);
      writeContent.writeClassC("\n{ ");
      if(classData != methodDef.method.declaringClass)
      { /*the method is overridden, the reference to the instance is given as ithis, cast and test it. */
        String sClassType_s = classData.getClassCtype_s();
        writeContent.writeClassC(sClassType_s + "* ythis = (" + sClassType_s + "*)ithis;\n  ");
      }
      writeContent.writeClassC("STACKTRC_TENTRY(\"" + sMethodNameCurrent + "\");");
      ZbnfParseResultItem itemBody = parent.getChild("methodbody");
      if(itemBody != null)
      { String sBody = gen_statementBlock(itemBody, 1, statementBlockMethodFrame, typeClazz, 'm');  //it is a statementBlock
        writeContent.writeClassC(sBody);
        //if(!genBlockStatment.lastWasReturn)
        { writeContent.writeClassC("\n  STACKTRC_LEAVE;");
        }
      }
      writeContent.writeClassC("\n}\n\n");
    }  
  }


  /**Generates C-code from a parsed statement block.
   * <ul>
   * <li>All < variableDefinition> of the < statement> block were generated using 
   * {@link GenerateClass#gen_variableDefinition(ZbnfParseResultItem, LocalIdents, List, char)}.
   * It should be done first because in C all variable should be define on the begin of a block.
   * The variable identifiers and types are stored in a local copy of {@link LocalIdents}, which are intialized
   * with the content of the localIdentsP given as argument, which comes either from the parent block or from the class.
   * <li>All assignments to variable are generated using {@link #gen_VariableInitAssignment(ZbnfParseResultItem, int, LocalIdents)}. 
   * It is separated from the variable definition and it isn't implemented as initializing of variable,
   * because in Java there are more variants. In C the <code>type name = value;</code> is an initializing of the variable,
   * but <code>type name; ...; name = value;</code> it is an assignment. It is a fine difficult explainable difference.
   * The Java2C produce assignments, not initializing.  
   * <li>All < statement> are generated, using {@link #gen_statement(ZbnfParseResultItem, int, LocalIdents)}.
   *   Therefore a separate String variable is used, because MemC-elements, see next:
   * <li>For all <code>new</code>-statements, the <code>MemC</code> definitions are generated. 
   * They are named with a incremental number. They are placed after the variable definitions of the block
   * but before the statements. This is necessary because the <code>MemC memX</code> is an variable also.
   * <li>The call of <code>activateGarbageCollectorAccess_BlockHeapJc(memX)</code> is generated for all MemC-elements.
   *   The MemC-Elements are countered on level of the statement block, represented by this class.
   *   This is also done in {@link #gen_statement(ZbnfParseResultItem, int, LocalIdents)} before a <code>return</code> statement.
   *   It is not done if the last statement was a <code>return</code>.
   *   Thats why it is done with an extra method {@link #gen_ActivateGarbageCollection(int)}.
   * </ul>
   * @param zbnfStatementBlock The ZBNF parse result item which is a < statementBlock>
   * @param indent number of indentation in the generated C-code. It is the level of blocks.
   * @param localIdentsP The local identifiers of the parent block.
   * @param intension Intension of call: 'c'-constructor body, 'm'-method body, 'b'internal block, 'f'-finalize body. 
   * @return The generated C-code for the block inclusive newlines, indentation and { }.
   * @throws ParseException
   */
  public String gen_statementBlock
  ( ZbnfParseResultItem zbnfStatementBlock
  , int indent
  , StatementBlock parentBlock
  //, LocalIdents localIdentsP
  , ClassData typeReturn
  , char intension
  )
  throws ParseException
  { boolean bFirst = true;
    String ret = genIndent(indent) + "{ ";
    //LocalIdents localIdents = localIdentsP;
  
    StatementBlock statementBlock = parentBlock != null 
                                     ? new StatementBlock(parentBlock)
                                     : new StatementBlock(classData.classLevelIdents, true)
                                     ;
    //all variableDefinitions writing at begin of block, it is necessary in C-language:
    //The initialization of this variable were done at that position, were the variable is defined in Java.
    Iterator<ZbnfParseResultItem> iterZbnfStatementBlock = zbnfStatementBlock.iterChildren("variableDefinition");
    if(iterZbnfStatementBlock != null)
    { //if at least one variableDefinition exists, the localIdents should be local:
      statementBlock.localIdents = new LocalIdents(statementBlock.localIdents, null);  //new based on existing
      while(iterZbnfStatementBlock !=null && iterZbnfStatementBlock.hasNext())
      { ZbnfParseResultItem item = iterZbnfStatementBlock.next();
        //don't init the variables like class variables, instead generate the assignment.
        //Because: The execution order of assignment should be bewared. Don't init at top of statement block.
        //The 3. argument is variablesToInit.
        CCodeData codeVariable = gen_variableDefinition(item, statementBlock.localIdents, null, 'b');
        ret += genIndent(indent+1) + codeVariable.cCode;
      }
    }
  
    /**accumulate the content of block in a variable, because before it the newObj-variables should be written. */
    String content = "";
    /**Statements of the block. They may contain several new(..) operations.
     * Every reference to a new Object is written in a newObj-variable. See nrofNew. 
     */
    //iter = zbnfStatement.iterChildren("statement");
    iterZbnfStatementBlock = zbnfStatementBlock.iterChildren();
    while(iterZbnfStatementBlock !=null && iterZbnfStatementBlock.hasNext())
    { ZbnfParseResultItem zbnfStatement = iterZbnfStatementBlock.next();
      final String semantic = zbnfStatement.getSemantic();
      final boolean isStatement = semantic.equals("statement");
      final boolean isVariableInitAssignemnt = semantic.equals("variableDefinition");
      //Note: beware the order of execution of variable initialization. Don't init at top of statement block,
      if(isStatement || isVariableInitAssignemnt)
      { if(bFirst)
        { //no indentation before first statement. 
          bFirst = false; 
        }
        else
        { //indentation for a next line.
          content += genIndent(indent+1); 
        }
      }
      if(isVariableInitAssignemnt)
      { //all variabledefinitions should have its initial value assignment.
        //the definition itself is translated before, therefore the zbnfStatement is evaluated twice. 
        content += statementBlock.gen_VariableInitAssignment(zbnfStatement, indent+1, statementBlock.localIdents);
      }
      else if(isStatement)        
      { content += statementBlock.gen_statement(zbnfStatement, indent+1, statementBlock.localIdents, typeReturn, intension);
      }
      else
      { //other items may be description etc.
        //ignore it here.
        stop();
      }
    }
    
    /**The statements of the block are generated, containing in 'content'.
     * Now write all variable definitions for newObj:
     */
    if(statementBlock.nrofNew >0)
    { ret += genIndent(indent+1) + "ObjectJc ";
      int ctNrofNew = 0;
      while(++ctNrofNew < statementBlock.nrofNew)
      { ret += "*newObj" + statementBlock.blockNestingCnt + "_" + ctNrofNew + "=null, "; //1..n-1 with ','
      }
      ret += "*newObj" + statementBlock.blockNestingCnt + "_"  + ctNrofNew + "=null;";  //the last without ','
      ret += genIndent(indent+1);
    }
    
    /**Add the content. */
    ret += content; 
    
    /**All references t new Objects should be managed by Garbage Collector. this.nrofNew is used. */
    if(!statementBlock.lastWasReturn)
    { //This call was be done also on a return statement. If it is the last, don't write second, -unreachable code.
      ret += statementBlock.gen_ActivateGarbageCollection(indent+1, false);
    }
    /**The end*/
    ret += genIndent(indent) + "}";
    return ret;
  }


  public String gen_for_statement
  ( final ZbnfParseResultItem itemStatement
  , int indent
  , final StatementBlock parentStatementBlock
  //, final LocalIdents localIdentsP
  , ClassData typeReturn
  ) throws ParseException  
  { String ret = ""; //new StringBuffer();
    //final LocalIdents localIdents;
    ZbnfParseResultItem zbnfIteratorObject = itemStatement.getChild("variableDefinition");
    final String startAssignment;
    final StatementBlock statementBlock;
    if(zbnfIteratorObject != null)
    { statementBlock = new StatementBlock(parentStatementBlock);
      statementBlock.localIdents = new LocalIdents(statementBlock.localIdents, null);  //new based on existing
      List<ClassData.VariablesToInit> variablesToInit1 = new LinkedList<ClassData.VariablesToInit>();
      CCodeData codeVariable = gen_variableDefinition(zbnfIteratorObject, statementBlock.localIdents, variablesToInit1, 'b');
      indent +=1;
      ret += "{ "+ codeVariable.cCode + genIndent(indent); 
      ClassData.VariablesToInit variableToInit = variablesToInit1.get(0);
      { String sName = variableToInit.identInfos.getName();
        String sType = variableToInit.identInfos.getTypeName();
        ClassData[] typeValue = new ClassData[1];  //classData of the part of expression
        final String startValue = statementBlock.gen_value(variableToInit.itemValue, typeValue, statementBlock.localIdents, 'e');
        startAssignment = sName + " = " + startValue + "; "; 
      }          
    }
    else
    { statementBlock = parentStatementBlock;
      ZbnfParseResultItem zbnfStartAssignment = itemStatement.getChild("startAssignment");
      startAssignment = statementBlock.gen_assignment(zbnfStartAssignment, indent+1, statementBlock.localIdents, 'b');
    }
    
    ZbnfParseResultItem zbnfEndCondition = itemStatement.getChild("endCondition");
    ClassData[] retTypeValue = new ClassData[1];
    String sCondition = statementBlock.gen_value(zbnfEndCondition, retTypeValue, statementBlock.localIdents, 'e');
    
    ZbnfParseResultItem zbnfIteratorExpression = itemStatement.getChild("iteratorExpression");
    String sIteratorExpression = statementBlock.gen_value(zbnfIteratorExpression, retTypeValue, statementBlock.localIdents, 'e');
    
    ret += "for(" + startAssignment
        + sCondition + "; " + sIteratorExpression + ")"; // + (genIndent(indent));
    
    ZbnfParseResultItem itemWhileStatement = itemStatement.getChild("statement");
    String sStatement = itemWhileStatement != null 
                      ? statementBlock.gen_statement(itemWhileStatement, indent+1, statementBlock.localIdents, typeReturn, 'z') 
                      : ";";
    ret += sStatement;
    
    if(zbnfIteratorObject != null)
    { ret += genIndent(indent-1) + "}";
    }
    return ret;          
  }






}
